/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "commondefs.h"

#include <cstdlib>

#include <qstring.h>
#include <qstringlist.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qregexp.h>
#include <qimage.h>

#include <kapplication.h>
#include <kstandarddirs.h>
#include <kiconeffect.h>
#include <kdebug.h>

#ifndef Q_WS_WIN
	#include <unistd.h>
#endif


class IconFactory : public QIconFactory
{
	public:

		IconFactory( const QString& name, int group, int size ):
			m_name( name ),
			m_group( group ),
			m_size( size )
		{
			setAutoDelete( true );
		}

		virtual QPixmap* createPixmap( const QIconSet& /*iconSet*/, QIconSet::Size /*size*/, QIconSet::Mode mode, QIconSet::State )
		{
			// ignore passed state (i.e. on/off) and translate QIconSet::Mode to KIcon::State
			static const KIcon::States tbl[] = { KIcon::DefaultState, KIcon::DisabledState, KIcon::ActiveState };
			int state = KIcon::DefaultState;
			if( mode <= QIconSet::Active )
				state = tbl[mode];

			return new QPixmap( GUI::customIcon( m_name, m_group, m_size, state ) );
		}

	private:

		QString m_name;
		int m_group;
		int m_size;
};


QPixmap GUI::customIcon( const QString& name, int group, int size, int state )
{
	static KIconEffect iconEffect;

	QString iconPath( ::locate( "appdata", "pics/" + name + ".png" ) );

	if ( iconPath.isEmpty() )
		return QPixmap( size, size );

	QPixmap pixmap( iconPath );
	QImage image = pixmap.convertToImage();
	if ( image.width() != size || image.height() != size )
		image = image.smoothScale( size, size );
	if ( state != KIcon::DefaultState )
		image = iconEffect.apply( image, group, state );

	pixmap.convertFromImage( image );

	return pixmap;
}

QIconSet GUI::customIconSet( const QString& name, int group, int size )
{
	QIconSet iconSet;
	iconSet.installIconFactory( new IconFactory( name, group, size ) );
	return iconSet;
}

/**
 * String
 */

QString String::title( const QString& text )
{
	QString auxText = text.lower();
	const QString separators( " -_([:,;./\\\t\n\"" );
	int len = strlen( separators ), sepIdx = 0;
	while ( sepIdx < len )
	{
		QStringList auxWordsList;
		QStringList wordsList = QStringList::split( separators[sepIdx], auxText, true );
		for ( QStringList::ConstIterator it = wordsList.begin(), end = wordsList.end(); it != end; ++it )
			if ( ! (*it).isEmpty() )
				auxWordsList.append( (*it)[0].upper()+(*it).mid(1) );
			else
				auxWordsList.append( "" );
		auxText = auxWordsList.join( separators[sepIdx] );
		sepIdx++;
	}
	return auxText;
}

QString String::capitalize( const QString& text )
{
	return text.isEmpty() ? "" : text[0].upper() + text.mid(1).lower();
}

QString String::sentence( const QString& text )
{
	return text.isEmpty() ? "" : text[0].upper() + text.mid(1).lower();
}

int String::rfindFunctionStart( const QString& text )
{
	int pos[5] = {
		text.findRev( "capitalize{" ),
		text.findRev( "titlecase{" ),
		text.findRev( "uppercase{" ),
		text.findRev( "lowercase{" ),
		text.findRev( "ascii{" ),
	};

	return QMAX( QMAX( QMAX( QMAX( pos[0], pos[1] ), pos[2] ), pos[3] ), pos[4] );
}

int String::rfindFunctionEnd( const QString& text, int startPos )
{
	int pos = startPos, len = text.length();
	while ( pos < len )
	{
		// next is safe because idx will always be greater than 0
		if ( text[pos] == '}' && text[pos-1] != '\\' )
			return pos;
		pos++;
	}
	return -1;
}

/**
 * System:
 */

bool System::recursiveMakeDir( const QString& path, QStringList* createdDirsList )
{
	if ( createdDirsList )
		createdDirsList->clear();

	QDir parcialPath( "/" );
	QStringList tokens = QStringList::split( "/", path );
	for ( QStringList::ConstIterator it = tokens.begin(), end = tokens.end(); it != end; ++it )
	{
		parcialPath.setPath( parcialPath.path() + "/" + *it );
		if ( ! QFileInfo( parcialPath.path() ).exists() )
		{
			if ( ! QDir().mkdir( parcialPath.path() ) )
				return false;
			if ( createdDirsList )
				createdDirsList->prepend( parcialPath.path() );
		}
	}

	return true;
}

bool System::copy( const QString& srcPath, const QString& dstPath )
{
	QFile srcFile( srcPath );
	if ( ! srcFile.open( IO_ReadOnly ) )
	{
		kdDebug() << "copy: couldn't open input file " << srcFile.name() << endl;
		return false;
	}

	QFile dstFile( dstPath );
	if ( ! dstFile.open( IO_WriteOnly ) )
	{
		kdDebug() << "copy: couldn't open output file " << dstFile.name() << endl;
		return false;
	}

	char* data = new char[1000000];
	int read = srcFile.readBlock( data, 1000000 );
	int write = 0;
	while ( read != -1 && read != 0 && write != -1 )
	{
		write = dstFile.writeBlock( data, read );
		read = srcFile.readBlock( data, 1000000 );
	}

	delete [] data;

	srcFile.close();
	dstFile.close();

	if ( read == -1 )
	{
		kdDebug() << "copy: error reading data from input file " << srcFile.name() << endl;
		return false;
	}
	else if ( write == -1 )
	{
		kdDebug() << "copy: error writing data to output file " << dstFile.name() << endl;
		return false;
	}
	else
		return true;
}

bool System::move( const QString& srcPath, const QString& dstPath )
{
	if ( ! System::copy( srcPath, dstPath ) )
	{
		kdDebug() << "move: error copying " << srcPath << " to " << dstPath << endl;

		return false;
	}

	if ( ! QFile::remove( srcPath ) )
	{
		kdDebug() << "move: error removing " << srcPath << endl;

		return false;
	}

	return true;
}

bool System::remove( const QString& path )
{
	return QFile::remove( path );
}

bool System::isReadable( const QString& path )
{
	QDir absPath( path );
	absPath.convertToAbs();
	QFileInfo fileInfo( absPath.path() );
	return fileInfo.isFile() && fileInfo.isReadable();
}

bool System::isWritable( const QString& path )
{
	QDir absPath( path );
	absPath.convertToAbs();
	QFileInfo fileInfo( absPath.path() );
	return fileInfo.isFile() && fileInfo.isWritable();
}

QString System::user()
{
#ifdef Q_WS_WIN
	// FIXME Windows code not tested!
	// Taken from http://lists.trolltech.com/qt-interest/2002-12/thread00130-0.html
	#ifdef UNICODE
	if ( qWinVersion() & Qt::WV_NT_based )
	{
		TCHAR winUserName[UNLEN + 1]; // UNLEN is defined in LMCONS.H
		DWORD winUserNameSize = sizeof( winUserName );
		GetUserName( winUserName, &winUserNameSize );
		return qt_winQString( winUserName );
	}
	else
	#endif
	{
		char winUserName[UNLEN + 1]; // UNLEN is defined in LMCONS.H
		DWORD winUserNameSize = sizeof( winUserName );
		GetUserNameA( winUserName, &winUserNameSize );
		return QString::fromLocal8Bit( winUserName );
	}
#else
	char* userName = getlogin();
	if ( userName )
		return QString( userName );
	else
		if ( (userName = getenv( "USER" )) )
			return QString( userName );
		else
			return QString::null;
#endif
}

QString System::homeDir()
{
	return QDir::homeDirPath();
}

QString System::tempDir()
{
	QString tempDir = kapp->dirs()->saveLocation( "tmp" );
	tempDir.replace( QRegExp( "/$" ), "" );
	return tempDir;
}

QString System::newFile( const QString& dirPath, const QString& fileName, const QString& extension )
{
	QFileInfo dirInfo( dirPath );
	if ( ! dirInfo.isDir() || ! dirInfo.isWritable() )
		dirInfo = QFileInfo( tempDir() );
	QFileInfo tempFileInfo( QDir( dirInfo.absFilePath() ), fileName + (extension.isEmpty() ? "" : ("." + extension)) );

	QString fileNameBuilder( fileName + "-%1" + (extension.isEmpty() ? "" : ("." + extension)) );
	for ( int i = 2; tempFileInfo.exists(); ++i )
		tempFileInfo = QFileInfo( QDir( dirInfo.absFilePath() ), fileNameBuilder.arg( i ) );

	return tempFileInfo.absFilePath();
}
