/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "appconfig.h"
#include <commondefs.h>

#include <qdir.h>

#include <ksimpleconfig.h>
#include <kconfigbase.h>
#include <kdebug.h>

using namespace SubtitleComposer;

AppConfig::AppConfig():
	QObject(),
	m_groups()
{
}

AppConfig::AppConfig( const AppConfig& config ):
	QObject(),
	m_groups()
{
	for ( QMapConstIterator<QString,AppConfigGroup*> it = config.m_groups.begin(); it != config.m_groups.end(); ++it )
		setGroup( *it.data() );
}

AppConfig& AppConfig::operator=( const AppConfig& config )
{
	if ( this == &config )
		return *this;

	// remove the now invalid groups (those that are not in config.m_groups)
	QStringList obsoletedKeys; // don't delete the items while iterating the map
	for ( QMapConstIterator<QString,AppConfigGroup*> it = m_groups.begin(), end = m_groups.end(); it != end; ++it )
	{
		if ( ! config.m_groups.contains( it.key() ) )
		{
			delete it.data();
			obsoletedKeys << it.key();
		}
	}

	// safe to delete the items now
	for ( QStringList::ConstIterator it = obsoletedKeys.begin(), end = obsoletedKeys.end(); it != end; ++it )
		m_groups.remove( *it );


	for ( QMapConstIterator<QString,AppConfigGroup*> it = config.m_groups.begin(); it != config.m_groups.end(); ++it )
		setGroup( *it.data() );

	return *this;
}

AppConfig::~AppConfig()
{
	for ( QMapIterator<QString,AppConfigGroup*> it = m_groups.begin(), end = m_groups.end(); it != end; ++it )
		delete it.data();
}

bool AppConfig::isCompatibleWith( const AppConfig& config )
{
	for ( QMapConstIterator<QString,AppConfigGroup*> it = m_groups.begin(), end = m_groups.end(); it != end; ++it )
	{
		if ( ! config.m_groups.contains( it.key() ) )
			return false;
		if ( ! it.data()->isCompatibleWith( *config.m_groups[it.key()] ) )
			return false;
	}

	return true;
}

void AppConfig::loadDefaults()
{
	for ( QMapIterator<QString,AppConfigGroup*> it = m_groups.begin(), end = m_groups.end(); it != end; ++it )
		it.data()->loadDefaults();
}

void AppConfig::readFrom( KConfigBase* config )
{
	for ( QMapIterator<QString,AppConfigGroup*> it = m_groups.begin(), end = m_groups.end(); it != end; ++it )
		it.data()->readFrom( config );
}

void AppConfig::writeTo( KConfigBase* config ) const
{
	for ( QMapConstIterator<QString,AppConfigGroup*> it = m_groups.begin(), end = m_groups.end(); it != end; ++it )
		it.data()->writeTo( config );
}

AppConfigGroup* AppConfig::group( const QString& name )
{
 	return m_groups.contains( name ) ? m_groups[name] : 0;
}

const AppConfigGroup* const AppConfig::group( const QString& name ) const
{
	return m_groups.contains( name ) ? m_groups[name] : 0;
}

void AppConfig::setGroup( const AppConfigGroup& group )
{
	if ( m_groups.contains( group.name() ) )
	{
		if ( m_groups[group.name()]->isCompatibleWith( group ) )
		{
			*m_groups[group.name()] = group;
			return;
		}
		else
		{
			kdWarning() << "attempted to replace group " + group.name() + " with incompatible settings" << endl;
			return;
		}
	}

	m_groups[group.name()] = new AppConfigGroup( group );
	m_groups[group.name()]->m_config = this;

	connect( m_groups[group.name()], SIGNAL(optionChanged(const QString&,const QString&,const QString&)),
			 this, SIGNAL(optionChanged(const QString&,const QString&,const QString&)) );
}

void AppConfig::delGroup( const QString& name )
{
	if ( m_groups.contains( name ) )
	{
		disconnect( m_groups[name], SIGNAL(optionChanged(const QString&,const QString&,const QString&)),
					this, SIGNAL(optionChanged(const QString&,const QString&,const QString&)) );

		delete m_groups[name];
		m_groups.remove( name );
	}
}
