/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actionmanager.h"

using namespace SubtitleComposer;

ActionManager::ActionManager():
	m_undoStack(),
	m_redoStack(),
	m_compressionThreshold( 350 )
{
	m_time.start();

	m_undoStack.setAutoDelete( true );
	m_redoStack.setAutoDelete( true );

	connect( this, SIGNAL( actionStored() ), this, SIGNAL( stateChanged() ) );
	connect( this, SIGNAL( actionRemoved() ), this, SIGNAL( stateChanged() ) );
	connect( this, SIGNAL( actionRedone() ), this, SIGNAL( stateChanged() ) );
	connect( this, SIGNAL( actionUndone() ), this, SIGNAL( stateChanged() ) );
	connect( this, SIGNAL( historyCleared() ), this, SIGNAL( stateChanged() ) );
}

ActionManager::~ActionManager()
{
	disconnect();
}

void ActionManager::execAndStore( Action* action )
{
	if ( action != 0 )
	{
		action->redo();

		bool compressed = false;
		if ( m_time.restart() < m_compressionThreshold && m_redoStack.isEmpty() )
		{
			Action* previousAction = m_undoStack.isEmpty() ? 0 : m_undoStack.first();
			if ( previousAction && action->mergeWithPrevious( previousAction ) )
			{
				delete m_undoStack.take( 0 );
				compressed = true;
			}
		}

		m_redoStack.clear();

		m_undoStack.prepend( action );

		if ( ! compressed )
			emit actionStored();

		emit actionRedone();
	}
}

void ActionManager::popUndo()
{
	Action* action = m_undoStack.take( 0 );
	if ( action == 0 )
		return;

	delete action;
	m_redoStack.clear();

	emit actionRemoved();
}

bool ActionManager::hasRedo() const
{
	return ! m_redoStack.isEmpty();
}

bool ActionManager::hasUndo() const
{
	return ! m_undoStack.isEmpty();
}

QString ActionManager::redoDescription() const
{
	return m_redoStack.isEmpty() ? QString::null : m_redoStack.first()->description();
}

QString ActionManager::undoDescription() const
{
	return m_undoStack.isEmpty() ? QString::null : m_undoStack.first()->description();
}

void ActionManager::redo()
{
	Action* action = m_redoStack.take( 0 );
	if ( action == 0 )
		return;

	action->redo();
	m_undoStack.prepend( action );

	emit actionRedone();
}

void ActionManager::undo()
{
	Action* action = m_undoStack.take( 0 );
	if ( action == 0 )
		return;

	action->undo();
	m_redoStack.prepend( action );

	emit actionUndone();
}

void ActionManager::clearHistory()
{
	m_undoStack.clear();

	m_redoStack.clear();

	emit historyCleared();
}
