#ifndef COMPOSITEACTION_H
#define COMPOSITE_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "action.h"

#include <qstring.h>
#include <qptrlist.h>

namespace SubtitleComposer
{
	class CompositeAction : public Action
	{
		public:

			CompositeAction( const QString& description, bool immediateExecution=true, bool delaySignals=true ):
				Action( description ),
				m_actions(),
				m_immediateExecution( immediateExecution ),
				m_delaySignals( delaySignals )
			{
				m_actions.setAutoDelete( true );
			}

			virtual ~CompositeAction()
			{
			}

			unsigned count()
			{
				return m_actions.count();
			}

			void appendAction( Action* action )
			{
				if ( m_immediateExecution )
					action->_redo( ! m_delaySignals );

				m_actions.append( action );
			}

		protected:

			void compressActions()
			{
				QPtrList<Action> obsoleted;
				obsoleted.setAutoDelete( false );

				Action* prevAction;
				for ( QPtrListIterator<Action> it( m_actions ); it.current(); ++it )
				{
					prevAction = it.current();
					for ( ++it; it.current(); ++it )
					{
						if ( it.current()->mergeWithPrevious( prevAction ) )
						{
							obsoleted.append( prevAction );
							prevAction = it.current();
						}
						else
							break;
					}
				}

				for ( QPtrListIterator<Action> it( obsoleted ); it.current(); ++it )
					m_actions.removeRef( it.current() );
			}

			virtual void _redo()
			{
				if ( executed() || ! m_immediateExecution )
					for ( QPtrList<Action>::ConstIterator it = m_actions.begin(), end = m_actions.end(); it != end; ++it )
						(*it)->_redo( ! m_delaySignals );

				if ( ! executed() )
					compressActions();
			}

			virtual void _undo()
			{
				if ( m_actions.isEmpty() )
					return;

				QPtrListIterator<Action> it( m_actions );
				for ( it.toLast(); it.current(); --it )
					(*it)->_undo( ! m_delaySignals );
			}

			virtual void _emitRedoSignals()
			{
				if ( m_delaySignals )
					for ( QPtrListIterator<Action> it( m_actions ); it.current(); ++it )
						(*it)->_emitRedoSignals();
			}

			virtual void _emitUndoSignals()
			{
				if ( m_actions.isEmpty() || ! m_delaySignals )
					return;

				QPtrListIterator<Action> it( m_actions );
				for ( it.toLast(); it.current(); --it )
					(*it)->_emitUndoSignals();
			}

		protected:

			QPtrList<Action> m_actions;
			bool m_immediateExecution;
			bool m_delaySignals;
	};

}

#endif
