#ifndef RANGES_H
#define RANGES_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <qvaluevector.h>

#include <kdebug.h>

namespace SubtitleComposer
{
	class Range
	{
		public:

			static const unsigned MaxIndex=(unsigned)-1;

			Range( unsigned index ):
				m_start( index ),
				m_end( index )
			{
				m_start = index;
				m_end = index;
			}

			Range( unsigned startIndex, unsigned endIndex ):
				m_start( startIndex ),
				m_end( endIndex )
			{
				if ( m_start > m_end )
				{
					kdWarning() << "Attempted to create invalid range: " << startIndex << ";" << endIndex << endl;
					m_start = m_end;
				}
			}

			Range( const Range& range ):
				m_start( range.m_start ),
				m_end( range.m_end )
			{
			}

			Range& operator=( const Range& range )
			{
				if ( this == &range )
					return *this;

				m_start = range.m_start;
				m_end = range.m_end;

				return *this;
			}

			inline static Range full()
			{
				return Range( 0, MaxIndex );
			}

			inline static Range lower( unsigned index )
			{
				return Range( 0, index );
			}

			inline static Range upper( unsigned index )
			{
				return Range( index, MaxIndex );
			}

			inline unsigned start() const { return m_start; }
			inline unsigned end() const { return m_end; }

			inline bool contains( unsigned index ) const
			{
				return index >= m_start && index <= m_end;
			}

			inline bool contains( const Range& range ) const
			{
				return m_end <= range.m_end && m_start >= range.m_start;
			}

			inline bool operator==( const Range& range ) const
			{
				return m_start == range.m_start && m_end == range.m_end;
			}

			inline bool operator!=( const Range& range ) const
			{
				return m_start != range.m_start || m_end != range.m_end;
			}

			inline bool operator<( const Range& range ) const
			{
				return m_start < range.m_end;
			}

			inline bool operator<=( const Range& range ) const
			{
				return m_start <= range.m_end;
			}

			inline bool operator>( const Range& range ) const
			{
				return m_end > range.m_start;
			}

			inline bool operator>=( const Range& range ) const
			{
				return m_end >= range.m_start;
			}

		private:

			friend class RangeList;
			friend class QValueVector<Range>;
			friend class QValueVectorPrivate<Range>;

			Range() {}

			unsigned m_start;
			unsigned m_end;
	};

	class RangeList
	{
		public:

			typedef QValueVector<Range>::Iterator Iterator;
			typedef QValueVector<Range>::ConstIterator ConstIterator;

			RangeList()
			{
			}

			RangeList( const Range& range )
			{
				m_ranges.append( range );
			}

			RangeList( const RangeList& ranges ):
				m_ranges( ranges.m_ranges )
			{
			}

			RangeList complement() const
			{
				if ( m_ranges.empty() )
					return Range( 0, Range::MaxIndex );

				RangeList ret;

				QValueVector<Range>::ConstIterator it = m_ranges.begin();

				if ( (*it).m_start > 0 )
					ret << Range( 0, (*it).m_start - 1 );
	
				unsigned lastEnd = (*it).m_end;
				++it;
				for ( QValueVector<Range>::ConstIterator end = m_ranges.end(); it != end; ++it )
				{
					ret << Range( lastEnd + 1, (*it).m_start - 1 );
					lastEnd = (*it).m_end;
				}

				if ( lastEnd < Range::MaxIndex )
					ret << Range( lastEnd + 1, Range::MaxIndex );

				return ret;
			}

			bool contains( unsigned index ) const
			{
				for ( QValueVector<Range>::ConstIterator it = m_ranges.begin(), end2 = m_ranges.end(); it != end2; ++it )
					if ( (*it).contains( index ) )
						return true;

// 				for ( ConstIterator it = begin(), end2 = end(); it != end2; ++it )
// 					if ( (*it).contains( index ) )
// 						return true;

				return false;
			}

			bool empty() const
			{
				return m_ranges.empty();
			}

			unsigned count() const
			{
				return m_ranges.count();
			}

			unsigned indexesCount() const
			{
				unsigned count = m_ranges.count();

				for ( QValueVector<Range>::ConstIterator it = m_ranges.begin(), end2 = m_ranges.end(); it != end2; ++it )
					count += (*it).m_end - (*it).m_start;

				return count;
			}

			Range first() const
			{
				if ( m_ranges.empty() )
				{
					kdWarning() << "Requesting first range on empty range list" << endl;
					return Range( 0, 0 );
				}

				return m_ranges.first();
			}

			Range last() const
			{
				if ( m_ranges.empty() )
				{
					kdWarning() << "Requesting first range on empty range list" << endl;
					return Range( Range::MaxIndex, Range::MaxIndex );
				}

				return m_ranges.last();
			}

			unsigned firstIndex() const
			{
				if ( m_ranges.empty() )
				{
					kdWarning() << "Requesting first index on empty range list" << endl;
					return 0;
				}

				return m_ranges.first().start();
			}

			unsigned lastIndex() const
			{
				if ( m_ranges.empty() )
				{
					kdWarning() << "Requesting first index on empty range list" << endl;
					return Range::MaxIndex;
				}

				return m_ranges.last().end();
			}

			void clear()
			{
				m_ranges.clear();
			}

			void trimToIndex( unsigned index )
			{
				trimToRange( Range::lower( index ) );
			}

			void trimToRange( const Range& range )
			{
				if ( m_ranges.empty() )
					return;

				// GENERAL ALGORITHM FOLLOWS
				// - we search the lower and the upper items affected.
				//   - update lower item start and end values
				//   - remove all items below lower up and above upper (lower and upper excluded)

				//       0         1         2
				//     XXXXX     XXXXX     XXXXX           [L, U]
				//  [    ]                                 [0, 0]
				//  []                                     [0, 4]
				//  [         ]                            [0, 0]
				//  [            ]                         [0, 1]
				//      [               ]                  [0, 1]
				//           []                            [1, 3]
				//           [     ]                       [1, 1]
				//                [     ]                  [1, 1]
				//  [                              ]       [0, 2]
				//                            [    ]       [2, 2]
				//                                 [  ]    [3, 2]
				//  [                         ]            [0, 2]

				const unsigned SIZE = m_ranges.count();
				const unsigned LAST_INDEX = SIZE - 1;

				unsigned lowerIndex = SIZE;
				unsigned upperIndex = LAST_INDEX;

				for ( unsigned index = 0; index <= LAST_INDEX; ++index )
				{
					Range& currentRange = m_ranges[index];

					if ( lowerIndex == SIZE )
					{
						if ( range.m_start <= currentRange.m_end )
						{
							lowerIndex = index;
							if ( range.m_end < currentRange.m_start )
							{
								// special case, we must remove all items
								upperIndex = LAST_INDEX + 1;
								break;
							}
						}
					}
					else
					{
						if ( range.m_end < currentRange.m_start )
						{
							upperIndex = index - 1;
							break;
						}
					}
				}

				if ( lowerIndex > LAST_INDEX || upperIndex > LAST_INDEX )
				{
					m_ranges.clear();
				}
				else
				{
					Range& lowerRange = m_ranges[lowerIndex];
					Range& upperRange = m_ranges[upperIndex];

					if ( range.m_start > lowerRange.m_start )
						lowerRange.m_start = range.m_start;
					if ( range.m_end < upperRange.m_end )
						upperRange.m_end = range.m_end;

					// remove all items after upperRange
					for ( unsigned index = upperIndex + 1; index < SIZE; ++index )
						m_ranges.erase( m_ranges.begin() + (upperIndex + 1) );

					// remove all items before lowerRange
					for ( unsigned index = 0; index < lowerIndex; ++index )
						m_ranges.erase( m_ranges.begin() );
				}
			}


			void operator<<( const Range& range )
			{
				// first search resolve the most common (and easy) cases
				if ( m_ranges.empty() )
				{
					m_ranges.append( range );
					return;
				}
				else
				{
					Range& lastRange = m_ranges.last();
					if ( lastRange.m_end < range.m_start ) // range is higher than lastRange
					{
						if ( lastRange.m_end + 1 == range.m_start ) // range is absorbed by lastRange
							lastRange.m_end = range.m_end;
						else
							m_ranges.append( range );
						return;
					}
					else
					{
						Range& firstRange = m_ranges.first();
						if ( firstRange.m_start > range.m_end ) // range is lower than firstRange
						{
							if ( range.m_end + 1 == firstRange.m_start ) // range is absorbed by firstRange
								firstRange.m_start = range.m_start;
							else
								m_ranges.append( range );
							return;
						}
					}
				}

				// GENERAL ALGORITHM FOLLOWS
				// - we search the lower and the upper items affected.
				//   - update lower item start and end values
				//   - remove all items following lower up to upper (lower excluded)

				//       0         1         2
				//     XXXXX     XXXXX     XXXXX           [L, U]
				//  [    ]                                 [0, 0]
				//  [         ]                            [0, 0]
				//  [            ]                         [0, 1]
				//  [                   ]                  [0, 1]
				//          [ ]                            [0, 0]
				//          [      ]                       [0, 1]
				//  [                              ]       [0, 2]
				//                            [    ]       [2, 2]
				//                                  [ ]    [2, 2] this case isn't correctly handled below but it's catched before
				//           []                            [1, 3]


				const unsigned LAST_INDEX = m_ranges.count() - 1;

				unsigned lowerIndex = LAST_INDEX;
				unsigned upperIndex = LAST_INDEX;

				for ( unsigned index = 0; index < LAST_INDEX; ++index )
				{
					Range& currentRange = m_ranges[index];

					if ( lowerIndex == LAST_INDEX )
					{
						if ( range.m_start <= currentRange.m_end + 1 )
						{
							lowerIndex = index;
							if ( range.m_end < currentRange.m_start - 1 )
							{
								// special case, we must insert a new Range at lowerIndex
								upperIndex = LAST_INDEX + 1;
								break;
							}
						}
					}
					else
					{
						if ( range.m_end < currentRange.m_start - 1 )
						{
							upperIndex = index - 1;
							break;
						}
					}
				}

				if ( upperIndex > LAST_INDEX )
				{
					// insert new range at lowerIndex
					m_ranges.insert( m_ranges.begin() + lowerIndex, range );
				}
				else
				{
					Range& lowerRange = m_ranges[lowerIndex];
					Range& upperRange = m_ranges[upperIndex];

					if ( range.m_start < lowerRange.m_start )
						lowerRange.m_start = range.m_start;
					if ( range.m_end > upperRange.m_end )
						lowerRange.m_end = range.m_end;
					else
						lowerRange.m_end = upperRange.m_end;

					for ( unsigned index = lowerIndex + 1; index <= upperIndex; ++index )
						m_ranges.erase( m_ranges.begin() + (lowerIndex + 1) );
				}
			}

			inline ConstIterator begin() const { return m_ranges.begin(); }
			inline ConstIterator end() const { return m_ranges.end(); }


		private:

			QValueVector<Range> m_ranges;
	};
}

#endif
