/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "subtitle.h"
#include "subtitleline.h"
#include "subtitleiterator.h"
#include "subtitleactions.h"
#include "compositeaction.h"

#include <klocale.h>

using namespace SubtitleComposer;

double Subtitle::s_defaultFramesPerSecond( 23.976 );

double Subtitle::defaultFramesPerSecond()
{
	return s_defaultFramesPerSecond;
}

void Subtitle::setDefaultFramesPerSecond( double framesPerSecond )
{
	s_defaultFramesPerSecond = framesPerSecond;
}

Subtitle::Subtitle( double framesPerSecond ):
	m_primaryState( 0 ),
	m_primaryCleanState( 0 ),
	m_secondaryState( 0 ),
	m_secondaryCleanState( 0 ),
	m_compositeAction( 0 ),
	m_compositeActionDepth( 0 ),
	m_framesPerSecond( framesPerSecond ),
	m_formatData( 0 )
{
	m_lines.setAutoDelete( true );
}

Subtitle::~Subtitle()
{
	delete m_compositeAction;

	delete m_formatData;
}

bool Subtitle::isPrimaryDirty() const
{
	return m_primaryState != m_primaryCleanState;
}

void Subtitle::clearPrimaryDirty()
{
	if ( m_primaryCleanState != m_primaryState )
	{
		m_primaryCleanState = m_primaryState;
		emit primaryDirtyStateChanged( false );
	}
}

bool Subtitle::isSecondaryDirty() const
{
	return m_secondaryState != m_secondaryCleanState;
}

void Subtitle::clearSecondaryDirty()
{
	if ( m_secondaryCleanState != m_secondaryState )
	{
		m_secondaryCleanState = m_secondaryState;
		emit secondaryDirtyStateChanged( false );
	}
}

FormatData* const Subtitle::formatData() const
{
	return m_formatData;
}

void Subtitle::setFormatData( const FormatData* formatData )
{
	delete m_formatData;

	m_formatData = formatData ? new FormatData( *formatData ) : 0;
}

ActionManager& Subtitle::actionManager()
{
	return m_actionManager;
}

const ActionManager& Subtitle::actionManager() const
{
	return m_actionManager;
}

double Subtitle::framesPerSecond() const
{
	return m_framesPerSecond;
}

void Subtitle::setFramesPerSecond( double framesPerSecond )
{
	if ( m_framesPerSecond != framesPerSecond )
		processAction( new SetFramesPerSecondAction( *this, framesPerSecond ) );
}

void Subtitle::changeFramesPerSecond( double toFramesPerSecond, double fromFramesPerSecond )
{
	if ( toFramesPerSecond <= 0 )
		return;

	if ( fromFramesPerSecond <= 0 )
		fromFramesPerSecond = m_framesPerSecond;

	double scaleFactor = fromFramesPerSecond/toFramesPerSecond;

	if ( scaleFactor == 1.0 )
		return;

	beginCompositeAction( i18n( "Change Frame Rate" ) );

	setFramesPerSecond( toFramesPerSecond );

	for ( SubtitleIterator it( *this, Range::full() ); it.current(); ++it )
	{
		Time showTime = it.current()->showTime();
		showTime.adjust( 0, scaleFactor );

		Time hideTime = it.current()->hideTime();
		hideTime.adjust( 0, scaleFactor );

		it.current()->setTimes( showTime, hideTime );
	}

	endCompositeAction();
}


unsigned Subtitle::linesCount() const
{
	return m_lines.count();
}

SubtitleLine* Subtitle::line( unsigned index )
{
	return index < (unsigned)m_lines.count() ? m_lines.at( index ) : 0;
}

const SubtitleLine* Subtitle::line( unsigned index ) const
{
	return index < (unsigned)m_lines.count() ? m_lines.at( index ) : 0;
}

SubtitleLine* Subtitle::firstLine()
{
	return m_lines.isEmpty() ? 0 : m_lines.first();
}

const SubtitleLine* Subtitle::firstLine() const
{
	return m_lines.isEmpty() ? 0 : m_lines.first();
}

SubtitleLine* Subtitle::lastLine()
{
	return m_lines.isEmpty() ? 0 : m_lines.last();
}

const SubtitleLine* Subtitle::lastLine() const
{
	return m_lines.isEmpty() ? 0 : m_lines.last();
}

SubtitleLine* Subtitle::prevLine( const SubtitleLine* line )
{
	int index = m_lines.findRef( line );
	return index > 0 ? m_lines.at( index - 1 ) : 0;
}

SubtitleLine* Subtitle::nextLine( const SubtitleLine* line )
{
	int index = m_lines.findRef( line );
	return index > 0 && (index+1) < (signed)m_lines.count() ? m_lines.at( index + 1 ) : 0;
}

unsigned Subtitle::lineNumber( const SubtitleLine* line ) const
{
	int index = m_lines.findRef( line );
	return index < 0 ? 0 : (unsigned)(index + 1);
}

int Subtitle::lineIndex( const SubtitleLine* line ) const
{
	return m_lines.findRef( line );
}

void Subtitle::insertLine( SubtitleLine* line, unsigned index )
{
	QPtrList<SubtitleLine> lines;
	lines.append( line );
	insertLines( lines, index );
}

void Subtitle::insertLines( const QPtrList<SubtitleLine>& lines, unsigned index )
{
	if ( index > (unsigned)m_lines.count() )
		index = m_lines.count();

	processAction( new InsertLinesAction( *this, index, lines ) );
}

SubtitleLine* Subtitle::insertNewLine( unsigned index, bool timeAfter, SubtitleLine::OpMode mode )
{
	if ( index > (unsigned)m_lines.count() )
		index = m_lines.count();

	SubtitleLine* newLine = new SubtitleLine();
	unsigned newLineIndex = (mode == SubtitleLine::Secondary) ? m_lines.count() : index;

	if ( timeAfter )
	{
		if ( newLineIndex ) // there is a previous line
		{
			SubtitleLine* prevLine = m_lines.at( newLineIndex - 1 );
			newLine->setTimes( prevLine->hideTime() + 100, prevLine->hideTime() + 1000 );
		}
		else if ( newLineIndex < (unsigned)m_lines.count() ) // there is a next line
		{
			SubtitleLine* nextLine = m_lines.at( newLineIndex );
			newLine->setTimes( nextLine->showTime() - 1100,  nextLine->showTime() - 100 );
		}
		else
			newLine->setHideTime( 1000 );
	}
	else // ! timeAfter
	{
		if ( newLineIndex < (unsigned)m_lines.count() ) // there is a next line
		{
			SubtitleLine* nextLine = m_lines.at( newLineIndex );
			newLine->setTimes( nextLine->showTime() - 1100,  nextLine->showTime() - 100 );
		}
		else if ( newLineIndex ) // there is a previous line
		{
			SubtitleLine* prevLine = m_lines.at( newLineIndex - 1 );
			newLine->setTimes( prevLine->hideTime() + 100, prevLine->hideTime() + 1000 );
		}
		else
			newLine->setHideTime( 1000 );
	}

	if ( mode == SubtitleLine::Both || index == (unsigned)m_lines.count() )
	{
		insertLine( newLine, newLineIndex );
	}
	else if ( mode == SubtitleLine::Primary )
	{
		beginCompositeAction( i18n( "Insert Line" ) );

		insertLine( newLine, newLineIndex );

		SubtitleLine* line = newLine;
		SubtitleIterator it( *this, Range::full(), false );
		for ( it.toIndex( newLineIndex + 1 ); it.current(); ++it )
		{
			line->setSecondaryText( it.current()->secondaryText() );
			line = it.current();
		}
		line->setSecondaryText( QString::null );

		endCompositeAction();
	}
	else if ( mode == SubtitleLine::Secondary )
	{
		beginCompositeAction( i18n( "Insert Line" ) );

		insertLine( newLine, newLineIndex );

		SubtitleIterator it( *this, Range::full(), true );
		SubtitleLine* line = it.current();
		for ( --it; it.index() >= (signed)index; --it )
		{
			line->setSecondaryText( it.current()->secondaryText() );
			line = it.current();
		}
		line->setSecondaryText( QString::null );

		newLine = line;

		endCompositeAction();
	}

	return newLine;
}

void Subtitle::removeLines( const RangeList& ranges, SubtitleLine::OpMode mode )
{
	if ( m_lines.isEmpty() || ranges.empty() )
		return;

	if ( mode == SubtitleLine::Both )
	{
		beginCompositeAction( i18n( "Remove Lines" ) );

		RangeList::ConstIterator rangesIt = ranges.end(), begin = ranges.begin();
		do
		{
			rangesIt--;
			processAction( new RemoveLinesAction( *this, (*rangesIt).start(), (*rangesIt).end() ) );
		}
		while ( rangesIt != begin );

		endCompositeAction();
	}
	else if ( mode == SubtitleLine::Secondary )
	{
		beginCompositeAction( i18n( "Remove Lines" ) );

		RangeList rangesComplement = ranges.complement();
		rangesComplement.trimToRange( Range( ranges.firstIndex(), m_lines.count() - 1 ) );

		// we have to move the secondary texts up (we do it in chunks)
		SubtitleIterator srcIt( *this, rangesComplement );
		SubtitleIterator dstIt( *this, Range::upper( ranges.firstIndex() ) );
		for ( ; srcIt.current() && dstIt.current(); ++srcIt, ++dstIt )
			dstIt.current()->setSecondaryText( srcIt.current()->secondaryText() );

		// the remaining lines secondary text must be cleared
		for ( ; dstIt.current(); ++dstIt )
			dstIt.current()->setSecondaryText( QString::null );

		endCompositeAction();
	}
	else // if mode == SubtitleLine::Primary
	{
		beginCompositeAction( i18n( "Remove Lines" ), true, false );

		RangeList mutableRanges( ranges );
		mutableRanges.trimToIndex( m_lines.count() - 1 );

		// first, we need to append as many empty lines as we're to remove
		// we insert them with a greater time than the one of the last (non deleted) line

		Range lastRange = mutableRanges.last();
		unsigned lastIndex = lastRange.end() == m_lines.count() - 1 ? lastRange.start() - 1 : m_lines.count() - 1;
		SubtitleLine* lastLine = lastIndex < m_lines.count() ? m_lines.at( lastIndex ) : 0;
		Time showTime( lastLine ? lastLine->hideTime() + 100 : 0 );
		Time hideTime( showTime + 1000 );

		QPtrList<SubtitleLine> lines;
		const unsigned size = ranges.indexesCount();
		for ( unsigned index = 0; index < size; ++index )
		{
			lines.append( new SubtitleLine( QString::null, QString::null, showTime, hideTime ) );
			showTime.shift( 1100 );
			hideTime.shift( 1100 );
		}

		processAction( new InsertLinesAction( *this, m_lines.count(), lines ) );

		// then, we move the secondary texts down (we need to iterate from bottom to top for that)
		RangeList rangesComplement = mutableRanges.complement();

		SubtitleIterator srcIt( *this, Range( ranges.firstIndex(), m_lines.count() - lines.count() - 1 ), true );
		SubtitleIterator dstIt( *this, rangesComplement, true );
		for ( ; srcIt.current() && dstIt.current(); --srcIt, --dstIt )
			dstIt.current()->setSecondaryText( srcIt.current()->secondaryText() );

		// finally, we can remove the specified lines
		RangeList::ConstIterator rangesIt = ranges.end(), begin = ranges.begin();
		do
		{
			rangesIt--;
			processAction( new RemoveLinesAction( *this, (*rangesIt).start(), (*rangesIt).end() ) );
		}
		while ( rangesIt != begin );

		endCompositeAction();
	}
}

void Subtitle::swapTexts( const RangeList& ranges )
{
	processAction( new SwapLinesTextsAction( *this, ranges ) );
}

void Subtitle::splitLines( const RangeList& ranges )
{
	beginCompositeAction( i18n( "Split Lines" ) );

	for ( SubtitleIterator it( *this, ranges, true ); it.current(); --it )
	{
		SubtitleLine* line = it.current();
		if ( line->primaryLines() > 1 )
		{
			line->simplifySpaces();

			long autoDurationsSum = 0;
			QValueList<int> autoDurations;
			QStringList primaryLines = QStringList::split( '\n', line->primaryText() );
			for ( QStringList::ConstIterator ptIt = primaryLines.begin(), ptEnd = primaryLines.end(); ptIt != ptEnd; ++ptIt )
			{
				const Time& autoDuration = SubtitleLine::autoDuration( *ptIt, 60, 50, 50 );
				autoDurations.append( autoDuration.toMillis() );
				autoDurationsSum += autoDuration.toMillis();
			}

			double factor = (double)line->durationTime().toMillis() / autoDurationsSum;

			QStringList secondaryLines = QStringList::split( '\n', line->secondaryText(), true );
			while ( secondaryLines.count() < primaryLines.count() )
				secondaryLines.append( QString::null );
			while ( secondaryLines.count() > primaryLines.count() )
			{
				QString lastLine = secondaryLines.last();
				secondaryLines.pop_back();
				secondaryLines.last() += "\n";
				secondaryLines.last() += lastLine;
			}

			unsigned subLineIndex = it.index();
			unsigned splitLineIndex = 0;
			for ( QStringList::ConstIterator ptIt = primaryLines.begin(), ptEnd = secondaryLines.end(), stIt = secondaryLines.begin(), stEnd = secondaryLines.end(); ptIt != ptEnd && stIt != stEnd; ++ptIt, ++stIt, ++subLineIndex, ++splitLineIndex )
			{
				if ( splitLineIndex )
				{
					SubtitleLine* newLine = new SubtitleLine();
					newLine->setStyleFlags( line->styleFlags() );
					newLine->setShowTime( line->hideTime() + 1 );
					insertLine( newLine, subLineIndex );
					line = newLine;
				}

				line->setTexts( *ptIt, *stIt );
				line->setDurationTime( Time( (long)(factor*autoDurations[splitLineIndex]) - 1 ) );
			}
		}
	}

	endCompositeAction();
}

void Subtitle::joinLines( const RangeList& ranges )
{
	beginCompositeAction( i18n( "Join Lines" ) );

	RangeList obsoletedRanges;

	for ( RangeList::ConstIterator rangesIt = ranges.begin(), end = ranges.end(); rangesIt != end; ++rangesIt )
	{
		unsigned rangeStart = (*rangesIt).start();
		unsigned rangeEnd = SubtitleAction::getLastIndex( *this, (*rangesIt).end() );

		if ( rangeStart == rangeEnd )
			continue;

		SubtitleLine* firstLine = m_lines.at( rangeStart );
		SubtitleLine* lastLine = m_lines.at( rangeEnd );

		QString primaryText, secondaryText;

		for ( SubtitleIterator it( *this, Range( rangeStart, rangeEnd-1 ) ); it.current(); ++it )
		{
			if ( ! it.current()->primaryText().isEmpty() )
			{
				primaryText.append( it.current()->primaryText() );
				primaryText.append( "\n" );
			}

			if ( ! it.current()->secondaryText().isEmpty() )
			{
				secondaryText.append( it.current()->secondaryText() );
				secondaryText.append( "\n" );
			}
		}

		primaryText.append( lastLine->primaryText() );
		secondaryText.append( lastLine->secondaryText() );

		firstLine->setTexts( primaryText, secondaryText );
		firstLine->setHideTime( lastLine->hideTime() );

		obsoletedRanges << Range( rangeStart + 1, rangeEnd );
	}

	removeLines( obsoletedRanges );

	endCompositeAction();
}


void Subtitle::shiftLines( const RangeList& ranges, long msecs )
{
	if ( msecs == 0 )
		return;

	beginCompositeAction( i18n( "Shift Lines" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->shiftTimes( msecs );

	endCompositeAction();
}

void Subtitle::adjustLines( const Range& range, long newFirstTime, long newLastTime )
{
	if ( m_lines.isEmpty() || newFirstTime >= newLastTime )
		return;

	unsigned firstIndex = range.start();
	unsigned lastIndex = SubtitleAction::getLastIndex( *this, range.end() );

	if ( firstIndex >= lastIndex )
		return;

	long oldFirstTime = m_lines.at( firstIndex )->showTime().toMillis();
	long oldLastTime = m_lines.at( lastIndex )->showTime().toMillis();
	long oldDeltaTime = oldLastTime - oldFirstTime;

	long newDeltaTime = newLastTime - newFirstTime;

	// special case in which we can't procede as there's no way to
	// linearly transform the same time into two different ones...
	if ( ! oldDeltaTime && newDeltaTime )
		return;

	long shiftMseconds;
	double scaleFactor;

	if ( oldDeltaTime )
	{
		shiftMseconds = (long)(newFirstTime-((double)newDeltaTime/oldDeltaTime)*oldFirstTime);
		scaleFactor = (double)newDeltaTime/oldDeltaTime;
	}
	else // oldDeltaTime == 0 && newDeltaTime == 0
	{
		// in this particular case we can make the adjust transformation act as a plain shift
		shiftMseconds = newFirstTime - oldFirstTime;
		scaleFactor = 1.0;
	}

	if ( shiftMseconds == 0 && scaleFactor == 1.0 )
		return;

	beginCompositeAction( i18n( "Adjust Lines" ) );

	for ( SubtitleIterator it( *this, range ); it.current(); ++it )
		it.current()->adjustTimes( shiftMseconds, scaleFactor );

	endCompositeAction();
}

void Subtitle::sortLines( const Range& range )
{
	beginCompositeAction( i18n( "Sort Lines" ), true, false );

	SubtitleIterator it( *this, range );
	SubtitleLine* line = it.current();
	++it;
	int itPositions = 1;
	for ( SubtitleLine* nextLine = it.current(); nextLine; ++it, line = nextLine, nextLine = it.current(), ++itPositions )
	{
		if ( nextLine->showTime() < line->showTime() ) // nextLine is not at the right position, we'll move it backwards
		{
			int fromIndex = it.index();		// nextLine current index

			// We set the first target index candidate. Note that we can't tell is this is the
			// correct target index as nextLine my be misplaced by more than one position.
			// We do now, however, that the new index must be one before fromIndex.
			int toIndex = (--it).index();

			// we iterate backwards finding the correct position knowing that all previous lines ARE sorted
			for ( ; it.current(); --it )
			{
				if ( it.current()->showTime() < nextLine->showTime() )
				{
					toIndex = (++it).index();
					break;
				}
			}

// 			kdDebug() << "moving from " << fromIndex << " to " << toIndex << endl;
			processAction( new MoveLineAction( *this, fromIndex, toIndex ) );

			// moving the lines invalidates the iterator so we recreate it and advance it to where it was
			it = SubtitleIterator( *this, range );
			it += itPositions;
			nextLine = it.current();
		}
	}

	endCompositeAction();
}


void Subtitle::applyDurationLimits( const RangeList& ranges, const Time& minDuration, const Time& maxDuration, bool canOverlap )
{
	if ( m_lines.isEmpty() || minDuration > maxDuration )
		return;

	beginCompositeAction( i18n( "Apply Duration Limits" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(), end = ranges.end(); rangesIt != end; ++rangesIt )
	{
		Time lineDuration;
		SubtitleIterator it( *this, *rangesIt );
		SubtitleLine* line = it.current();
		++it;
		SubtitleLine* nextLine = it.current();

		for ( ; line; ++it, line = nextLine, nextLine = it.current() )
		{
			lineDuration = line->durationTime();

			if ( lineDuration > maxDuration )
				line->setDurationTime( maxDuration );
			else if ( lineDuration < minDuration )
			{
				if ( ! nextLine ) // the last line doesn't have risk of overlapping
					line->setDurationTime( minDuration );
				else
				{
					if ( canOverlap || line->showTime() + minDuration < nextLine->showTime() )
						line->setDurationTime( minDuration );
					else // setting the duration to minDuration will cause an unwanted overlap
					{
						if ( line->hideTime() < nextLine->showTime() ) // make duration as big as possible without overlap
							line->setHideTime( nextLine->showTime() - 1 );
						// else line is already at the maximun duration without overlap (or overlapping) so we don't change it
					}
				}
			}
		}
	}

	endCompositeAction();
}



void Subtitle::setMaximunDurations( const RangeList& ranges )
{
	if ( m_lines.isEmpty() )
		return;

	beginCompositeAction( i18n( "Maximize Durations" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(), end = ranges.end(); rangesIt != end; ++rangesIt )
	{
		SubtitleIterator it( *this, *rangesIt );
		SubtitleLine* line = it.current();
		++it;
		SubtitleLine* nextLine = it.current();

		for ( ; line && nextLine; ++it, line = nextLine, nextLine = it.current() )
		{
			if ( line->hideTime() < nextLine->showTime() )
				line->setHideTime( nextLine->showTime() - 1 );
		}
	}

	endCompositeAction();
}


void Subtitle::setAutoDurations( const RangeList& ranges, unsigned charMsecs, unsigned wordMsecs, unsigned lineMsecs, bool canOverlap, SubtitleLine::OpMode mode )
{
	if ( m_lines.isEmpty() )
		return;

	beginCompositeAction( i18n( "Set Automatic Durations" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(), end = ranges.end(); rangesIt != end; ++rangesIt )
	{
		Time autoDuration;

		SubtitleIterator it( *this, *rangesIt );
		SubtitleLine* line = it.current();
		++it;
		SubtitleLine* nextLine = it.current();

		for ( ; line; ++it, line = nextLine, nextLine = it.current() )
		{
			autoDuration = line->autoDuration( charMsecs, wordMsecs, lineMsecs, mode );

			if ( ! nextLine ) // the last line doesn't have risk of overlapping
				line->setDurationTime( autoDuration );
			else
			{
				if ( canOverlap || line->showTime() + autoDuration < nextLine->showTime() )
					line->setDurationTime( autoDuration );
				else // setting the duration to autoDuration will cause an unwanted overlap
					line->setHideTime( nextLine->showTime() - 1 );
			}
		}
	}

	endCompositeAction();
}

void Subtitle::fixOverlappingLines( const RangeList& ranges, const Time& minInterval )
{
	if ( m_lines.isEmpty() )
		return;

	beginCompositeAction( i18n( "Fix Overlapping Lines" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(), end = ranges.end(); rangesIt != end; ++rangesIt )
	{
		SubtitleIterator it( *this, Range( (*rangesIt).start(), SubtitleAction::getLastIndex( *this, (*rangesIt).end() ) + 1 ) );
		SubtitleLine* line = it.current();
		++it;
		SubtitleLine* nextLine = it.current();

		for ( ; nextLine; ++it, line = nextLine, nextLine = it.current() )
		{
			if ( line->hideTime() + minInterval >= nextLine->showTime() )
				line->setHideTime( nextLine->showTime() - minInterval );
		}
	}

	endCompositeAction();
}


void Subtitle::fixPunctuation( const RangeList& ranges, bool spaces, bool quotes, bool engI, bool ellipsis, SubtitleLine::OpMode mode )
{
	if ( m_lines.isEmpty() || (! spaces && ! quotes && ! engI && ! ellipsis) || mode >= SubtitleLine::OpModeUNKNOWN )
		return;

	beginCompositeAction( i18n( "Fix Lines Punctuation" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(), end = ranges.end(); rangesIt != end; ++rangesIt )
	{
		SubtitleIterator it( *this, *rangesIt );

		bool primaryContinues = false;
		bool secondaryContinues = false;

		if ( it.index() > 0 )
		{
			if ( mode == SubtitleLine::Primary || mode == SubtitleLine::Both )
				// Initialize the value of primaryContinues
				SubtitleLine::fixPunctuation(
					m_lines.at( it.index() - 1 )->primaryText(),
					spaces,
					quotes,
					engI,
					ellipsis,
					&primaryContinues
				);

			if ( mode == SubtitleLine::Secondary || mode == SubtitleLine::Both )
				// Initialize the value of secondaryContinues
				SubtitleLine::fixPunctuation(
					m_lines.at( it.index() - 1 )->secondaryText(),
					spaces,
					quotes,
					engI,
					ellipsis,
					&secondaryContinues
				);
		}

		for ( ; it.current(); ++it )
		{
			switch ( mode )
			{
				case SubtitleLine::Primary:
					it.current()->setPrimaryText(
						SubtitleLine::fixPunctuation(
							it.current()->primaryText(),
							spaces,
							quotes,
							engI,
							ellipsis,
							&primaryContinues
						)
					);
					break;
				case SubtitleLine::Secondary:
					it.current()->setSecondaryText(
						SubtitleLine::fixPunctuation(
							it.current()->secondaryText(),
							spaces,
							quotes,
							engI,
							ellipsis,
							&secondaryContinues
						)
					);
					break;
				case SubtitleLine::Both:
					it.current()->setTexts(
						SubtitleLine::fixPunctuation(
							it.current()->primaryText(),
							spaces,
							quotes,
							engI,
							ellipsis,
							&primaryContinues
						),
						SubtitleLine::fixPunctuation(
							it.current()->secondaryText(),
							spaces,
							quotes,
							engI,
							ellipsis,
							&secondaryContinues
						)
					);
					break;
				default:
					break;
			}

		}
	}

	endCompositeAction();
}

void Subtitle::lowerCase( const RangeList& ranges, SubtitleLine::OpMode mode )
{
	if ( m_lines.isEmpty() || mode >= SubtitleLine::OpModeUNKNOWN )
		return;

	beginCompositeAction( i18n( "Lower Case" ) );

	switch ( mode )
	{
		case SubtitleLine::Primary:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setPrimaryText( it.current()->primaryText().lower() );
			break;
		}
		case SubtitleLine::Secondary:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setSecondaryText( it.current()->secondaryText().lower() );
			break;
		}
		case SubtitleLine::Both:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setTexts( it.current()->primaryText().lower(), it.current()->secondaryText().lower() );
			break;
		}
		default:
			break;
	}

	endCompositeAction();
}

void Subtitle::upperCase( const RangeList& ranges, SubtitleLine::OpMode mode )
{
	if ( m_lines.isEmpty() || mode >= SubtitleLine::OpModeUNKNOWN )
		return;

	beginCompositeAction( i18n( "Upper Case" ) );

	switch ( mode )
	{
		case SubtitleLine::Primary:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setPrimaryText( it.current()->primaryText().upper() );
			break;
		}
		case SubtitleLine::Secondary:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setSecondaryText( it.current()->secondaryText().upper() );
			break;
		}
		case SubtitleLine::Both:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setTexts( it.current()->primaryText().upper(), it.current()->secondaryText().upper() );
			break;
		}
		default:
			break;
	}

	endCompositeAction();
}

void Subtitle::titleCase( const RangeList& ranges, bool lowerFirst, SubtitleLine::OpMode mode )
{
	if ( m_lines.isEmpty() || mode >= SubtitleLine::OpModeUNKNOWN )
		return;

	beginCompositeAction( i18n( "Title Case" ) );

	switch ( mode )
	{
		case SubtitleLine::Primary:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setPrimaryText( SubtitleLine::titleCase( it.current()->primaryText(), lowerFirst ) );
			break;
		}
		case SubtitleLine::Secondary:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setSecondaryText( SubtitleLine::titleCase( it.current()->secondaryText(), lowerFirst ) );
			break;
		}
		case SubtitleLine::Both:
		{
			for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
				it.current()->setTexts(
					SubtitleLine::titleCase( it.current()->primaryText(), lowerFirst ),
					SubtitleLine::titleCase( it.current()->secondaryText(), lowerFirst )
				);
			break;
		}
		default:
			break;
	}

	endCompositeAction();
}

void Subtitle::sentenceCase( const RangeList& ranges, bool lowerFirst, SubtitleLine::OpMode mode )
{
	if ( m_lines.isEmpty() || mode >= SubtitleLine::OpModeUNKNOWN )
		return;

	beginCompositeAction( i18n( "Sentence Case" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(), rangesEnd = ranges.end(); rangesIt != rangesEnd; ++rangesIt )
	{
		SubtitleIterator it( *this, *rangesIt );

		bool primaryContinues = false;
		bool secondaryContinues = false;

		if ( it.index() > 0 )
		{
			if ( mode == SubtitleLine::Primary || mode == SubtitleLine::Both ) // Initialize primaryContinues
				SubtitleLine::sentenceCase( m_lines.at( it.index() - 1 )->primaryText(), lowerFirst, &primaryContinues );
			if ( mode == SubtitleLine::Secondary || mode == SubtitleLine::Both ) // Initialize secondaryContinues
				SubtitleLine::sentenceCase( m_lines.at( it.index() - 1 )->secondaryText(), lowerFirst, &secondaryContinues );
		}

		switch ( mode )
		{
			case SubtitleLine::Primary:
			{
				for ( ; it.current(); ++it )
					it.current()->setPrimaryText(
						SubtitleLine::sentenceCase( it.current()->primaryText(), lowerFirst, &primaryContinues )
					);
				break;
			}
			case SubtitleLine::Secondary:
			{
				for ( ; it.current(); ++it )
					it.current()->setSecondaryText(
						SubtitleLine::sentenceCase( it.current()->secondaryText(), lowerFirst, &secondaryContinues )
					);
				break;
			}
			case SubtitleLine::Both:
			{
				for ( ; it.current(); ++it )
					it.current()->setTexts(
						SubtitleLine::sentenceCase( it.current()->primaryText(), lowerFirst, &primaryContinues ),
						SubtitleLine::sentenceCase( it.current()->secondaryText(), lowerFirst, &secondaryContinues )
					);
				break;
			}
			default:
				break;
		}
	}

	endCompositeAction();
}


void Subtitle::adjustTexts( const RangeList& ranges, unsigned minLengthForLineBreak, SubtitleLine::OpMode mode )
{
	SubtitleCompositeActionExecutor executor( *this, i18n( "Smart Lines Adjust" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->adjustText( minLengthForLineBreak, mode );
}

void Subtitle::unbreakTexts( const RangeList& ranges, SubtitleLine::OpMode mode )
{
	SubtitleCompositeActionExecutor executor( *this, i18n( "Unbreak Lines" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->unbreakText( mode );
}

void Subtitle::simplifySpaces( const RangeList& ranges, SubtitleLine::OpMode mode )
{
	SubtitleCompositeActionExecutor executor( *this, i18n( "Simplify Spaces" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->simplifySpaces( mode );
}


void Subtitle::syncToSubtitle( const Subtitle& referenceSubtitle )
{
	beginCompositeAction( i18n( "Synchronize Subtitles" ) );

	for ( SubtitleIterator it( *this, Range::full() ), refIt( referenceSubtitle, Range::full() );
		  it.current() && refIt.current();
		  ++it, ++refIt )
		it.current()->setTimes( refIt.current()->showTime(), refIt.current()->hideTime() );

	endCompositeAction();
}

void Subtitle::appendSubtitle( const Subtitle& subtitle, long shiftMsecsBeforeAppend )
{
	if ( ! subtitle.m_lines.count() )
		return;

	QPtrList<SubtitleLine> lines;
	for ( SubtitleIterator it( subtitle ); it.current(); ++it )
	{
		SubtitleLine* newLine = new SubtitleLine( *(it.current()) );
		newLine->shiftTimes( shiftMsecsBeforeAppend );
		lines.append( newLine );
	}

	beginCompositeAction( i18n( "Join Subtitles" ) );

	processAction( new InsertLinesAction( *this, m_lines.count(), lines ) );

	endCompositeAction();
}


void Subtitle::splitSubtitle( Subtitle& subtitle, const Time& splitTime, bool shiftSplittedLines )
{
	if ( ! m_lines.count() )
		return;

	int splitIndex = -1;

	QPtrList<SubtitleLine> lines;
	for ( SubtitleIterator it( *this, Range::full() ); it.current(); ++it )
	{
		if ( it.current()->showTime() < splitTime )
			continue;

		if ( splitIndex < 0 )
			splitIndex = it.current()->number()-1;

		SubtitleLine* newLine = new SubtitleLine( *(it.current()) );

		if ( it.current()->m_formatData )
			newLine->m_formatData = new FormatData( *(it.current()->m_formatData) );

		if ( shiftSplittedLines )
			newLine->shiftTimes( - splitTime.toMillis() );

		lines.append( newLine );
	}

	if ( splitIndex >= 0 )
	{
		subtitle.m_formatData = m_formatData ? new FormatData( *m_formatData ) : 0;

		subtitle.beginCompositeAction( i18n( "Split Subtitles" ) );
		if ( subtitle.m_lines.count() )
			subtitle.processAction( new RemoveLinesAction( subtitle, 0, Range::MaxIndex ) );
		subtitle.processAction( new InsertLinesAction( subtitle, 0, lines ) );
		subtitle.endCompositeAction();

		beginCompositeAction( i18n( "Split Subtitles" ) );
		processAction( new RemoveLinesAction( *this, splitIndex, Range::MaxIndex ) );
		endCompositeAction();
	}
}

void Subtitle::setStyleFlags( const RangeList& ranges, int styleFlags )
{
	beginCompositeAction( i18n( "Set Lines Style" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->setStyleFlags( styleFlags );

	endCompositeAction();
}

void Subtitle::setStyleFlags( const RangeList& ranges, int styleFlags, bool value )
{
	beginCompositeAction( i18n( "Set Lines Style" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->setStyleFlags( styleFlags, value );

	endCompositeAction();
}

void Subtitle::toggleStyleFlag( const RangeList& ranges, SubtitleLine::StyleFlag styleFlag )
{
	SubtitleIterator it( *this, ranges );
	if ( ! it.current() )
		return;

	beginCompositeAction( i18n( "Toggle Lines Style" ) );

	setStyleFlags( ranges, styleFlag, ! (it.current()->styleFlags() & styleFlag) );

	endCompositeAction();
}


void Subtitle::setMarked( const RangeList& ranges, bool value )
{
	beginCompositeAction( value ? i18n( "Set Lines Mark" ) : i18n( "Clear Lines Mark" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->setMarked( value );

	endCompositeAction();
}

void Subtitle::toggleMarked( const RangeList& ranges )
{
	SubtitleIterator it( *this, ranges );
	if ( ! it.current() )
		return;

	beginCompositeAction( i18n( "Toggle Lines Marked" ) );

	setMarked( ranges, ! it.current()->marked() );

	endCompositeAction();
}


void Subtitle::clearErrors( const RangeList& ranges, int errorFlags )
{
	beginCompositeAction( i18n( "Clear Lines Errors" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->setErrorFlags( errorFlags, false );

	endCompositeAction();
}

void Subtitle::checkErrors( const RangeList& ranges, int errorFlags, int minDurationMsecs, int maxDurationMsecs, int minMsecsPerChar, int maxMsecsPerChar, int maxChars, int maxLines )
{
	beginCompositeAction( i18n( "Check Lines Errors" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
	{
		it.current()->check(
			errorFlags,
			minDurationMsecs,
			maxDurationMsecs,
			minMsecsPerChar,
			maxMsecsPerChar,
			maxChars,
			maxLines
		);
	}

	endCompositeAction();
}

void Subtitle::recheckErrors( const RangeList& ranges, int minDurationMsecs, int maxDurationMsecs, int minMsecsPerChar, int maxMsecsPerChar, int maxChars, int maxLines )
{
	beginCompositeAction( i18n( "Check Lines Errors" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
	{
		it.current()->check(
			it.current()->errorFlags(),
			minDurationMsecs,
			maxDurationMsecs,
			minMsecsPerChar,
			maxMsecsPerChar,
			maxChars,
			maxLines
		);
	}

	endCompositeAction();
}


void Subtitle::processAction( Action* action )
{
	if ( m_compositeAction )
		m_compositeAction->appendAction( action );
	else
		m_actionManager.execAndStore( action );
}

void Subtitle::beginCompositeAction( const QString& title, bool immediateExecution, bool delaySignals )
{
	m_compositeActionDepth++;

	if ( ! m_compositeAction )
		m_compositeAction = new CompositeAction( title, immediateExecution, delaySignals );
}

void Subtitle::endCompositeAction()
{
	m_compositeActionDepth--;

	if ( ! m_compositeActionDepth )
	{
		// NOTE: it is EXTREMELY IMPORTANT to mark the composite action as finished (setting it to 0)
		// before performing m_actionManager.execAndStore() because that call may spawn other composite
		// actions that won't be correctly initialized if m_compositeAction was not set to 0.
		CompositeAction* action = m_compositeAction;
		m_compositeAction = 0;
		if ( action->count() )
			m_actionManager.execAndStore( action );
		else
			delete action;
	}
}

void Subtitle::incrementState( int dirtyMode )
{
	switch ( dirtyMode )
	{
		case SubtitleAction::Primary:
			m_primaryState++;
			if ( m_primaryState == (m_primaryCleanState + 1) )
				emit primaryDirtyStateChanged( true );
			else if ( m_primaryState == m_primaryCleanState )
				emit primaryDirtyStateChanged( false );
			break;
		case SubtitleAction::Secondary:
			m_secondaryState++;
			if ( m_secondaryState == (m_secondaryCleanState + 1) )
				emit secondaryDirtyStateChanged( true );
			else if ( m_secondaryState == m_secondaryCleanState )
				emit secondaryDirtyStateChanged( false );
			break;
		case SubtitleAction::Both:
			m_primaryState++;
			m_secondaryState++;
			if ( m_primaryState == (m_primaryCleanState + 1) )
				emit primaryDirtyStateChanged( true );
			else if ( m_primaryState == m_primaryCleanState )
				emit primaryDirtyStateChanged( false );
			if ( m_secondaryState == (m_secondaryCleanState + 1) )
				emit secondaryDirtyStateChanged( true );
			else if ( m_secondaryState == m_secondaryCleanState )
				emit secondaryDirtyStateChanged( false );
			break;
		default:
			break;
	}
}

void Subtitle::decrementState( int dirtyMode )
{
	switch ( dirtyMode )
	{
		case SubtitleAction::Primary:
			m_primaryState--;
			if ( m_primaryState == m_primaryCleanState )
				emit primaryDirtyStateChanged( false );
			else if ( m_primaryState == (m_primaryCleanState - 1) )
				emit primaryDirtyStateChanged( true );
			break;
		case SubtitleAction::Secondary:
			m_secondaryState--;
			if ( m_secondaryState == m_secondaryCleanState )
				emit secondaryDirtyStateChanged( false );
			else if ( m_secondaryState == (m_secondaryCleanState - 1) )
				emit secondaryDirtyStateChanged( true );
			break;
		case SubtitleAction::Both:
			m_primaryState--;
			m_secondaryState--;
			if ( m_primaryState == m_primaryCleanState )
				emit primaryDirtyStateChanged( false );
			else if ( m_primaryState == (m_primaryCleanState - 1) )
				emit primaryDirtyStateChanged( true );
			if ( m_secondaryState == m_secondaryCleanState )
				emit secondaryDirtyStateChanged( false );
			else if ( m_secondaryState == (m_secondaryCleanState - 1) )
				emit secondaryDirtyStateChanged( true );
			break;
	}
}




/// SUBTITLECOMPOSITEACTIONEXECUTOR

SubtitleCompositeActionExecutor::SubtitleCompositeActionExecutor( Subtitle& subtitle, const QString& title ):
	m_subtitle( subtitle )
{
	m_subtitle.beginCompositeAction( title );
}

SubtitleCompositeActionExecutor::~SubtitleCompositeActionExecutor()
{
	m_subtitle.endCompositeAction();
}

