/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "subtitleactions.h"
#include "subtitleline.h"
#include "subtitleiterator.h"

#include <klocale.h>

using namespace SubtitleComposer;

/// SUBTITLE ACTION
/// ===============

SubtitleAction::SubtitleAction( Subtitle& subtitle, SubtitleAction::DirtyMode dirtyMode, const QString& description ):
	Action( description ),
	m_subtitle( subtitle ),
	m_dirtyMode( dirtyMode )
{
}

SubtitleAction::~SubtitleAction()
{
}

void SubtitleAction::_preRedo()
{
	m_subtitle.incrementState( m_dirtyMode );
}

void SubtitleAction::_preUndo()
{
	m_subtitle.decrementState( m_dirtyMode );
}

void SubtitleAction::setLineSubtitle( SubtitleLine* line )
{
	line->m_subtitle = &m_subtitle;
}

void SubtitleAction::clearLineSubtitle( SubtitleLine* line )
{
	line->m_subtitle = 0;
}

/// SET FRAMES PER SECOND ACTION
/// ============================

SetFramesPerSecondAction::SetFramesPerSecondAction( Subtitle& subtitle, double framesPerSecond ):
	SubtitleAction( subtitle, SubtitleAction::Both, i18n( "Set Frame Rate" ) ),
	m_framesPerSecond( framesPerSecond ) {}
SetFramesPerSecondAction::~SetFramesPerSecondAction() {}

void SetFramesPerSecondAction::_redo()
{
	double aux = m_subtitle.m_framesPerSecond;
	m_subtitle.m_framesPerSecond = m_framesPerSecond;
	m_framesPerSecond = aux;
}

void SetFramesPerSecondAction::_undo()
{
	_redo();
}

void SetFramesPerSecondAction::_emitRedoSignals()
{
	emit m_subtitle.framesPerSecondChanged( m_subtitle.m_framesPerSecond );
}

void SetFramesPerSecondAction::_emitUndoSignals()
{
	_emitRedoSignals();
}


/// INSERT LINES ACTION
/// ===================

InsertLinesAction::InsertLinesAction( Subtitle& subtitle, unsigned insertIndex, const QPtrList<SubtitleLine>& lines ):
	SubtitleAction( subtitle, SubtitleAction::Both, i18n( lines.count() == 1 ? "Insert Line" : "Insert Lines" ) ),
	m_insertIndex( getInsertionIndex( subtitle, insertIndex ) ),
	m_lastIndex( m_insertIndex + lines.count() - 1 ),
	m_lines( lines )
{
	m_lines.setAutoDelete( true );
}

InsertLinesAction::~InsertLinesAction()
{
}

void InsertLinesAction::_redo()
{
	if ( m_insertIndex > m_lastIndex )
		return; // m_lines is empty

	unsigned insertOffset = 0;
	while ( ! m_lines.isEmpty() )
	{
		SubtitleLine* line = m_lines.take( 0 );
		m_subtitle.m_lines.insert( m_insertIndex + insertOffset++, line );
		setLineSubtitle( line );
	}
}

void InsertLinesAction::_undo()
{
	if ( m_insertIndex > m_lastIndex )
		return; // m_lines is empty

	for ( unsigned index = m_insertIndex; index <= m_lastIndex; ++index )
	{
		SubtitleLine* line = m_subtitle.m_lines.take( m_insertIndex );
		clearLineSubtitle( line );
		m_lines.append( line );
	}
}

void InsertLinesAction::_emitRedoSignals()
{
	emit m_subtitle.linesInserted( m_insertIndex, m_lastIndex );
}

void InsertLinesAction::_emitUndoSignals()
{
	emit m_subtitle.linesRemoved( m_insertIndex, m_lastIndex );
}



/// REMOVE LINES ACTION
/// ===================

RemoveLinesAction::RemoveLinesAction( Subtitle& subtitle, unsigned firstIndex, unsigned lastIndex ):
	SubtitleAction( subtitle, SubtitleAction::Both, i18n( "Remove Line" ) ),
	m_firstIndex( firstIndex ),
	m_lastIndex( getLastIndex( subtitle, lastIndex ) ),
	m_lines()
{
	if ( m_firstIndex != m_lastIndex )
		m_description = i18n( "Remove Lines" );

	m_lines.setAutoDelete( true );
}

RemoveLinesAction::~RemoveLinesAction()
{
}

void RemoveLinesAction::_redo()
{
	if ( m_firstIndex > m_lastIndex )
		return;

	for ( unsigned index = m_firstIndex; index <= m_lastIndex; ++index )
	{
		SubtitleLine* line = m_subtitle.m_lines.take( m_firstIndex );
		clearLineSubtitle( line );
		m_lines.append( line );
	}
}

void RemoveLinesAction::_undo()
{
	if ( m_firstIndex > m_lastIndex )
		return;

	unsigned insertOffset = 0;
	while ( ! m_lines.isEmpty() )
	{
		SubtitleLine* line = m_lines.take( 0 );
		m_subtitle.m_lines.insert( m_firstIndex + insertOffset++, line );
		setLineSubtitle( line );
	}
}

void RemoveLinesAction::_emitRedoSignals()
{
	emit m_subtitle.linesRemoved( m_firstIndex, m_lastIndex );
}

void RemoveLinesAction::_emitUndoSignals()
{
	emit m_subtitle.linesInserted( m_firstIndex, m_lastIndex );
}




/// MOVE LINE ACTION
/// ================

MoveLineAction::MoveLineAction( Subtitle& subtitle, unsigned fromIndex, unsigned toIndex ):
	SubtitleAction( subtitle, SubtitleAction::Both, i18n( "Move Line" ) ),
	m_fromIndex( fromIndex ),
	m_toIndex( getLastIndex( subtitle, toIndex ) ) {}
MoveLineAction::~MoveLineAction() {}

bool MoveLineAction::mergeWithPrevious( Action* pa )
{
	MoveLineAction* prevAction = tryCastTo<MoveLineAction>( pa );
	if ( ! prevAction || &prevAction->m_subtitle != &m_subtitle )
		return false;

	bool compressed = false;

	if ( prevAction->m_toIndex == m_fromIndex )
	{
		m_fromIndex = prevAction->m_fromIndex;
		compressed = true;
	}

	// when the distance between fromIndex and toIndex is 1, the action is the same as if the values were swaped
	else if ( m_toIndex - m_fromIndex == 1 || m_fromIndex - m_toIndex == 1 )
	{
		if ( prevAction->m_toIndex == m_toIndex )
		{
			m_toIndex = m_fromIndex;
			m_fromIndex = prevAction->m_fromIndex;
			compressed = true;
		}

		// same as before, but now we consider inverting the previous action too
		else if ( prevAction->m_toIndex - prevAction->m_fromIndex == 1 || prevAction->m_fromIndex - prevAction->m_toIndex == 1 )
		{
			if ( prevAction->m_fromIndex == m_toIndex )
			{
				m_toIndex = m_fromIndex;
				m_fromIndex = prevAction->m_toIndex;
				compressed = true;
			}
		}

	}

	// again, same as before, but now we consider inverting only the previous action
	else if ( prevAction->m_toIndex - prevAction->m_fromIndex == 1 || prevAction->m_fromIndex - prevAction->m_toIndex == 1 )
	{
		if ( prevAction->m_fromIndex == m_fromIndex )
		{
			m_fromIndex = prevAction->m_toIndex;
			compressed = true;
		}
	}

	if ( compressed )
		prevAction->_preUndo();

	return compressed;
}

void MoveLineAction::_redo()
{
	SubtitleLine* line = m_subtitle.m_lines.take( m_fromIndex );
	m_subtitle.m_lines.insert( m_toIndex, line );
/*
	kDebug() << "redo";
	for ( int idx = 0; idx < m_subtitle.m_lines.count(); ++idx )
		kDebug() << m_subtitle.m_lines.at( idx )->text();*/
}

void MoveLineAction::_undo()
{
	SubtitleLine* line = m_subtitle.m_lines.take( m_toIndex );
	m_subtitle.m_lines.insert( m_fromIndex, line );
/*
	kDebug() << "undo";
	for ( int idx = 0; idx < m_subtitle.m_lines.count(); ++idx )
		kDebug() << m_subtitle.m_lines.at( idx )->text();*/
}

void MoveLineAction::_emitRedoSignals()
{
	emit m_subtitle.linesRemoved( m_fromIndex, m_fromIndex );
	emit m_subtitle.linesInserted( m_toIndex, m_toIndex );
}

void MoveLineAction::_emitUndoSignals()
{
	emit m_subtitle.linesRemoved( m_toIndex, m_toIndex );
	emit m_subtitle.linesInserted( m_fromIndex, m_fromIndex );
}







/// SWAP LINES TEXTS ACTION
/// =======================

SwapLinesTextsAction::SwapLinesTextsAction( Subtitle& subtitle, const RangeList& ranges ):
	SubtitleAction( subtitle, SubtitleAction::Both, i18n( "Swap Texts" ) ),
	m_ranges( ranges )
{
}

SwapLinesTextsAction::~SwapLinesTextsAction()
{
}

void SwapLinesTextsAction::_redo()
{
	for ( SubtitleIterator it( m_subtitle, m_ranges ); it.current(); ++it )
	{
		SubtitleLine* line = it.current();
		QString aux = line->m_primaryText;
		line->m_secondaryText = line->m_primaryText;
		line->m_primaryText = aux;
	}
}

void SwapLinesTextsAction::_undo()
{
	_redo();
}

void SwapLinesTextsAction::_emitRedoSignals()
{
	for ( SubtitleIterator it( m_subtitle, m_ranges ); it.current(); ++it )
	{
		SubtitleLine* line = it.current();
		emit line->primaryTextChanged( line->m_primaryText );
		emit line->secondaryTextChanged( line->m_secondaryText );
	}
}

void SwapLinesTextsAction::_emitUndoSignals()
{
	_emitRedoSignals();
}
