/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "subtitle.h"
#include "subtitleline.h"
#include "subtitlelineactions.h"

#include <qregexp.h>

#include <klocale.h>

#include <math.h>

using namespace SubtitleComposer;

SubtitleLine::ErrorFlag SubtitleLine::errorFlag( SubtitleLine::ErrorID id )
{
	if ( id < 0 || id >= ErrorSIZE )
		return (ErrorFlag)0;

	return (ErrorFlag)(0x1 << id);
}

SubtitleLine::ErrorID SubtitleLine::errorID( SubtitleLine::ErrorFlag flag )
{
	if ( flag < 1 )
		return ErrorUNKNOWN;

	int id = (int)log2( flag );
	return id < ErrorSIZE ? (ErrorID)id : ErrorUNKNOWN;
}

/// ERRORS DESCRIPTIONS
/// ===================

const QString& SubtitleLine::errorText( SubtitleLine::ErrorFlag errorFlag, bool full )
{
	return errorText( errorID( errorFlag ), full );
}

const QString& SubtitleLine::errorText( SubtitleLine::ErrorID errorID, bool full )
{
	static QString texts[ErrorSIZE];
	static QString texts2[ErrorSIZE];

	if ( texts[EmptyPrimaryTextID].isEmpty() )
	{
		texts[EmptyPrimaryTextID]				= i18n( "Empty text" );
		texts[EmptySecondaryTextID]				= texts[EmptySecondaryTextID - 1];
		texts[MaxPrimaryCharsID]				= i18n( "Too many characters" );
		texts[MaxSecondaryCharsID]				= texts[MaxSecondaryCharsID - 1];
		texts[MaxPrimaryLinesID]				= i18n( "Too many lines" );
		texts[MaxSecondaryLinesID]				= texts[MaxSecondaryLinesID - 1];
		texts[PrimaryUnneededSpacesID]			= i18n( "Unnecessary spaces" );
		texts[SecondaryUnneededSpacesID]		= texts[SecondaryUnneededSpacesID - 1];
		texts[PrimaryUnneededDashID]			= i18n( "Unnecessary dash" );
		texts[SecondaryUnneededDashID]			= texts[SecondaryUnneededDashID - 1];
		texts[PrimaryCapitalAfterEllipsisID]	= i18n( "Capital letter after line continuation ellipsis" );
		texts[SecondaryCapitalAfterEllipsisID]	= texts[SecondaryCapitalAfterEllipsisID - 1];
		texts[MaxDurationPerPrimaryCharID]		= i18n( "Too long duration per character" );
		texts[MaxDurationPerSecondaryCharID]	= texts[MaxDurationPerSecondaryCharID - 1];
		texts[MinDurationPerPrimaryCharID]		= i18n( "Too short duration per character" );
		texts[MinDurationPerSecondaryCharID]	= texts[MinDurationPerSecondaryCharID - 1];
		texts[MaxDurationID]					= i18n( "Too long duration" );
		texts[MinDurationID]					= i18n( "Too short duration" );
		texts[OverlapsWithNextID]				= i18n( "Overlapping times" );

		texts2[EmptyPrimaryTextID]				= i18n( "Has no primary text." );
		texts2[EmptySecondaryTextID]			= i18n( "Has no translation text." );
		texts2[MaxPrimaryCharsID]				= i18n( "Has too many characters in primary text (%1 chars)." );
		texts2[MaxSecondaryCharsID]				= i18n( "Has too many characters in translation text (%1 chars)." );
		texts2[MaxPrimaryLinesID]				= i18n( "Has too many lines in primary text (%1 lines)." );
		texts2[MaxSecondaryLinesID]				= i18n( "Has too many lines in translation text (%1 lines)." );
		texts2[PrimaryUnneededSpacesID]			= i18n( "Has unnecessary spaces in primary text." );
		texts2[SecondaryUnneededSpacesID]		= i18n( "Has unnecessary spaces in translation text." );
		texts2[PrimaryUnneededDashID]			= i18n( "Has unnecessary dash in primary text." );
		texts2[SecondaryUnneededDashID]			= i18n( "Has unnecessary dash in translation text." );
		texts2[PrimaryCapitalAfterEllipsisID]	= i18n( "Has capital letter after line continuation ellipsis in primary text." );
		texts2[SecondaryCapitalAfterEllipsisID]	= i18n( "Has capital letter after line continuation ellipsis in translation "
														"text." );
		texts2[MaxDurationPerPrimaryCharID]		= i18n( "Has too long duration per character in primary text (%1 msecs/char)." );
		texts2[MaxDurationPerSecondaryCharID]	= i18n( "Has too long duration per character in translation text "
														"(%1 msecs/char)." );
		texts2[MinDurationPerPrimaryCharID]		= i18n( "Has too short duration per character in primary text (%1 msecs/char)." );
		texts2[MinDurationPerSecondaryCharID]	= i18n( "Has too short duration per character in translation text "
														"(%1 msecs/char)." );
		texts2[MaxDurationID]					= i18n( "Has too long duration (%1 msecs)." );
		texts2[MinDurationID]					= i18n( "Has too short duration (%1 msecs)." );
		texts2[OverlapsWithNextID]				= i18n( "Overlaps with next line." );
	}

	if ( errorID < 0 || errorID >= ErrorSIZE )
		return QString::null;
	else
		return full ? texts2[errorID] : texts[errorID];
}



SubtitleLine::SubtitleLine( const QString& primaryText, const QString& secondaryText ):
	QObject(),
	m_subtitle( 0 ),
	m_primaryText( primaryText ),
	m_secondaryText( secondaryText ),
	m_showTime(),
	m_hideTime(),
	m_styleFlags( 0 ),
	m_errorFlags( 0 ),
	m_marked( false ),
	m_formatData( 0 )
{
}

SubtitleLine::SubtitleLine( const QString& primaryText, const QString& secondaryText, const Time& showTime, const Time& hideTime, int styleFlags ):
	QObject(),
	m_subtitle( 0 ),
	m_primaryText( primaryText ),
	m_secondaryText( secondaryText ),
	m_showTime( showTime ),
	m_hideTime( hideTime ),
	m_styleFlags( styleFlags ),
	m_errorFlags( 0 ),
	m_marked( false ),
	m_formatData( 0 )
{
}

SubtitleLine::SubtitleLine( const SubtitleLine& line ):
	QObject(),
	m_subtitle( 0 ),
	m_primaryText( line.m_primaryText ),
	m_secondaryText( line.m_secondaryText ),
	m_showTime( line.m_showTime ),
	m_hideTime( line.m_hideTime ),
	m_styleFlags( line.m_styleFlags ),
	m_errorFlags( line.m_errorFlags ),
	m_marked( line.m_marked ),
	m_formatData( 0 )
{
}

SubtitleLine& SubtitleLine::operator=( const SubtitleLine& line )
{
	if ( this == &line )
		return *this;

	m_primaryText = line.m_primaryText;
	m_secondaryText = line.m_secondaryText;
	m_showTime = line.m_showTime;
	m_hideTime = line.m_hideTime;
	m_styleFlags = line.m_styleFlags;
	m_errorFlags = line.m_errorFlags;
	m_marked = line.m_marked;

	return *this;
}

SubtitleLine::~SubtitleLine()
{
	delete m_formatData;
}


FormatData* const SubtitleLine::formatData() const
{
	return m_formatData;
}

void SubtitleLine::setFormatData( const FormatData* formatData )
{
	delete m_formatData;

	m_formatData = formatData ? new FormatData( *formatData ) : 0;
}

unsigned SubtitleLine::number() const
{
	return m_subtitle ? m_subtitle->lineNumber( this ) : 0;
}

int SubtitleLine::index() const
{
	return m_subtitle ? m_subtitle->lineIndex( this ) : -1;
}

Subtitle* SubtitleLine::subtitle()
{
	return m_subtitle;
}

const Subtitle* SubtitleLine::subtitle() const
{
	return m_subtitle;
}

SubtitleLine* SubtitleLine::prevLine()
{
	return m_subtitle ? m_subtitle->prevLine( this ) : 0;
}

SubtitleLine* SubtitleLine::nextLine()
{
	return m_subtitle ? m_subtitle->nextLine( this ) : 0;
}

QString SubtitleLine::primaryText() const
{
	return m_primaryText;
}

void SubtitleLine::setPrimaryText( const QString& text )
{
	if ( m_primaryText != text )
		processAction( new SetLinePrimaryTextAction( *this, text ) );
}

QString SubtitleLine::secondaryText() const
{
	return m_secondaryText;
}

void SubtitleLine::setSecondaryText( const QString& text )
{
	if ( m_secondaryText != text )
		processAction( new SetLineSecondaryTextAction( *this, text ) );
}

void SubtitleLine::setTexts( const QString& primaryText, const QString& secondaryText )
{
	if ( m_primaryText != primaryText || m_secondaryText != secondaryText )
		processAction( new SetLineTextsAction( *this, primaryText, secondaryText ) );
}

QString SubtitleLine::fixPunctuation( const QString& t, bool spaces, bool quotes, bool engI, bool ellipsis, bool* cont )
{
	if ( ! t.length() )
		return t;

	QString text( t );

	if ( spaces )
	{
		text = simplifySpaces( text );

		// remove spaces after " or ' at the beginning of line
		text.replace( QRegExp( "^([\"'])\\s" ), "\\1" );

		// remove space before " or ' at the end of line
		text.replace( QRegExp( "\\s([\"'])$" ), "\\1" );

		// if not present, add space after '?', '!', ',', ';', ':', ')' and ']'
		text.replace( QRegExp( "([\\?!,;:\\)\\]])([^\\s\"'])" ), "\\1 \\2" );

		// if not present, add space after '.'
		text.replace( QRegExp( "(\\.)([^\\s\\.\"'])" ), "\\1 \\2" );

		// remove space after '¿', '¡', '(' and '['
		text.replace( QRegExp( "([¿¡\\(\\[])\\s" ), "\\1" );

		// remove space before '?', '!', ',', ';', ':', '.', ')' and ']'
		text.replace( QRegExp( "\\s([\\?!,;:\\.\\)\\]])" ), "\\1" );

		// remove space after ... at the beginning of sentence
		text.replace( QRegExp( "^\\.\\.\\.\\s" ), "..." );
	}

	if ( quotes ) // quotes and double quotes
	{
		text.replace( QRegExp( "`|´|" ), "'" );
		text.replace( QRegExp( "''|«|»" ), "\"" );
	}

	if ( engI ) // fix english I pronoun capitalization
		text.replace( QRegExp( "([\\s\"'\\(\\[])i([\\s'\",;:\\.\\?!\\]\\)]|$)" ), "\\1I\\2" );

	if ( ellipsis ) // fix ellipsis
	{
		text.replace( QRegExp( "[,;]?\\.{2,}" ), "..." );
		text.replace( QRegExp( "[,;]\\s*$" ), "..." );

		if ( text.find( QRegExp( "[\\.:?!\\)\\]'\\\"]$" ) ) == -1 )
			text.append( "..." );

		if ( cont )
		{
			if ( *cont && text.find( QRegExp( "^\\s*\\.{3}[^\\.]?" ) ) == -1 )
				text.replace( QRegExp( "^\\s*\\.*\\s*" ), "..." );

			*cont = text.find( QRegExp( "\\.{3,3}\\s*$" ) ) != -1;
		}
	}
	else
	{
		if ( cont )
		{
			*cont = text.find( QRegExp( "[?!\\)\\]'\\\"]\\s*$" ) ) == -1;

			if ( ! *cont )
				*cont = text.find( QRegExp( "[^\\.]?\\.\\s*$" ) ) == -1;
		}
	}

	return text;
}

QString SubtitleLine::titleCase( const QString& t, bool lowerFirst )
{
	static const QString wordSeparators( " -_([:,;./\\\t\n\"" );

	QString text( lowerFirst ? t.lower() : t );

	bool wordStart = true;
	unsigned length = text.length();
	for ( unsigned idx = 0; idx < length; ++idx )
	{
		QCharRef chr = text[idx];
		if ( wordStart )
		{
			if ( ! wordSeparators.contains( chr ) )
			{
				wordStart = false;
				chr = chr.upper();
			}
		}
		else if ( wordSeparators.contains( chr ) )
			wordStart = true;
	}

	return text;
}

QString SubtitleLine::sentenceCase( const QString& t, bool lowerFirst, bool* cont )
{
	if ( ! t.length() )
		return t;

	static const QString sentenceEndChars( ".?!" );

	QString text( lowerFirst ? t.lower() : t );

	bool startSentence = cont ? ! *cont : true;
	int prevDots = 0;
	for ( int idx = 0, size = text.length(); idx < size; ++idx )
	{
		QCharRef chr = text[idx];
		if ( sentenceEndChars.contains( chr ) )
		{
			if ( chr == '.' )
			{
				prevDots++;
				startSentence = prevDots < 3;
			}
			else
			{
				prevDots = 0;
				startSentence = true;
			}
		}
		else
		{
			if ( startSentence && chr.isLetterOrNumber() )
			{
				chr = chr.upper();
				startSentence = false;
			}

			if ( ! chr.isSpace() )
				prevDots = 0;
		}
	}

	if ( cont )
		*cont = prevDots != 1 && ! startSentence;

	return text;
}

QString SubtitleLine::adjustText( const QString& t, unsigned minLengthForBreak )
{
	QString text( t );
	text.replace( "\n", " " );

	if ( (unsigned)text.length() <= minLengthForBreak )
		return text;

	static const QRegExp spaceRegExp( "[^ \t\n][ \t\n]" );

	// this is a fricking test case
	// this is a fri|cking test case	IDEAL: 13
	// this is a| fricking test case	LOWER: 10 <--
	// this is a fricking| test case	UPPER: 18

	int position = text.length()/2;
	int lowerPosition = text.findRev( spaceRegExp, position );
	int upperPosition = text.find( spaceRegExp, position );

	if ( (lowerPosition == -1 || lowerPosition == 0) && (upperPosition == -1 || upperPosition == (signed)(text.length()-1)) )
		return text;
	else if ( position - lowerPosition <= upperPosition - position )
		position = lowerPosition;
	else
		position = upperPosition;

	position++;
	text.insert( position, "\n" );
	position++;
	while ( text.at( position ).isSpace() && position < (signed)text.length() )
		text.remove( position, 1 );

	return text;
}

void SubtitleLine::adjustText( unsigned minLengthForBreak, OpMode mode )
{
	switch ( mode )
	{
		case Primary:
			setPrimaryText( adjustText( m_primaryText, minLengthForBreak ) );
			break;
		case Secondary:
			setSecondaryText( adjustText( m_secondaryText, minLengthForBreak ) );
			break;
		case Both:
			setTexts( adjustText( m_primaryText, minLengthForBreak ), adjustText( m_secondaryText, minLengthForBreak ) );
			break;
		default:
			break;
	}
}

void SubtitleLine::unbreakText( OpMode mode )
{
	switch ( mode )
	{
		case Primary:
			setPrimaryText( primaryText().replace( "\n", " " ) );
			break;
		case Secondary:
			setSecondaryText( secondaryText().replace( "\n", " " ) );
			break;
		case Both:
			setTexts( primaryText().replace( "\n", " " ), secondaryText().replace( "\n", " " ) );
			break;
		default:
			break;
	}
}

QString SubtitleLine::simplifySpaces( QString text )
{
	static const QRegExp regExp1( " *\n *" );
	static const QRegExp regExp2( " +" );
	static const QRegExp regExp3( "\n+" );

	text.replace( "\t", " " );
	text.replace( regExp1, "\n" );
	text.replace( regExp2, " " );
	text.replace( regExp3, "\n" );

	return text.stripWhiteSpace();
}

void SubtitleLine::simplifySpaces( OpMode mode )
{
	switch ( mode )
	{
		case Primary:
			setPrimaryText( simplifySpaces( m_primaryText ) );
			break;
		case Secondary:
			setSecondaryText( simplifySpaces( m_secondaryText ) );
			break;
		case Both:
			setTexts( simplifySpaces( m_primaryText ), simplifySpaces( m_secondaryText ) );
			break;
		default:
			break;
	}
}

Time SubtitleLine::showTime() const
{
	return m_showTime;
}

void SubtitleLine::setShowTime( const Time& showTime )
{
	if ( m_showTime != showTime )
		processAction( new SetLineShowTimeAction( *this, showTime ) );
}


Time SubtitleLine::hideTime() const
{
	return m_hideTime;
}

void SubtitleLine::setHideTime( const Time& hideTime )
{
	if ( m_hideTime != hideTime )
		processAction( new SetLineHideTimeAction( *this, hideTime ) );
}

Time SubtitleLine::durationTime() const
{
	return Time( m_hideTime.toMillis() - m_showTime.toMillis() );
}

void SubtitleLine::setDurationTime( const Time& durationTime )
{
	setHideTime( m_showTime + durationTime );
}

unsigned SubtitleLine::primaryCharacters()
{
	return m_primaryText.simplifyWhiteSpace().length();
}

unsigned SubtitleLine::primaryWords()
{
	QString text( m_primaryText.simplifyWhiteSpace() );
	return text.length() ? text.contains( ' ' ) + 1 : 0;
}

unsigned SubtitleLine::primaryLines()
{
	QString text( simplifySpaces( m_primaryText ) );
	return text.length() ? text.contains( '\n' ) + 1 : 0;
}

unsigned SubtitleLine::secondaryCharacters()
{
	return m_secondaryText.simplifyWhiteSpace().length();
}

unsigned SubtitleLine::secondaryWords()
{
	QString text( m_secondaryText.simplifyWhiteSpace() );
	return text.length() ? text.contains( ' ' ) + 1 : 0;
}

unsigned SubtitleLine::secondaryLines()
{
	QString text( simplifySpaces( m_secondaryText ) );
	return text.length() ? text.contains( '\n' ) + 1 : 0;
}

Time SubtitleLine::autoDuration( const QString& t, unsigned charMsecs, unsigned wordMsecs, unsigned lineMsecs )
{
	QString text( simplifySpaces( t ) );
	if ( ! text.length() )
		return 0;

	unsigned chars = text.length();
	unsigned lines = text.contains( '\n' ) + 1;
	unsigned words = text.contains( ' ' ) + lines;

	return chars*charMsecs + words*wordMsecs + lines*lineMsecs;
}

Time SubtitleLine::autoDuration( unsigned charMsecs, unsigned wordMsecs, unsigned lineMsecs, OpMode mode )
{
	switch ( mode )
	{
		case Secondary:
			return autoDuration( m_secondaryText, charMsecs, wordMsecs, lineMsecs );
		case Both:
		{
			Time primary = autoDuration( m_primaryText, charMsecs, wordMsecs, lineMsecs );
			Time secondary = autoDuration( m_secondaryText, charMsecs, wordMsecs, lineMsecs );
			return primary > secondary ? primary : secondary;
		}
		case Primary:
		default:
			return autoDuration( m_primaryText, charMsecs, wordMsecs, lineMsecs );
	}
}

void SubtitleLine::setTimes( const Time& showTime, const Time& hideTime )
{
	if ( m_showTime != showTime || m_hideTime != hideTime )
		processAction( new SetLineTimesAction( *this, showTime, hideTime, i18n( "Set Line Times" ) ) );
}

void SubtitleLine::shiftTimes( long mseconds )
{
	if ( mseconds )
		processAction(
			new SetLineTimesAction(
				*this,
				Time( m_showTime.toMillis() + mseconds ),
				Time( m_hideTime.toMillis() + mseconds ),
				i18n( "Shift Line Times" )
			)
		);
}

void SubtitleLine::adjustTimes( long shiftMseconds, double scaleFactor )
{
	if ( shiftMseconds || scaleFactor != 1.0 )
	{
		processAction(
			new SetLineTimesAction(
				*this,
				Time( (long)(shiftMseconds + m_showTime.toMillis()*scaleFactor) ),
				Time( (long)(shiftMseconds + m_hideTime.toMillis()*scaleFactor) ),
				i18n( "Adjust Line Times" )
			)
		);
	}
}



/// STYLES

int SubtitleLine::styleFlags() const
{
	return m_styleFlags;
}

void SubtitleLine::setStyleFlags( int styleFlags )
{
	if ( m_styleFlags != styleFlags )
		processAction( new SetLineStyleFlagsAction( *this, styleFlags ) );
}

void SubtitleLine::setStyleFlags( int styleFlags, bool value )
{
	int newStyleFlags = m_styleFlags;

	for ( int styleFlag = 0x1, styleID = 0; styleID < StyleSIZE; ++styleID, styleFlag = styleFlag << 1 )
	{
		if ( styleFlags & styleFlag )
		{
			if ( value )
				newStyleFlags = newStyleFlags | styleFlag;
			else
				newStyleFlags = newStyleFlags & ~styleFlag;
		}
	}

	setStyleFlags( newStyleFlags );
}



/// MARKS

bool SubtitleLine::marked() const
{
	return m_marked;
}

void SubtitleLine::setMarked( bool marked )
{
	if ( m_marked != marked )
		processAction( new ToggleLineMarkedAction( *this ) );
}



/// ERRORS

int SubtitleLine::errorFlags() const
{
	return m_errorFlags;
}

void SubtitleLine::setErrorFlags( int errorFlags )
{
	if ( m_errorFlags != errorFlags )
		processAction( new SetLineErrorsAction( *this, errorFlags ) );
}

void SubtitleLine::setErrorFlags( int errorFlags, bool value )
{
	setErrorFlags( value ? (m_errorFlags | errorFlags) : (m_errorFlags & ~errorFlags) );
}

bool SubtitleLine::checkEmptyPrimaryText( bool update )
{
	static const QRegExp emptyTextRegExp( "^\\s*$" );

	bool error = m_primaryText.isEmpty() || m_primaryText.find( emptyTextRegExp ) != -1;

	if ( update )
		setErrorFlags( EmptyPrimaryText, error );

	return error;
}

bool SubtitleLine::checkEmptySecondaryText( bool update )
{
	static const QRegExp emptyTextRegExp( "^\\s*$" );

	bool error = m_secondaryText.isEmpty() || m_secondaryText.find( emptyTextRegExp ) != -1;

	if ( update )
		setErrorFlags( EmptySecondaryText, error );

	return error;
}

bool SubtitleLine::checkOverlapsWithNext( bool update )
{
	SubtitleLine* nextLine = this->nextLine();
	bool error = nextLine && nextLine->m_showTime <= m_hideTime;

	if ( update )
		setErrorFlags( OverlapsWithNext, error );

	return error;
}

bool SubtitleLine::checkMaxDuration( unsigned maxMsecs, bool update )
{
	bool error = (unsigned)durationTime().toMillis() > maxMsecs;

	if ( update )
		setErrorFlags( MaxDuration, error );

	return error;
}

bool SubtitleLine::checkMinDuration( unsigned minMsecs, bool update )
{
	bool error = (unsigned)durationTime().toMillis() < minMsecs;

	if ( update )
		setErrorFlags( MinDuration, error );

	return error;
}

bool SubtitleLine::checkMaxDurationPerPrimaryChar( unsigned maxMsecsPerChar, bool update )
{
	unsigned characters = primaryCharacters();
	bool error = characters ? (durationTime().toMillis()/characters > maxMsecsPerChar) : false;

	if ( update )
		setErrorFlags( MaxDurationPerPrimaryChar, error );

	return error;
}

bool SubtitleLine::checkMaxDurationPerSecondaryChar( unsigned maxMsecsPerChar, bool update )
{
	unsigned characters = secondaryCharacters();
	bool error = characters ? (durationTime().toMillis()/characters > maxMsecsPerChar) : false;

	if ( update )
		setErrorFlags( MaxDurationPerSecondaryChar, error );

	return error;
}

bool SubtitleLine::checkMinDurationPerPrimaryChar( unsigned minMsecsPerChar, bool update )
{
	unsigned characters = primaryCharacters();
	bool error = characters ? (durationTime().toMillis()/characters < minMsecsPerChar) : false;

	if ( update )
		setErrorFlags( MinDurationPerPrimaryChar, error );

	return error;
}

bool SubtitleLine::checkMinDurationPerSecondaryChar( unsigned minMsecsPerChar, bool update )
{
	unsigned characters = secondaryCharacters();
	bool error = characters ? (durationTime().toMillis()/characters < minMsecsPerChar) : false;

	if ( update )
		setErrorFlags( MinDurationPerSecondaryChar, error );

	return error;
}

bool SubtitleLine::checkMaxPrimaryChars( unsigned maxCharacters, bool update )
{
	bool error = primaryCharacters() > maxCharacters;

	if ( update )
		setErrorFlags( MaxPrimaryChars, error );

	return error;
}

bool SubtitleLine::checkMaxSecondaryChars( unsigned maxCharacters, bool update )
{
	bool error = secondaryCharacters() > maxCharacters;

	if ( update )
		setErrorFlags( MaxSecondaryChars, error );

	return error;
}

bool SubtitleLine::checkMaxPrimaryLines( unsigned maxLines, bool update )
{
	bool error = primaryLines() > maxLines;

	if ( update )
		setErrorFlags( MaxPrimaryLines, error );

	return error;
}

bool SubtitleLine::checkMaxSecondaryLines( unsigned maxLines, bool update )
{
	bool error = secondaryLines() > maxLines;

	if ( update )
		setErrorFlags( MaxSecondaryLines, error );

	return error;
}

bool SubtitleLine::checkPrimaryUnneededSpaces( bool update )
{
	static const QRegExp unneededSpaceRegExp( "(^\\s|\\s$|¿\\s|¡\\s|\\s\\s|\\s!|\\s\\?|\\s:|\\s;|\\s,|\\s\\.)" );

	bool error = m_primaryText.find( unneededSpaceRegExp ) != -1;

	if ( update )
		setErrorFlags( PrimaryUnneededSpaces, error );

	return error;
}

bool SubtitleLine::checkSecondaryUnneededSpaces( bool update )
{
	static const QRegExp unneededSpaceRegExp( "(^\\s|\\s$|¿\\s|¡\\s|\\s\\s|\\s!|\\s\\?|\\s:|\\s;|\\s,|\\s\\.)" );

	bool error = m_secondaryText.find( unneededSpaceRegExp ) != -1;

	if ( update )
		setErrorFlags( SecondaryUnneededSpaces, error );

	return error;
}

bool SubtitleLine::checkPrimaryCapitalAfterEllipsis( bool update )
{
	static const QRegExp capitalAfterEllipsisRegExp( "^\\s*\\.\\.\\.[!¿\\.,;\\(\\[\\{\"'\\s]*" );

	bool error = m_primaryText.find( capitalAfterEllipsisRegExp ) != -1;
	if ( error )
	{
		QChar chr = m_primaryText.at( capitalAfterEllipsisRegExp.matchedLength() );
		error = chr.isLetter() && chr == chr.upper();
	}

	if ( update )
		setErrorFlags( PrimaryCapitalAfterEllipsis, error );

	return error;
}

bool SubtitleLine::checkSecondaryCapitalAfterEllipsis( bool update )
{
	static const QRegExp capitalAfterEllipsisRegExp( "^\\s*\\.\\.\\.[!¿\\.,;\\(\\[\\{\"'\\s]*" );

	bool error = m_secondaryText.find( capitalAfterEllipsisRegExp ) != -1;
	if ( error )
	{
		QChar chr = m_secondaryText.at( capitalAfterEllipsisRegExp.matchedLength() );
		error = chr.isLetter() && chr == chr.upper();
	}

	if ( update )
		setErrorFlags( SecondaryCapitalAfterEllipsis, error );

	return error;
}

bool SubtitleLine::checkPrimaryUnneededDash( bool update )
{
	static const QRegExp unneededDashRegExp( "(^|\n)\\s*-[^-]" );

	bool error = m_primaryText.contains( unneededDashRegExp ) == 1;

	if ( update )
		setErrorFlags( PrimaryUnneededDash, error );

	return error;
}

bool SubtitleLine::checkSecondaryUnneededDash( bool update )
{
	static const QRegExp unneededDashRegExp( "(^|\n)\\s*-[^-]" );

	bool error = m_secondaryText.contains( unneededDashRegExp ) == 1;

	if ( update )
		setErrorFlags( SecondaryUnneededDash, error );

	return error;
}

int SubtitleLine::check( int errorFlagsToCheck, int minDurationMsecs, int maxDurationMsecs, int minMsecsPerChar, int maxMsecsPerChar, int maxChars, int maxLines, bool update )
{
	int lineErrorFlags = m_errorFlags & ~errorFlagsToCheck; // clear the flags we're going to (re)check

	if ( errorFlagsToCheck & EmptyPrimaryText )
		if ( checkEmptyPrimaryText( false ) )
			lineErrorFlags |= EmptyPrimaryText;

	if ( errorFlagsToCheck & EmptySecondaryText )
		if ( checkEmptySecondaryText( false ) )
			lineErrorFlags |= EmptySecondaryText;

	if ( errorFlagsToCheck & OverlapsWithNext )
		if ( checkOverlapsWithNext( false ) )
			lineErrorFlags |= OverlapsWithNext;

	if ( errorFlagsToCheck & MaxDuration )
		if ( checkMaxDuration( maxDurationMsecs, false ) )
			lineErrorFlags |= MaxDuration;

	if ( errorFlagsToCheck & MinDuration )
		if ( checkMinDuration( minDurationMsecs, false ) )
			lineErrorFlags |= MinDuration;

	if ( errorFlagsToCheck & MaxDurationPerPrimaryChar )
		if ( checkMaxDurationPerPrimaryChar( maxMsecsPerChar, false ) )
			lineErrorFlags |= MaxDurationPerPrimaryChar;

	if ( errorFlagsToCheck & MaxDurationPerSecondaryChar )
		if ( checkMaxDurationPerSecondaryChar( maxMsecsPerChar, false ) )
			lineErrorFlags |= MaxDurationPerSecondaryChar;

	if ( errorFlagsToCheck & MinDurationPerPrimaryChar )
		if ( checkMinDurationPerPrimaryChar( minMsecsPerChar, false ) )
			lineErrorFlags |= MinDurationPerPrimaryChar;

	if ( errorFlagsToCheck & MinDurationPerSecondaryChar )
		if ( checkMinDurationPerSecondaryChar( minMsecsPerChar, false ) )
			lineErrorFlags |= MinDurationPerSecondaryChar;

	if ( errorFlagsToCheck & MaxPrimaryChars )
		if ( checkMaxPrimaryChars( maxChars, false ) )
			lineErrorFlags |= MaxPrimaryChars;

	if ( errorFlagsToCheck & MaxSecondaryChars )
		if ( checkMaxSecondaryChars( maxChars, false ) )
			lineErrorFlags |= MaxSecondaryChars;

	if ( errorFlagsToCheck & MaxPrimaryLines )
		if ( checkMaxPrimaryLines( maxLines, false ) )
			lineErrorFlags |= MaxPrimaryLines;

	if ( errorFlagsToCheck & MaxSecondaryLines )
		if ( checkMaxSecondaryLines( maxLines, false ) )
			lineErrorFlags |= MaxSecondaryLines;

	if ( errorFlagsToCheck & MaxSecondaryLines )
		if ( checkMaxSecondaryLines( maxLines, false ) )
			lineErrorFlags |= MaxSecondaryLines;

	if ( errorFlagsToCheck & PrimaryUnneededSpaces )
		if ( checkPrimaryUnneededSpaces( false ) )
			lineErrorFlags |= PrimaryUnneededSpaces;

	if ( errorFlagsToCheck & SecondaryUnneededSpaces )
		if ( checkSecondaryUnneededSpaces( false ) )
			lineErrorFlags |= SecondaryUnneededSpaces;

	if ( errorFlagsToCheck & PrimaryCapitalAfterEllipsis )
		if ( checkPrimaryCapitalAfterEllipsis( false ) )
			lineErrorFlags |= PrimaryCapitalAfterEllipsis;

	if ( errorFlagsToCheck & SecondaryCapitalAfterEllipsis )
		if ( checkSecondaryCapitalAfterEllipsis( false ) )
			lineErrorFlags |= SecondaryCapitalAfterEllipsis;

	if ( errorFlagsToCheck & PrimaryUnneededDash )
		if ( checkPrimaryUnneededDash( false ) )
			lineErrorFlags |= PrimaryUnneededDash;

	if ( errorFlagsToCheck & SecondaryUnneededDash )
		if ( checkSecondaryUnneededDash( false ) )
			lineErrorFlags |= SecondaryUnneededDash;

	if ( update )
		setErrorFlags( lineErrorFlags );

	return lineErrorFlags;
}

void SubtitleLine::processAction( Action* action )
{
	if ( m_subtitle )
		m_subtitle->processAction( action );
	else
	{
		action->redo();
		delete action;
	}
}
