#ifndef SUBTITLELINE_H
#define SUBTITLELINE_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "action.h"
#include "sctime.h"
#include "formatdata.h"

#include <qobject.h>
#include <qstring.h>

namespace SubtitleComposer
{
	class Subtitle;

	class SubtitleLine : public QObject
	{
		Q_OBJECT

		friend class Subtitle;
		friend class SubtitleAction;
		friend class SwapLinesTextsAction;
		friend class SubtitleLineAction;
		friend class SetLinePrimaryTextAction;
		friend class SetLineSecondaryTextAction;
		friend class SetLineTextsAction;
		friend class SetLineShowTimeAction;
		friend class SetLineHideTimeAction;
		friend class SetLineTimesAction;
		friend class SetLineStyleFlagsAction;
		friend class SetLineErrorsAction;
		friend class ToggleLineMarkedAction;

		friend class Format;

		public:

			typedef enum {
				Primary,
				Secondary,
				Both,
				OpModeUNKNOWN
			} OpMode;

			typedef enum {
				ItalicID = 0,
				BoldID,
				UnderlineID,
				StyleSIZE,
				StyleUNKNOWN
			} StyleID;

			typedef enum {
				Italic =		0x1 << ItalicID,
				Bold =			0x1 << BoldID,
				Underline = 	0x1 << UnderlineID,
				AllStyles =		Italic|Bold|Underline
			} StyleFlag;

			typedef enum {
				EmptyPrimaryTextID = 0,				// Empty primary text
				EmptySecondaryTextID,				// Empty secondary text
				MaxPrimaryCharsID,					// Too many characters in primary text
				MaxSecondaryCharsID,				// Too many characters in secondary text
				MaxPrimaryLinesID,					// Too many line breaks in primary text
				MaxSecondaryLinesID,				// Too many line breaks in secondary text
				PrimaryUnneededSpacesID,			// Unnecessary spaces in primary text
				SecondaryUnneededSpacesID,			// Unnecessary spaces in secondary text
				PrimaryUnneededDashID,				// Unnecessary dash (-) in primary text
				SecondaryUnneededDashID,			// Unnecessary dash (-) in secondary text
				PrimaryCapitalAfterEllipsisID,		// Capital letter after ellipsis in primary text
				SecondaryCapitalAfterEllipsisID,	// Capital letter after ellipsis in secondary text
				MinDurationPerPrimaryCharID	,		// Too short duration per primary text character
				MinDurationPerSecondaryCharID,		// Too short duration per secondary text character
				MaxDurationPerPrimaryCharID	,		// Too long duration per primary text character
				MaxDurationPerSecondaryCharID,		// Too long duration per secondary text character
				MinDurationID,						// Too short duration
				MaxDurationID,						// Too long duration
				OverlapsWithNextID,					// Overlaps with next line
				ErrorSIZE,
				ErrorUNKNOWN
			} ErrorID;

			typedef enum {
				EmptyPrimaryText =					0x1 << EmptyPrimaryTextID,
				EmptySecondaryText =				0x1 << EmptySecondaryTextID,
				MaxPrimaryChars =					0x1 << MaxPrimaryCharsID,
				MaxSecondaryChars =					0x1 << MaxSecondaryCharsID,
				MaxPrimaryLines =					0x1 << MaxPrimaryLinesID,
				MaxSecondaryLines =					0x1 << MaxSecondaryLinesID,
				PrimaryUnneededSpaces =				0x1 << PrimaryUnneededSpacesID,
				SecondaryUnneededSpaces =			0x1 << SecondaryUnneededSpacesID,
				PrimaryUnneededDash =				0x1 << PrimaryUnneededDashID,
				SecondaryUnneededDash =				0x1 << SecondaryUnneededDashID,
				PrimaryCapitalAfterEllipsis =		0x1 << PrimaryCapitalAfterEllipsisID,
				SecondaryCapitalAfterEllipsis =		0x1 << SecondaryCapitalAfterEllipsisID,
				MinDurationPerPrimaryChar =			0x1 << MinDurationPerPrimaryCharID,
				MinDurationPerSecondaryChar =		0x1 << MinDurationPerSecondaryCharID,
				MaxDurationPerPrimaryChar =			0x1 << MaxDurationPerPrimaryCharID,
				MaxDurationPerSecondaryChar =		0x1 << MaxDurationPerSecondaryCharID,
				MinDuration =						0x1 << MinDurationID,
				MaxDuration =						0x1 << MaxDurationID,
				OverlapsWithNext =					0x1 << OverlapsWithNextID,

				PrimaryErrors =   EmptyPrimaryText|MaxPrimaryChars|MaxPrimaryLines|PrimaryUnneededSpaces|
								  PrimaryUnneededDash|PrimaryCapitalAfterEllipsis|MinDurationPerPrimaryChar|
								  MaxDurationPerPrimaryChar,
				SecondaryErrors = EmptySecondaryText|MaxSecondaryChars|MaxSecondaryLines|SecondaryUnneededSpaces|
								  SecondaryUnneededDash|SecondaryCapitalAfterEllipsis|MinDurationPerSecondaryChar|
								  MaxDurationPerSecondaryChar,
				OtherErrors =     MinDuration|MaxDuration,

				TimesErrors =     MinDuration|MaxDuration|MinDurationPerPrimaryChar|MinDurationPerSecondaryChar|
							      MaxDurationPerPrimaryChar|MaxDurationPerSecondaryChar|OverlapsWithNext,

				AllErrors =       PrimaryErrors|SecondaryErrors|OtherErrors,

			} ErrorFlag;

			static ErrorFlag errorFlag( ErrorID id );
			static ErrorID errorID( ErrorFlag flag );

			static const QString& errorText( SubtitleLine::ErrorID errorID, bool full=false );
			static const QString& errorText( SubtitleLine::ErrorFlag errorFlag, bool full=false );

			SubtitleLine( const QString& primaryText=QString::null, const QString& secondaryText=QString::null );
			SubtitleLine( const QString& pText, const QString& sText, const Time& showTime, const Time& hideTime, int styles=0 );
			SubtitleLine( const SubtitleLine& line );
			SubtitleLine& operator=( const SubtitleLine& line );
			virtual ~SubtitleLine();

			unsigned number() const;
			int index() const;

			Subtitle* subtitle();
			const Subtitle* subtitle() const;

			SubtitleLine* prevLine();
			SubtitleLine* nextLine();

			QString primaryText() const;
			void setPrimaryText( const QString& text );

			QString secondaryText() const;
			void setSecondaryText( const QString& text );

			void setTexts( const QString& primaryText, const QString& secondaryText );

			static QString fixPunctuation( const QString& text, bool spaces, bool quotes, bool engI, bool ellipsis, bool*cont );
			static QString titleCase( const QString& text, bool lowerFirst );
			static QString sentenceCase( const QString& text, bool lowerFirst, bool* continues=0 );
			static QString adjustText( const QString& text, unsigned minLengthForBreak );

			void adjustText( unsigned minLengthForBreak, OpMode mode=Primary );
			void unbreakText( OpMode mode=Primary );

			static QString simplifySpaces( QString text );
			void simplifySpaces( OpMode mode=Primary );

			Time showTime() const;
			void setShowTime( const Time& showTime );

			Time hideTime() const;
			void setHideTime( const Time& hideTime );

			Time durationTime() const;
			void setDurationTime( const Time& durationTime );

			unsigned primaryCharacters();
			unsigned primaryWords();
			unsigned primaryLines();

			unsigned secondaryCharacters();
			unsigned secondaryWords();
			unsigned secondaryLines();

			static Time autoDuration( const QString& text, unsigned charMsecs, unsigned wordMsecs, unsigned lineMsecs );
			Time autoDuration( unsigned charMsecs, unsigned wordMsecs, unsigned lineMsecs, OpMode mode=Primary );

			void setTimes( const Time& showTime, const Time& hideTime );

			void shiftTimes( long mseconds );
			void adjustTimes( long shiftMseconds, double scaleFactor );

			int styleFlags() const;
			void setStyleFlags( int styleFlags );
			void setStyleFlags( int styleFlags, bool value );

			bool marked() const;
			void setMarked( bool marked );

			int errorFlags() const;
			void setErrorFlags( int errorFlags );
			void setErrorFlags( int errorFlags, bool value );
			bool checkEmptyPrimaryText( bool update=true );
			bool checkEmptySecondaryText( bool update=true );
			bool checkOverlapsWithNext( bool update=true );
			bool checkMaxDuration( unsigned maxMsecs, bool update=true );
			bool checkMinDuration( unsigned minMsecs, bool update=true );
			bool checkMaxDurationPerPrimaryChar( unsigned maxMsecsPerChar, bool update=true );
			bool checkMaxDurationPerSecondaryChar( unsigned maxMsecsPerChar, bool update=true );
			bool checkMinDurationPerPrimaryChar( unsigned minMsecsPerChar, bool update=true );
			bool checkMinDurationPerSecondaryChar( unsigned minMsecsPerChar, bool update=true );
			bool checkMaxPrimaryChars( unsigned maxCharacters, bool update=true );
			bool checkMaxSecondaryChars( unsigned maxCharacters, bool update=true );
			bool checkMaxPrimaryLines( unsigned maxLines, bool update=true );
			bool checkMaxSecondaryLines( unsigned maxLines, bool update=true );
			bool checkPrimaryUnneededSpaces( bool update=true );
			bool checkSecondaryUnneededSpaces( bool update=true );
			bool checkPrimaryCapitalAfterEllipsis( bool update=true );
			bool checkSecondaryCapitalAfterEllipsis( bool update=true );
			bool checkPrimaryUnneededDash( bool update=true );
			bool checkSecondaryUnneededDash( bool update=true );
			int check( int errorFlagsToCheck, int minDurationMsecs, int maxDurationMsecs, int minMsecsPerChar, int maxMsecsPerChar, int maxChars, int maxLines, bool update=true );

		signals:

			void primaryTextChanged( const QString& text );
			void secondaryTextChanged( const QString& text );
			void showTimeChanged( const Time& showTime );
			void hideTimeChanged( const Time& hideTime );
			void styleFlagsChanged( int styleFlags );
			void errorFlagsChanged( int errorFlags );
			void markChanged( bool marked );

		private:

			FormatData* const formatData() const;
			void setFormatData( const FormatData* formatData );

			void processAction( Action* action );

		private:

			Subtitle* m_subtitle;
			QString m_primaryText;
			QString m_secondaryText;
			Time m_showTime;
			Time m_hideTime;
			int m_styleFlags;
			int m_errorFlags;
			bool m_marked;

			FormatData* m_formatData;
	};
}

#endif
