/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "formatsmanager.h"

#include "microdvd/microdvdinputformat.h"
#include "microdvd/microdvdoutputformat.h"
#include "mplayer/mplayerinputformat.h"
#include "mplayer/mplayeroutputformat.h"
#include "mplayer2/mplayer2inputformat.h"
#include "mplayer2/mplayer2outputformat.h"
#include "subrip/subripinputformat.h"
#include "subrip/subripoutputformat.h"
#include "substationalpha/substationalphainputformat.h"
#include "substationalpha/substationalphaoutputformat.h"
#include "subviewer1/subviewer1inputformat.h"
#include "subviewer1/subviewer1outputformat.h"
#include "subviewer2/subviewer2inputformat.h"
#include "subviewer2/subviewer2outputformat.h"
#include "tmplayer/tmplayerinputformat.h"
#include "tmplayer/tmplayeroutputformat.h"

using namespace SubtitleComposer;

QPtrList<InputFormat> FormatsManager::s_inputFormats;
QPtrList<OutputFormat> FormatsManager::s_outputFormats;
bool FormatsManager::s_loaded( false );

void FormatsManager::loadFormats( bool force )
{
	if ( force )
	{
		if ( s_loaded )
		{
			s_inputFormats.clear();
			s_outputFormats.clear();
			s_loaded = false;
		}
	}
	else if ( s_loaded )
		return;

	s_inputFormats.append( new SubRipInputFormat() );
	s_inputFormats.append( new MicroDVDInputFormat() );
	s_inputFormats.append( new MPlayerInputFormat() );
	s_inputFormats.append( new MPlayer2InputFormat() );
	s_inputFormats.append( new SubStationAlphaInputFormat() );
	s_inputFormats.append( new AdvancedSubStationAlphaInputFormat() );
	s_inputFormats.append( new SubViewer1InputFormat() );
	s_inputFormats.append( new SubViewer2InputFormat() );
	s_inputFormats.append( new TMPlayerInputFormat() );
	s_inputFormats.append( new TMPlayerPlusInputFormat() );
	s_inputFormats.setAutoDelete( true );

	s_outputFormats.append( new SubRipOutputFormat() );
	s_outputFormats.append( new MicroDVDOutputFormat() );
	s_outputFormats.append( new MPlayerOutputFormat() );
	s_outputFormats.append( new MPlayer2OutputFormat() );
	s_outputFormats.append( new SubStationAlphaOutputFormat() );
	s_outputFormats.append( new AdvancedSubStationAlphaOutputFormat() );
	s_outputFormats.append( new SubViewer1OutputFormat() );
	s_outputFormats.append( new SubViewer2OutputFormat() );
	s_outputFormats.append( new TMPlayerOutputFormat() );
	s_outputFormats.append( new TMPlayerPlusOutputFormat() );
	s_outputFormats.setAutoDelete( true );

	s_loaded = true;
}

QStringList FormatsManager::inputFormatNames()
{
	loadFormats();

	QStringList ret;

	for ( QPtrListIterator<InputFormat> it( s_inputFormats ); it.current() != 0; ++it )
		ret.append( (*it)->name() );

	return ret;
}

QStringList FormatsManager::outputFormatNames()
{
	loadFormats();

	QStringList ret;

	for ( QPtrListIterator<OutputFormat> it( s_outputFormats ); it.current() != 0; ++it )
		ret.append( (*it)->name() );

	return ret;
}

bool FormatsManager::hasInputFormat( const QString& name )
{
	loadFormats();

	for ( QPtrListIterator<InputFormat> it( s_inputFormats ); it.current() != 0; ++it )
		if ( (*it)->name() == name )
			return true;

	return false;
}

bool FormatsManager::hasOutputFormat( const QString& name )
{
	loadFormats();

	for ( QPtrListIterator<OutputFormat> it( s_outputFormats ); it.current() != 0; ++it )
		if ( (*it)->name() == name )
			return true;

	return false;
}

const InputFormat* FormatsManager::inputFormat( const QString& name )
{
	loadFormats();

	for ( QPtrListIterator<InputFormat> it( s_inputFormats ); it.current() != 0; ++it )
		if ( (*it)->name() == name )
			return *it;

	return 0;
}

const OutputFormat* FormatsManager::outputFormat( const QString& name )
{
	loadFormats();

	for ( QPtrListIterator<OutputFormat> it( s_outputFormats ); it.current() != 0; ++it )
		if ( (*it)->name() == name )
			return *it;

	return 0;
}

bool FormatsManager::readSubtitle( Subtitle& subtitle, bool primary, const QString& path, QTextCodec* codec, Format::NewLine* newLine, QString* formatName )
{
	loadFormats();

	QString extension = QFileInfo( path ).extension( false );

	// attempt to parse subtitles based on extension information
	for ( QPtrListIterator<InputFormat> it( s_inputFormats ); it.current() != 0; ++it )
	{
		if ( (*it)->knowsExtension( extension ) )
		{
			if ( (*it)->readSubtitle( subtitle, newLine, primary, path, codec, true ) )
			{
				if ( formatName )
					*formatName = (*it)->name();
				return true;
			}
		}
	}

	// that didn't worked, attempt to parse subtitles based on content
	for ( QPtrListIterator<InputFormat> it( s_inputFormats ); it.current() != 0; ++it )
	{
		if ( ! (*it)->knowsExtension( extension ) )
		{
			if ( (*it)->readSubtitle( subtitle, newLine, primary, path, codec, true ) )
			{
				if ( formatName )
					*formatName = (*it)->name();
				return true;
			}
		}
	}

	return false;
}

bool FormatsManager::writeSubtitle( const Subtitle& subtitle, bool primary, const QString& path, QTextCodec* codec, Format::NewLine newLine, const QString& formatName, bool overwrite )
{
	loadFormats();

	const OutputFormat* format = outputFormat( formatName );
	if ( format == 0 )
	{
		QString extension = QFileInfo( path ).extension( false );
		// attempt find format based on extension information
		for ( QPtrListIterator<OutputFormat> it( s_outputFormats ); it.current() != 0; ++it )
			if ( (*it)->knowsExtension( extension ) )
			{
				format = *it;
				break;
			}
	}

	if ( format == 0 )
		return false;

	return format->writeSubtitle( subtitle, newLine, primary, path, codec, overwrite );
}
