#ifndef SUBSTATIONALPHAINPUTFORMAT_H
#define SUBSTATIONALPHAINPUTFORMAT_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <inputformat.h>
#include <qregexp.h>
#include <qmap.h>

namespace SubtitleComposer
{
	class SubStationAlphaInputFormat : public InputFormat
	{
		friend class FormatsManager;
		friend class AdvancedSubStationAlphaInputFormat;

		public:

			~SubStationAlphaInputFormat() {}

			virtual bool parseSubtitles( Subtitle& subtitle, bool primary, const QString& data ) const
			{
				if ( m_scriptInfoRegExp.search( data ) == -1 )
					return false;

				int stylesStart = m_stylesRegExp.search( data );
				if ( stylesStart == -1 )
					return false;

				FormatData formatData = createFormatData();

				formatData.setValue( "ScriptInfo", data.mid( 0, stylesStart - 1 ) );

				int eventsStart = m_eventsRegExp.search( data, stylesStart );
				if ( eventsStart == -1 )
					return false;

				formatData.setValue( "Styles", data.mid( stylesStart, eventsStart - stylesStart ) );

				if ( m_formatRegExp.search( data, eventsStart ) == -1 )
					return false;

				if ( primary )
				{
					setFormatData( subtitle, formatData );
					formatData.clear();
				}

				unsigned readLines = 0;

				int offset = m_formatRegExp.pos() + m_formatRegExp.matchedLength();
				for ( ; m_dialogueRegExp.search( data, offset ) != -1; offset += m_dialogueRegExp.matchedLength() )
				{
					if ( m_timeRegExp.search( m_dialogueRegExp.cap( 1 ) ) == -1 )
						continue;

					Time showTime(
						m_timeRegExp.cap( 1 ).toInt(),
						m_timeRegExp.cap( 2 ).toInt(),
						m_timeRegExp.cap( 3 ).toInt(),
						m_timeRegExp.cap( 4 ).toInt() * 10
					);

					if ( m_timeRegExp.search( m_dialogueRegExp.cap( 2 ) ) == -1 )
						continue;

					Time hideTime(
						m_timeRegExp.cap( 1 ).toInt(),
						m_timeRegExp.cap( 2 ).toInt(),
						m_timeRegExp.cap( 3 ).toInt(),
						m_timeRegExp.cap( 4 ).toInt() * 10
					);

					QString text( m_dialogueRegExp.cap( 3 ) );

					int styleFlags = 0;
					if ( text.contains( m_boldRegExp ) )
						styleFlags |= SubtitleLine::Bold;
					if ( text.contains( m_italicRegExp ) )
						styleFlags |= SubtitleLine::Italic;

					text.replace( "\\N", "\n" );
					text.replace( "\\n", "\n" );
					text.replace( m_boldRegExp, "" );
					text.replace( m_italicRegExp, "" );
					//text.replace( m_commandsRegExp, "" );
					text = text.stripWhiteSpace();

					SubtitleLine* line = setLine( subtitle, readLines, primary, text, showTime, hideTime, styleFlags );
					if ( primary )
					{
						formatData.setValue(
							"Dialogue",
							m_dialogueRegExp.cap( 0 ).replace( m_dialogue2RegExp, "\\1%1\\2%2\\3%3\n" )
						);

						setFormatData( line, formatData );
					}

					readLines++;
				}

				return readLines > 0;
			}

		protected:

			SubStationAlphaInputFormat():
				InputFormat( "SubStation Alpha", "ssa" ),
				m_scriptInfoRegExp( "^ *\\[Script Info\\] *\n" ),
				m_stylesRegExp( s_stylesRegExp ),
				m_eventsRegExp( "\n\\ *\\[Events\\] *\n" ),
				m_formatRegExp( s_formatRegExp ),
				m_dialogueRegExp( s_dialogueRegExp ),
				m_dialogue2RegExp( s_dialogue2RegExp ),
				m_timeRegExp( "([0-9]):([0-5][0-9]):([0-5][0-9]).([0-9][0-9])" ),
				m_boldRegExp( "\\{\\\\b[10]\\}" ),
				m_italicRegExp( "\\{\\\\i[10]\\}" ),
				m_commandsRegExp( "\\{\\\\[a-zA-Z0-9]+\\( *[a-zA-Z0-9]+ *(, *[a-zA-Z0-9]+){0,} *\\)\\}" )
			{
			}

			SubStationAlphaInputFormat( const QString& name, const QStringList& extensions, const QString& stylesRegExp, const QString& formatRegExp ):
				InputFormat( name, extensions ),
				m_scriptInfoRegExp( "^ *\\[Script Info\\] *\n" ),
				m_stylesRegExp( stylesRegExp ),
				m_eventsRegExp( "\n\\ *\\[Events\\] *\n" ),
				m_formatRegExp( formatRegExp ),
				m_dialogueRegExp( s_dialogueRegExp ),
				m_dialogue2RegExp( s_dialogue2RegExp ),
				m_timeRegExp( "([0-9]):([0-5][0-9]):([0-5][0-9]).([0-9][0-9])" ),
				m_boldRegExp( "\\{\\\\b[10]\\}" ),
				m_italicRegExp( "\\{\\\\i[10]\\}" ),
				m_commandsRegExp( "\\{\\\\[a-zA-Z0-9]+\\( *[a-zA-Z0-9]+ *(, *[a-zA-Z0-9]+){0,} *\\)\\}" )
			{
			}

			mutable QRegExp m_scriptInfoRegExp;
			mutable QRegExp m_stylesRegExp;
			mutable QRegExp m_eventsRegExp;
			mutable QRegExp m_formatRegExp;
			mutable QRegExp m_dialogueRegExp;
			mutable QRegExp m_dialogue2RegExp;
			mutable QRegExp m_timeRegExp;
			mutable QRegExp m_boldRegExp;
			mutable QRegExp m_italicRegExp;
			mutable QRegExp m_commandsRegExp;

			static const char* s_stylesRegExp;
			static const char* s_formatRegExp;
			static const char* s_dialogueRegExp;
			static const char* s_dialogue2RegExp;
	};

	const char* SubStationAlphaInputFormat::s_stylesRegExp = "\n\\ *\\[V4 Styles\\] *\n";

	const char* SubStationAlphaInputFormat::s_formatRegExp =
		" *Format: *Marked, *Start, *End, *Style, *Name, *MarginL, *MarginR, *MarginV, *Effect, *Text *\n";

	const char* SubStationAlphaInputFormat::s_dialogueRegExp =
		" *Dialogue: *[^,]+, *([^,]+), *([^,]+), *[^,]+, *[^,]*, *[0-9]{4,4}, *[0-9]{4,4}, *[0-9]{4,4}, *[^,]*, *([^\n]*)\n?";

	const char* SubStationAlphaInputFormat::s_dialogue2RegExp =
		" *(Dialogue: *[^,]+, *)[^,]+(, *)[^,]+(, *[^,]+, *[^,]*, *[0-9]{4,4}, *[0-9]{4,4}, *[0-9]{4,4}, *[^,]*, *).*";

	class AdvancedSubStationAlphaInputFormat : public SubStationAlphaInputFormat
	{
		friend class FormatsManager;

		protected:

			AdvancedSubStationAlphaInputFormat():
				SubStationAlphaInputFormat( "Advanced SubStation Alpha", "ass", s_stylesRegExp, s_formatRegExp )
			{
			}

			static const char* s_stylesRegExp;
			static const char* s_formatRegExp;
	};

	const char* AdvancedSubStationAlphaInputFormat::s_stylesRegExp = "\n\\ *\\[V4\\+ Styles\\] *\n";

	const char* AdvancedSubStationAlphaInputFormat::s_formatRegExp =
		" *Format: *Layer, *Start, *End, *Style, *(Actor|Name), *MarginL, *MarginR, *MarginV, *Effect, *Text *\n";
// Format: Layer, Start, End, Style, Name, MarginL, MarginR, MarginV, Effect, Text
}

#endif

