/**************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "application.h"
#include "mainwindow.h"
#include "audiolevelswidget.h"
#include "playerwidget.h"
#include "lineswidget.h"
#include "currentlinewidget.h"
#include "configdialog.h"
#include "errorswidget.h"
#include "errorsdialog.h"
#include "useractions/useraction.h"
#include "useractions/useractionnames.h"
#include "useractions/kactionext.h"
#include "useractions/ktoggleactionext.h"
#include "useractions/krecentfilesactionext.h"
#include "configdefs/generalconfigdefs.h"
#include "configdefs/playerconfigdefs.h"
#include "configdefs/errorsconfigdefs.h"
#include "dialogs/opensubtitledialog.h"
#include "dialogs/savesubtitledialog.h"
#include "dialogs/joinsubtitlesdialog.h"
#include "dialogs/splitsubtitledialog.h"
#include "dialogs/gotolinedialog.h"
#include "dialogs/targetablelinesactiondialog.h"
#include "dialogs/shifttimesdialog.h"
#include "dialogs/adjusttimesdialog.h"
#include "dialogs/durationlimitsdialog.h"
#include "dialogs/autodurationsdialog.h"
#include "dialogs/changetextscasedialog.h"
#include "dialogs/fixoverlappingtimesdialog.h"
#include "dialogs/fixpunctuationdialog.h"
#include "dialogs/smarttextsadjustdialog.h"
#include "dialogs/changeframeratedialog.h"
#include "dialogs/syncsubtitlesdialog.h"
#include "dialogs/checkerrorsdialog.h"
#include "dialogs/clearerrorsdialog.h"
#include "dialogs/insertlinedialog.h"
#include "dialogs/removelinesdialog.h"
#include "utils/find.h"
#include "utils/replace.h"
#include "utils/finderror.h"
#include "utils/spelling.h"
#include <formatsmanager.h>
#include <player.h>
#include <playerbackend.h>
#include <commondefs.h>

#include <qprocess.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qtextcodec.h>
#include <qlayout.h>

#include <kaboutdata.h>
#include <kcharsets.h>
#include <kmessagebox.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kaction.h>
#include <kaccel.h>
#include <kstdaction.h>
#include <kconfig.h>
#include <kcombobox.h>

using namespace SubtitleComposer;

Application* SubtitleComposer::app()
{
	return Application::instance();
}

Application::Application():
	KApplication(),
	m_config(),
	m_subtitle( 0 ),
	m_subtitlePath(),
	m_subtitleFileName(),
	m_subtitleEncoding(),
	m_subtitleEOL( -1 ),
	m_subtitleFormat(),
	m_translationMode( false ),
	m_subtitleTrPath(),
	m_subtitleTrFileName(),
	m_subtitleTrEncoding(),
	m_subtitleTrEOL( -1 ),
	m_subtitleTrFormat(),
	m_player( Player::instance() ),
	m_lastFoundLine( 0 ),
	m_audiolevels( 0 ),
	m_lastSubtitleDir( QDir::homeDirPath() ),
	m_lastVideoDir( QDir::homeDirPath() ),
	m_linkCurrentLineToPosition( false )
{
	// The config object has to be filled first with all the groups and default data...
	m_config.setGroup( AppConfigGroup( "General", generalConfigDefaults() ) );
	m_config.setGroup( AppConfigGroup( "Errors", errorsConfigDefaults() ) );
	m_config.setGroup( AppConfigGroup( "Player", playerConfigDefaults() ) );
	QStringList backendNames( m_player->backendNames() );
	for ( QStringList::ConstIterator it = backendNames.begin(), end = backendNames.end(); it != end; ++it )
		m_config.setGroup( m_player->backend( *it )->configGroup() );

	// ...only then can be loaded
	m_config.readFrom( KGlobal::config() );

	// and feeded to the player backends
	for ( QStringList::ConstIterator it = backendNames.begin(), end = backendNames.end(); it != end; ++it )
		m_player->backend( *it )->setConfigGroup( *m_config.group( *it ) );

	m_mainWindow = new MainWindow();

	m_audiolevelsWidget = m_mainWindow->m_audiolevelsWidget;
	m_playerWidget = m_mainWindow->m_playerWidget;
	m_linesWidget = m_mainWindow->m_linesWidget;
	m_curLineWidget = m_mainWindow->m_curLineWidget;

	m_configDialog = new ConfigDialog( m_config, m_mainWindow );
	m_errorsDialog = new ErrorsDialog( m_mainWindow );

	m_errorsWidget = m_errorsDialog->errorsWidget();

	m_find = new Find( m_linesWidget );
	m_replace = new Replace( m_linesWidget );
	m_findError = new FindError( m_linesWidget );
	m_spelling = new Spelling( m_linesWidget );

	setMainWidget( m_mainWindow );

	connect( playerConfig(), SIGNAL(optionChanged(const QString&,const QString&)),
			 this, SLOT(onPlayerOptionChanged(const QString&,const QString&)) );

	connect( m_player, SIGNAL(fileOpened(const QString&)), this, SLOT(onPlayerFileOpened(const QString&)) );
	connect( m_player, SIGNAL(playing()), this, SLOT(onPlayerPlaying()) );
	connect( m_player, SIGNAL(paused()), this, SLOT(onPlayerPaused()) );
	connect( m_player, SIGNAL(stopped()), this, SLOT(onPlayerStopped()) );
	connect( m_player, SIGNAL(audioStreamsChanged(const QStringList&)),
			 this, SLOT(onPlayerAudioStreamsChanged(const QStringList&)) );
	connect( m_player, SIGNAL(activeAudioStreamChanged(int)), this, SLOT(onPlayerActiveAudioStreamChanged(int)) );
	connect( m_player, SIGNAL(mutedChanged(bool)), this, SLOT(onPlayerMutedChanged(bool)) );

	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), &m_actionManager, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_mainWindow, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_audiolevelsWidget, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_playerWidget, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_linesWidget, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_curLineWidget, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_errorsWidget, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_find, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_replace, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_findError, SLOT( setSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_spelling, SLOT( setSubtitle(Subtitle*) ) );

	connect( this, SIGNAL( subtitleClosed() ), &m_actionManager, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_mainWindow, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_audiolevelsWidget, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_playerWidget, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_linesWidget, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_curLineWidget, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_errorsWidget, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_find, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_replace, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_findError, SLOT( setSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_spelling, SLOT( setSubtitle() ) );

	connect( this, SIGNAL( translationModeChanged(bool) ), &m_actionManager, SLOT( setTranslationMode(bool) ) );
	connect( this, SIGNAL( translationModeChanged(bool) ), m_playerWidget, SLOT( setTranslationMode(bool) ) );
	connect( this, SIGNAL( translationModeChanged(bool) ), m_linesWidget, SLOT( setTranslationMode(bool) ) );
	connect( this, SIGNAL( translationModeChanged(bool) ), m_curLineWidget, SLOT( setTranslationMode(bool) ) );
	connect( this, SIGNAL( translationModeChanged(bool) ), m_find, SLOT( setTranslationMode(bool) ) );
	connect( this, SIGNAL( translationModeChanged(bool) ), m_replace, SLOT( setTranslationMode(bool) ) );
	connect( this, SIGNAL( translationModeChanged(bool) ), m_spelling, SLOT( setTranslationMode(bool) ) );

	connect( this, SIGNAL( fullScreenModeChanged(bool) ), &m_actionManager, SLOT( setFullScreenMode(bool) ) );


	connect( this, SIGNAL( audiolevelsOpened(AudioLevels*) ), &m_actionManager, SLOT( setAudioLevels(AudioLevels*) ) );
	connect( this, SIGNAL( audiolevelsOpened(AudioLevels*) ), m_audiolevelsWidget, SLOT( setAudioLevels(AudioLevels*) ) );

	connect( this, SIGNAL( audiolevelsClosed() ), &m_actionManager, SLOT( setAudioLevels() ) );
	connect( this, SIGNAL( audiolevelsClosed() ), m_audiolevelsWidget, SLOT( setAudioLevels() ) );


	connect( m_configDialog, SIGNAL( accepted() ), this, SLOT( updateConfigFromDialog() ) );

	connect( m_linesWidget, SIGNAL( currentLineChanged(SubtitleLine*) ),
			 m_curLineWidget, SLOT( setCurrentLine(SubtitleLine*)) );
	connect( m_linesWidget, SIGNAL( lineDoubleClicked(SubtitleLine*) ),
			 this, SLOT( onLineDoubleClicked(SubtitleLine*) ) );
	connect( m_linesWidget, SIGNAL(currentLineChanged(SubtitleLine*)),
			 m_errorsWidget, SLOT(setCurrentLine(SubtitleLine*)) );

	connect( m_errorsWidget, SIGNAL(lineDoubleClicked(SubtitleLine*)),
			 m_linesWidget, SLOT(setCurrentLine(SubtitleLine*)) );

	connect( m_playerWidget, SIGNAL(playingLineChanged(SubtitleLine*)),
			 this, SLOT(onPlayingLineChanged(SubtitleLine*)) );

	connect( m_find, SIGNAL( found(SubtitleLine*,bool,int,int) ),
			 this, SLOT( onHighlightLine(SubtitleLine*,bool,int,int) ) );
	connect( m_replace, SIGNAL( found(SubtitleLine*,bool,int,int) ),
			 this, SLOT( onHighlightLine(SubtitleLine*,bool,int,int) ) );
	connect( m_findError, SIGNAL( found(SubtitleLine*) ),
			 this, SLOT( onHighlightLine(SubtitleLine*) ) );
	connect( m_spelling, SIGNAL( mispelled(SubtitleLine*,bool,int,int) ),
			 this, SLOT( onHighlightLine(SubtitleLine*,bool,int,int) ) );

	m_actionManager.setActionCollection( m_mainWindow->actionCollection() );
	m_actionManager.setLinesWidget( m_linesWidget );
	m_actionManager.setPlayer( m_player );
	m_actionManager.setFullScreenMode( false );

	setupActions();

	m_playerWidget->plugActions();

	m_mainWindow->setupGUI();

	loadConfig();
}

Application::~Application()
{
// 	saveConfig();
// 	delete m_mainWindow;
// 	delete m_subtitle;

	delete m_audiolevels;
}

Application* Application::instance()
{
	return static_cast<Application*>( kapp );
}

void Application::loadConfig()
{
	KConfigGroupSaver saver( config(), "Application Settings" );

	m_lastSubtitleDir = config()->readPathEntry( "LastSubtitleDir", QDir::homeDirPath() );
	m_recentSubtitlesAction->loadEntries( config(), "Recent Subtitles" );
	m_recentTrSubtitlesAction->loadEntries( config(), "Recent Translation Subtitles" );

	m_lastAudioLevelsDir = config()->readPathEntry( "LastAudioLevelsDir", QDir::homeDirPath() );
	m_recentAudioLevelssAction->loadEntries( config(), "Recent AudioLevelss" );

	m_lastVideoDir = config()->readPathEntry( "LastVideoDir", QDir::homeDirPath() );
	m_recentVideosAction->loadEntries( config(), "Recent Videos" );

	m_player->setMuted( config()->readBoolEntry( "Muted", false ) );
	m_player->setVolume( config()->readDoubleNumEntry( "Volume", 100.0 ) );

	((KToggleActionExt*)action( ACT_TOGGLE_MUTED ))->setChecked( m_player->isMuted() );

	m_mainWindow->loadConfig();
	m_audiolevelsWidget->loadConfig();
	m_playerWidget->loadConfig();
	m_linesWidget->loadConfig();
	m_curLineWidget->loadConfig();
}

void Application::saveConfig()
{
	KConfigGroupSaver saver( config(), "Application Settings" );

	config()->writePathEntry( "LastSubtitleDir", m_lastSubtitleDir );
	m_recentSubtitlesAction->saveEntries( config(), "Recent Subtitles" );
	m_recentTrSubtitlesAction->saveEntries( config(), "Recent Translation Subtitles" );

	config()->writePathEntry( "LastAudioLevelsDir", m_lastAudioLevelsDir );
	m_recentAudioLevelssAction->saveEntries( config(), "Recent AudioLevelss" );

	config()->writePathEntry( "LastVideoDir", m_lastVideoDir );
	m_recentVideosAction->saveEntries( config(), "Recent Videos" );

	config()->writeEntry( "Muted", m_player->isMuted() );
	config()->writeEntry( "Volume", m_player->volume() );

	m_mainWindow->saveConfig();
	m_audiolevelsWidget->saveConfig();
	m_playerWidget->saveConfig();
	m_linesWidget->saveConfig();
	m_curLineWidget->saveConfig();
}

AppConfigGroup* Application::generalConfig()
{
	return m_config.group( "General" );
}

AppConfigGroup* Application::errorsConfig()
{
	return m_config.group( "Errors" );
}

AppConfigGroup* Application::playerConfig()
{
	return m_config.group( "Player" );
}

QMap<QString,QString> Application::generalConfigDefaults()
{
	QMap<QString,QString> defaults;

// 	defaults[OPT_SPELL_CHECK_DICT] = KSpellConfig().dictionary();
	defaults[OPT_DEFAULT_SUBS_ENCODING] = KGlobal::locale()->codecForEncoding()->name();
	defaults[OPT_JUMP_TO_POSITION] = "1500";
	defaults[OPT_AUTO_VIDEO_LOAD] = "true";
	defaults[OPT_SHIFT_SELECTED_LINES_MSECS] = "100";
	defaults[OPT_VIDEO_POS_COMP_MSECS] = "250";

	return defaults;
}

QMap<QString,QString> Application::errorsConfigDefaults()
{
	QMap<QString,QString> defaults;

	defaults[OPT_MIN_DURATION] = "700";
	defaults[OPT_MAX_DURATION] = "5000";
	defaults[OPT_MIN_DURATION_PER_CHARACTER] = "30";
	defaults[OPT_MAX_DURATION_PER_CHARACTER] = "185";
	defaults[OPT_MAX_CHARACTERS] = "80";
	defaults[OPT_MAX_LINES] = "2";
	defaults[OPT_AUTO_CLEAR_FIXED] = "true";

	return defaults;
}

QMap<QString,QString> Application::playerConfigDefaults()
{
	QMap<QString,QString> defaults;

#ifdef HAVE_GSTREAMER
	defaults[OPT_PLAYER_BACKEND] = "GStreamer";
#else
	defaults[OPT_PLAYER_BACKEND] = "MPlayer";
#endif
	defaults[OPT_JUMP_ON_SEEK_SECS] = "15";

	defaults[OPT_FONT_FAMILY] = "Sans Serif";
	defaults[OPT_FONT_SIZE] = "15";
	defaults[OPT_FONT_COLOR] = Qt::yellow.name();

	return defaults;
}

KAction* Application::action( const char* actionName )
{
	return m_actionManager.actionCollection()->action( actionName );
}

void Application::triggerAction( const KShortcut& shortcut )
{
	QValueList<KAction*> actions( m_actionManager.actionCollection()->actions() );

	for ( QValueListIterator<KAction*> it = actions.begin(), end = actions.end(); it != end; ++it )
		if ( (*it)->shortcut() == shortcut && (*it)->isEnabled() )
		{
			(*it)->activate();
			return;
		}
}

const QStringList& Application::availableEncodingNames() const
{
	static QStringList availableEncodingNames;

	if ( availableEncodingNames.empty() )
	{
		QStringList encodings( KGlobal::charsets()->availableEncodingNames() );
		for ( QStringList::ConstIterator it = encodings.begin(), end = encodings.end(); it != end; ++it )
		{
			bool found = false;
			QTextCodec* codec = KGlobal::charsets()->codecForName( *it, found );
			if ( found )
				availableEncodingNames.append( QString( codec->name() ).upper() );
		}
	}

	return availableEncodingNames;
}

const QString& Application::lastSubtitleDirectory() const
{
	return m_lastSubtitleDir;
}

const QString& Application::buildMediaFilesFilter()
{
	static QString filter;

	if ( filter.isEmpty() )
	{
		QString mediaExtensions;

		QString videoExtensions;
		QStringList videoExts( QStringList::split( ':', "avi:flv:mkv:mov:mpg:mpeg:mp4:wmv:ogm:rmvb:ts:vob" ) );
		for ( QStringList::ConstIterator it = videoExts.begin(), end = videoExts.end(); it != end; ++it )
// 			videoExtensions += " *." + *it /*+ " *." + (*it).upper()*/;
			videoExtensions += " *." + *it + " *." + (*it).upper();
		mediaExtensions += videoExtensions;
		filter += "\n" + videoExtensions.stripWhiteSpace() + "|" + i18n( "Video Files" );

		QString audioExtensions;
		QStringList audioExts( QStringList::split( ':', "aac:ac3:ape:flac:la:m4a:mac:mp2:mp3:mp4:mp+:mpc:mpp:ofr:ogg:pac:ra:spx:tta:wav:wma:wv" ) );
		for ( QStringList::ConstIterator it = audioExts.begin(), end = audioExts.end(); it != end; ++it )
// 			audioExtensions += " *." + *it /*+ " *." + (*it).upper()*/;
			audioExtensions += " *." + *it + " *." + (*it).upper();
		mediaExtensions += audioExtensions;
		filter += "\n" + audioExtensions.stripWhiteSpace() + "|" + i18n( "Audio Files" );

		filter = mediaExtensions + "|" + i18n( "Media Files" ) + filter;
		filter += "\n*|" + i18n( "All Files" );
	}

	return filter;
}

const QString& Application::buildLevelsFilesFilter()
{
	static QString filter;

	if ( filter.isEmpty() )
	{
		QString levelsExtensions;
		QStringList videoExts( QStringList::split( ':', "wf" ) );
		for ( QStringList::ConstIterator it = videoExts.begin(), end = videoExts.end(); it != end; ++it )
			levelsExtensions += " *." + *it + " *." + (*it).upper();
		filter += "\n" + levelsExtensions.stripWhiteSpace() + "|" + i18n( "Audio Levels Files" );

		filter += buildMediaFilesFilter();
	}

	return filter;
}

void Application::setupActions()
{
	KActionCollection* actionCollection = m_actionManager.actionCollection();


	KAction* quitAction = KStdAction::quit( this, SLOT( showConfirmQuit() ), actionCollection );
	quitAction->setToolTip( i18n( "Exit the application" ) );


	KAction* prefsAction = KStdAction::preferences( m_configDialog, SLOT( show() ), actionCollection );
	prefsAction->setToolTip( i18n( "Configure Subtitle Composer" ) );


	KActionExt* newSubtitleAction = new KActionExt( KStdAccel::openNew(), actionCollection, ACT_NEW_SUBTITLE );
	newSubtitleAction->setIcon( "filenew" );
	newSubtitleAction->setText( i18n( "New" ) );
	newSubtitleAction->setToolTip( i18n( "Create an empty subtitle" ) );
// 	newSubtitleAction->setShortcut( KStdAccel::openNew(), KActionExt::DefaultShortcut );
	connect( newSubtitleAction, SIGNAL( activated() ), this, SLOT( newSubtitle() ) );
	m_actionManager.addAction( new UserAction( newSubtitleAction, UserAction::FullScreenOff ) );


	KActionExt* openSubtitleAction = new KActionExt( KStdAccel::open(), actionCollection, ACT_OPEN_SUBTITLE );
	openSubtitleAction->setIcon( "fileopen" );
	openSubtitleAction->setText( i18n( "Open..." ) );
	openSubtitleAction->setToolTip( i18n( "Open subtitle file" ) );
// 	openSubtitleAction->setShortcut( KStdAccel::open(), KActionExt::DefaultShortcut );
	connect( openSubtitleAction, SIGNAL( activated() ), this, SLOT( openSubtitle() ) );
	m_actionManager.addAction( new UserAction( openSubtitleAction, UserAction::FullScreenOff ) );


	m_recentSubtitlesAction = new KRecentFilesActionExt( actionCollection, ACT_RECENT_SUBTITLES );
	m_recentSubtitlesAction->setIgnoreURLQueryItems( true );
	m_recentSubtitlesAction->setMaxItems( 10 );
	m_recentSubtitlesAction->setIcon( "fileopen" );
	m_recentSubtitlesAction->setText( i18n( "Open &Recent" ) );
	m_recentSubtitlesAction->setToolTip( i18n( "Open subtitle file" ) );
	connect( m_recentSubtitlesAction, SIGNAL(urlSelected(const KURL&)), this, SLOT(openSubtitle(const KURL&)) );


	m_encodingsComboBox = new KComboBox( m_mainWindow );
	m_encodingsComboBox->insertStringList( availableEncodingNames() );
	m_encodingsComboBox->setCurrentItem( generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING ) );
	connect( m_encodingsComboBox, SIGNAL(activated(const QString&)), this, SLOT(openSubtitle(const QString&)) );

	KWidgetAction* changeSubtitleEncodingAction = new KWidgetAction( m_encodingsComboBox,
		i18n( "Change Encoding" ),			// Action text
		KShortcut(),						// Action accelerator
		this,								// Action receiver
		SLOT( openSubtitle() ),				// Action slot NOT REALLY USED HERE
		actionCollection,					// Action collection manager
		ACT_CHANGE_SUBTITLE_ENCODING );		// Action name/identifier
	changeSubtitleEncodingAction->plug( m_mainWindow->toolBar( "mainToolBar" ) );
	changeSubtitleEncodingAction->setToolTip( i18n( "Reload opened subtitle file with a different encoding" ) );
 	m_actionManager.addAction( new UserAction( changeSubtitleEncodingAction, UserAction::SubPClean|UserAction::FullScreenOff ) );


	KActionExt* saveSubtitleAction = new KActionExt( KStdAccel::save(), actionCollection, ACT_SAVE_SUBTITLE );
	saveSubtitleAction->setIcon( "filesave" );
	saveSubtitleAction->setText( i18n( "Save" ) );
	saveSubtitleAction->setToolTip( i18n( "Save opened subtitle" ) );
// 	saveSubtitleAction->setShortcut( KStdAccel::save(), KActionExt::DefaultShortcut );
	connect( saveSubtitleAction, SIGNAL( activated() ), this, SLOT( saveSubtitle() ) );
	m_actionManager.addAction( new UserAction( saveSubtitleAction, UserAction::SubPDirty|UserAction::FullScreenOff ) );


	KActionExt* saveSubtitleAsAction = new KActionExt( actionCollection, ACT_SAVE_SUBTITLE_AS );
	saveSubtitleAsAction->setIcon( "filesaveas" );
	saveSubtitleAsAction->setText( i18n( "Save As..." ) );
	saveSubtitleAsAction->setToolTip( i18n( "Save opened subtitle with different settings" ) );
	connect( saveSubtitleAsAction, SIGNAL( activated() ), this, SLOT( saveSubtitleAs() ) );
	m_actionManager.addAction( new UserAction( saveSubtitleAsAction, UserAction::SubOpened|UserAction::FullScreenOff ) );


	KActionExt* closeSubtitleAction = new KActionExt( KStdAccel::close(), actionCollection, ACT_CLOSE_SUBTITLE );
	closeSubtitleAction->setIcon( "fileclose" );
	closeSubtitleAction->setText( i18n( "Close" ) );
	closeSubtitleAction->setToolTip( i18n( "Close opened subtitle" ) );
// 	closeSubtitleAction->setShortcut( KStdAccel::close(), KActionExt::DefaultShortcut );
	connect( closeSubtitleAction, SIGNAL( activated() ), this, SLOT( closeSubtitle() ) );
	m_actionManager.addAction( new UserAction( closeSubtitleAction, UserAction::SubOpened|UserAction::FullScreenOff ) );

	KActionExt* newTrSubtitleAction = new KActionExt( "Ctrl+Shift+N", actionCollection, ACT_NEW_TR_SUBTITLE );
	newTrSubtitleAction->setIcon( "filenew" );
	newTrSubtitleAction->setText( i18n( "New Translation" ) );
	newTrSubtitleAction->setToolTip( i18n( "Create an empty translation subtitle" ) );
// 	newTrSubtitleAction->setShortcut( "Ctrl+Shift+N", KActionExt::DefaultShortcut );
	connect( newTrSubtitleAction, SIGNAL( activated() ), this, SLOT( newTrSubtitle() ) );
	m_actionManager.addAction( new UserAction( newTrSubtitleAction, UserAction::SubOpened|UserAction::FullScreenOff ) );


	KActionExt* openTrSubtitleAction = new KActionExt( "Ctrl+Shift+O", actionCollection, ACT_OPEN_TR_SUBTITLE );
	openTrSubtitleAction->setIcon( "fileopen" );
	openTrSubtitleAction->setText( i18n( "Open Translation..." ) );
	openTrSubtitleAction->setToolTip( i18n( "Open translation subtitle file" ) );
// 	openTrSubtitleAction->setShortcut( "Ctrl+Shift+O", KActionExt::DefaultShortcut );
	connect( openTrSubtitleAction, SIGNAL( activated() ), this, SLOT( openTrSubtitle() ) );
	m_actionManager.addAction( new UserAction( openTrSubtitleAction, UserAction::SubOpened|UserAction::FullScreenOff ) );


	m_recentTrSubtitlesAction = new KRecentFilesActionExt( actionCollection, ACT_RECENT_TR_SUBTITLES );
	m_recentTrSubtitlesAction->setIgnoreURLQueryItems( true );
	m_recentTrSubtitlesAction->setMaxItems( 10 );
	m_recentTrSubtitlesAction->setIcon( "fileopen" );
	m_recentTrSubtitlesAction->setText( i18n( "Open &Recent Translation" ) );
	m_recentTrSubtitlesAction->setToolTip( i18n( "Open translation subtitle file" ) );
	connect( m_recentTrSubtitlesAction, SIGNAL(urlSelected(const KURL&)), this, SLOT(openTrSubtitle(const KURL&)) );
	m_actionManager.addAction( new UserAction( m_recentTrSubtitlesAction, UserAction::SubOpened|UserAction::FullScreenOff ) );


	KActionExt* saveTrSubtitleAction = new KActionExt( "Ctrl+Shift+S", actionCollection, ACT_SAVE_TR_SUBTITLE );
	saveTrSubtitleAction->setIcon( "filesave" );
	saveTrSubtitleAction->setText( i18n( "Save Translation" ) );
	saveTrSubtitleAction->setToolTip( i18n( "Save opened translation subtitle" ) );
// 	saveTrSubtitleAction->setShortcut( "Ctrl+Shift+S", KActionExt::DefaultShortcut );
	connect( saveTrSubtitleAction, SIGNAL( activated() ), this, SLOT( saveTrSubtitle() ) );
	m_actionManager.addAction( new UserAction( saveTrSubtitleAction, UserAction::SubSDirty|UserAction::FullScreenOff ) );


	KActionExt* saveTrSubtitleAsAction = new KActionExt( actionCollection, ACT_SAVE_TR_SUBTITLE_AS );
	saveTrSubtitleAsAction->setIcon( "filesaveas" );
	saveTrSubtitleAsAction->setText( i18n( "Save Translation As..." ) );
	saveTrSubtitleAsAction->setToolTip( i18n( "Save opened translation subtitle with different settings" ) );
	connect( saveTrSubtitleAsAction, SIGNAL( activated() ), this, SLOT( saveTrSubtitleAs() ) );
	m_actionManager.addAction( new UserAction( saveTrSubtitleAsAction, UserAction::SubTrOpened|UserAction::FullScreenOff ) );


	KActionExt* closeTrSubtitleAction = new KActionExt( "Ctrl+Shift+F4;Ctrl+Shift+W", actionCollection, ACT_CLOSE_TR_SUBTITLE );
	closeTrSubtitleAction->setIcon( "fileclose" );
	closeTrSubtitleAction->setText( i18n( "Close Translation" ) );
	closeTrSubtitleAction->setToolTip( i18n( "Close opened translation subtitle" ) );
// 	closeTrSubtitleAction->setShortcut( "Ctrl+Shift+F4", KActionExt::DefaultShortcut );
	connect( closeTrSubtitleAction, SIGNAL( activated() ), this, SLOT( closeTrSubtitle() ) );
	m_actionManager.addAction( new UserAction( closeTrSubtitleAction, UserAction::SubTrOpened|UserAction::FullScreenOff ) );


	KAction* undoAction = KStdAction::undo(
		this,								// Action receiver
		SLOT( undo() ),						// Action slot
		actionCollection,					// Action collection manager
		ACT_UNDO );							// Action name/identifier
 	m_actionManager.addAction( new UserAction( undoAction, UserAction::SubHasUndo ) );


	KAction* redoAction = KStdAction::redo(
		this,								// Action receiver
		SLOT( redo() ),						// Action slot
		actionCollection,					// Action collection manager
		ACT_REDO );							// Action name/identifier
 	m_actionManager.addAction( new UserAction( redoAction, UserAction::SubHasRedo ) );


	KActionExt* splitSubtitleAction = new KActionExt( actionCollection, ACT_SPLIT_SUBTITLE );
	splitSubtitleAction->setText( i18n( "Split Subtitle..." ) );
	splitSubtitleAction->setToolTip( i18n( "Split the opened subtitle in two parts" ) );
	connect( splitSubtitleAction, SIGNAL( activated() ), this, SLOT( splitSubtitle() ) );
	m_actionManager.addAction( new UserAction( splitSubtitleAction, UserAction::SubHasLines|UserAction::FullScreenOff ) );


	KActionExt* joinSubtitlesAction = new KActionExt( actionCollection, ACT_JOIN_SUBTITLES );
	joinSubtitlesAction->setText( i18n( "Join Subtitles..." ) );
	joinSubtitlesAction->setToolTip( i18n( "Append to the opened subtitle another one" ) );
	connect( joinSubtitlesAction, SIGNAL( activated() ), this, SLOT( joinSubtitles() ) );
	m_actionManager.addAction( new UserAction( joinSubtitlesAction, UserAction::SubOpened|UserAction::FullScreenOff ) );


	KActionExt* insertBeforeCurrentLineAction = new KActionExt( "Ctrl+Insert", actionCollection, ACT_INSERT_BEFORE_CURRENT_LINE );
	insertBeforeCurrentLineAction->setText( i18n( "Insert Before" ) );
	insertBeforeCurrentLineAction->setToolTip( i18n( "Insert empty line before current one" ) );
// 	insertBeforeCurrentLineAction->setShortcut( "Ctrl+Insert", KActionExt::DefaultShortcut );
	connect( insertBeforeCurrentLineAction, SIGNAL( activated() ), this, SLOT( insertBeforeCurrentLine() ) );
	m_actionManager.addAction( new UserAction( insertBeforeCurrentLineAction, UserAction::SubOpened|UserAction::FullScreenOff ) );


	KActionExt* insertAfterCurrentLineAction = new KActionExt( "Insert", actionCollection, ACT_INSERT_AFTER_CURRENT_LINE );
	insertAfterCurrentLineAction->setText( i18n( "Insert After" ) );
	insertAfterCurrentLineAction->setToolTip( i18n( "Insert empty line after current one" ) );
// 	insertAfterCurrentLineAction->setShortcut( "Insert", KActionExt::DefaultShortcut );
	connect( insertAfterCurrentLineAction, SIGNAL( activated() ), this, SLOT( insertAfterCurrentLine() ) );
	m_actionManager.addAction( new UserAction( insertAfterCurrentLineAction, UserAction::SubOpened|UserAction::FullScreenOff ) );


	KActionExt* removeSelectedLinesAction = new KActionExt( "Delete", actionCollection, ACT_REMOVE_SELECTED_LINES );
	removeSelectedLinesAction->setText( i18n( "Remove" ) );
	removeSelectedLinesAction->setToolTip( i18n( "Remove selected lines" ) );
// 	removeSelectedLinesAction->setShortcut( "Delete", KActionExt::DefaultShortcut );
	connect( removeSelectedLinesAction, SIGNAL( activated() ), this, SLOT( removeSelectedLines() ) );
	m_actionManager.addAction( new UserAction( removeSelectedLinesAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* splitSelectedLinesAction = new KActionExt( actionCollection, ACT_SPLIT_SELECTED_LINES );
	splitSelectedLinesAction->setText( i18n( "Split Lines" ) );
	splitSelectedLinesAction->setToolTip( i18n( "Split selected lines" ) );
	connect( splitSelectedLinesAction, SIGNAL( activated() ), this, SLOT( splitSelectedLines() ) );
	m_actionManager.addAction( new UserAction( splitSelectedLinesAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* joinSelectedLinesAction = new KActionExt( actionCollection, ACT_JOIN_SELECTED_LINES );
	joinSelectedLinesAction->setText( i18n( "Join Lines" ) );
	joinSelectedLinesAction->setToolTip( i18n( "Join selected lines" ) );
	connect( joinSelectedLinesAction, SIGNAL( activated() ), this, SLOT( joinSelectedLines() ) );
	m_actionManager.addAction( new UserAction( joinSelectedLinesAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* selectAllLinesAction = new KActionExt( KStdAccel::selectAll(), actionCollection, ACT_SELECT_ALL_LINES );
	selectAllLinesAction->setText( i18n( "Select All" ) );
	selectAllLinesAction->setToolTip( i18n( "Select all lines" ) );
// 	selectAllLinesAction->setShortcut( KStdAccel::selectAll(), KActionExt::DefaultShortcut );
	connect( selectAllLinesAction, SIGNAL( activated() ), this, SLOT( selectAllLines() ) );
	m_actionManager.addAction( new UserAction( selectAllLinesAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* gotoLineAction = new KActionExt( KStdAccel::gotoLine(), actionCollection, ACT_GOTO_LINE );
	gotoLineAction->setText( i18n( "Go to Line..." ) );
	gotoLineAction->setToolTip( i18n( "Go to specified line" ) );
// 	gotoLineAction->setShortcut( KStdAccel::gotoLine(), KActionExt::DefaultShortcut );
	connect( gotoLineAction, SIGNAL( activated() ), this, SLOT( gotoLine() ) );
	m_actionManager.addAction( new UserAction( gotoLineAction, UserAction::SubHasLines|UserAction::FullScreenOff ) );


	KActionExt* findAction = new KActionExt( KStdAccel::find(), actionCollection, ACT_FIND );
	findAction->setIcon( "find" );
	findAction->setText( i18n( "Find..." ) );
	findAction->setToolTip( i18n( "Find occurrences of strings or regular expressions" ) );
// 	findAction->setShortcut( KStdAccel::find(), KActionExt::DefaultShortcut );
	connect( findAction, SIGNAL( activated() ), this, SLOT( find() ) );
	m_actionManager.addAction( new UserAction( findAction, UserAction::SubHasLine ) );


	KActionExt* findNextAction = new KActionExt( KStdAccel::findNext(), actionCollection, ACT_FIND_NEXT );
	findNextAction->setIcon( "next" );
	findNextAction->setText( i18n( "Find Next" ) );
	findNextAction->setToolTip( i18n( "Find next occurrence of string or regular expression" ) );
// 	findNextAction->setShortcut( KStdAccel::findNext(), KActionExt::DefaultShortcut );
	connect( findNextAction, SIGNAL( activated() ), this, SLOT( findNext() ) );
	m_actionManager.addAction( new UserAction( findNextAction, UserAction::SubHasLine ) );


	KActionExt* findPreviousAction = new KActionExt( KStdAccel::findPrev(), actionCollection, ACT_FIND_PREVIOUS );
	findPreviousAction->setIcon( "previous" );
	findPreviousAction->setText( i18n( "Find Previous" ) );
	findPreviousAction->setToolTip( i18n( "Find previous occurrence of string or regular expression" ) );
// 	findPreviousAction->setShortcut( KStdAccel::findPrev(), KActionExt::DefaultShortcut );
	connect( findPreviousAction, SIGNAL( activated() ), this, SLOT( findPrevious() ) );
	m_actionManager.addAction( new UserAction( findPreviousAction, UserAction::SubHasLine ) );


	KActionExt* replaceAction = new KActionExt( KStdAccel::replace(), actionCollection, ACT_REPLACE );
	replaceAction->setText( i18n( "Replace..." ) );
	replaceAction->setToolTip( i18n( "Replace occurrences of strings or regular expressions" ) );
// 	replaceAction->setShortcut( KStdAccel::replace(), KActionExt::DefaultShortcut );
	connect( replaceAction, SIGNAL( activated() ), this, SLOT( replace() ) );
	m_actionManager.addAction( new UserAction( replaceAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* checkErrorsAction = new KActionExt( "Ctrl+E", actionCollection, ACT_CHECK_ERRORS );
	checkErrorsAction->setText( i18n( "Check Errors..." ) );
	checkErrorsAction->setToolTip( i18n( "Check for errors in the current subtitle" ) );
// 	checkErrorsAction->setShortcut( "Ctrl+E", KActionExt::DefaultShortcut );
	connect( checkErrorsAction, SIGNAL( activated() ), this, SLOT( checkErrors() ) );
	m_actionManager.addAction( new UserAction( checkErrorsAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* clearErrorsAction = new KActionExt( actionCollection, ACT_CLEAR_ERRORS );
	clearErrorsAction->setText( i18n( "Clear Errors..." ) );
	clearErrorsAction->setToolTip( i18n( "Clear errors from selected lines" ) );
	connect( clearErrorsAction, SIGNAL( activated() ), this, SLOT( clearErrors() ) );
	m_actionManager.addAction( new UserAction( clearErrorsAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* showErrorsAction = new KActionExt( "Shift+E", actionCollection, ACT_SHOW_ERRORS );
	showErrorsAction->setText( i18n( "Show Errors..." ) );
	showErrorsAction->setToolTip( i18n( "Show errors information for the current subtitle" ) );
// 	showErrorsAction->setShortcut( "Shift+E", KActionExt::DefaultShortcut );
	connect( showErrorsAction, SIGNAL( activated() ), this, SLOT( showErrors() ) );
	m_actionManager.addAction( new UserAction( showErrorsAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* findNextErrorAction = new KActionExt( "F4", actionCollection, ACT_FIND_NEXT_ERROR );
	findNextErrorAction->setText( i18n( "Find Next Error" ) );
	findNextErrorAction->setToolTip( i18n( "Find next line with errors" ) );
// 	findNextErrorAction->setShortcut( "F4", KActionExt::DefaultShortcut );
	connect( findNextErrorAction, SIGNAL( activated() ), this, SLOT( findNextError() ) );
	m_actionManager.addAction( new UserAction( findNextErrorAction, UserAction::SubHasLine ) );


	KActionExt* findPreviousErrorAction = new KActionExt( "Shift+F4", actionCollection, ACT_FIND_PREVIOUS_ERROR );
	findPreviousErrorAction->setText( i18n( "Find Previous Error" ) );
	findPreviousErrorAction->setToolTip( i18n( "Find previous line with errors" ) );
// 	findPreviousErrorAction->setShortcut( "Shift+F4", KActionExt::DefaultShortcut );
	connect( findPreviousErrorAction, SIGNAL( activated() ), this, SLOT( findPreviousError() ) );
	m_actionManager.addAction( new UserAction( findPreviousErrorAction, UserAction::SubHasLine ) );


	KActionExt* spellCheckAction = new KActionExt( actionCollection, ACT_SPELL_CHECK );
	spellCheckAction->setIcon( "spellcheck" );
	spellCheckAction->setText( i18n( "Spelling..." ) );
	spellCheckAction->setToolTip( i18n( "Check lines spelling" ) );
	connect( spellCheckAction, SIGNAL( activated() ), this, SLOT( spellCheck() ) );
	m_actionManager.addAction( new UserAction( spellCheckAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* toggleSelectedLinesMarkAction = new KActionExt( "Ctrl+M", actionCollection, ACT_TOGGLE_SELECTED_LINES_MARK );
	toggleSelectedLinesMarkAction->setText( i18n( "Toggle Mark" ) );
	toggleSelectedLinesMarkAction->setToolTip( i18n( "Toggle selected lines mark" ) );
// 	toggleSelectedLinesMarkAction->setShortcut( "Ctrl+M", KActionExt::DefaultShortcut );
	connect( toggleSelectedLinesMarkAction, SIGNAL( activated() ), this, SLOT( toggleSelectedLinesMark() ) );
	m_actionManager.addAction( new UserAction( toggleSelectedLinesMarkAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* toggleSelectedLinesBoldAction = new KActionExt( "Ctrl+B", actionCollection, ACT_TOGGLE_SELECTED_LINES_BOLD );
	toggleSelectedLinesBoldAction->setIcon( "text_bold" );
	toggleSelectedLinesBoldAction->setText( i18n( "Toggle Bold" ) );
	toggleSelectedLinesBoldAction->setToolTip( i18n( "Toggle selected lines bold attribute" ) );
// 	toggleSelectedLinesBoldAction->setShortcut( "Ctrl+B", KActionExt::DefaultShortcut );
	connect( toggleSelectedLinesBoldAction, SIGNAL( activated() ), this, SLOT( toggleSelectedLinesBold() ) );
	m_actionManager.addAction( new UserAction( toggleSelectedLinesBoldAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* toggleSelectedLinesItalicAction = new KActionExt( "Ctrl+I", actionCollection, ACT_TOGGLE_SELECTED_LINES_ITALIC );
	toggleSelectedLinesItalicAction->setIcon( "text_italic" );
	toggleSelectedLinesItalicAction->setText( i18n( "Toggle Italic" ) );
	toggleSelectedLinesItalicAction->setToolTip( i18n( "Toggle selected lines italic attribute" ) );
// 	toggleSelectedLinesItalicAction->setShortcut( "Ctrl+I", KActionExt::DefaultShortcut );
	connect( toggleSelectedLinesItalicAction, SIGNAL( activated() ), this, SLOT( toggleSelectedLinesItalic() ) );
	m_actionManager.addAction( new UserAction( toggleSelectedLinesItalicAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* toggleSelectedLinesUnderlineAction = new KActionExt( "Ctrl+U", actionCollection, ACT_TOGGLE_SELECTED_LINES_UNDERLINE );
	toggleSelectedLinesUnderlineAction->setIcon( "text_under" );
	toggleSelectedLinesUnderlineAction->setText( i18n( "Toggle Underline" ) );
	toggleSelectedLinesUnderlineAction->setToolTip( i18n( "Toggle selected lines underline attribute" ) );
// 	toggleSelectedLinesUnderlineAction->setShortcut( "Ctrl+U", KActionExt::DefaultShortcut );
	connect( toggleSelectedLinesUnderlineAction, SIGNAL( activated() ), this, SLOT( toggleSelectedLinesUnderline() ) );
	m_actionManager.addAction( new UserAction( toggleSelectedLinesUnderlineAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* shiftAction = new KActionExt( "Ctrl+D", actionCollection, ACT_SHIFT );
	shiftAction->setText( i18n( "Shift Lines..." ) );
	shiftAction->setToolTip( i18n( "Shift lines an specified amount of time" ) );
// 	shiftAction->setShortcut( "Ctrl+D", KActionExt::DefaultShortcut );
	connect( shiftAction, SIGNAL( activated() ), this, SLOT( shiftLines() ) );
	m_actionManager.addAction( new UserAction( shiftAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	QString shiftTimeMillis( generalConfig()->option( OPT_SHIFT_SELECTED_LINES_MSECS ) );

	KActionExt* shiftSelectedLinesFwdAction = new KActionExt( "Shift++", actionCollection, ACT_SHIFT_SELECTED_LINES_FORWARDS );
	shiftSelectedLinesFwdAction->setText( i18n( "Shift +%1 Milliseconds" ).arg( shiftTimeMillis ) );
	shiftSelectedLinesFwdAction->setToolTip( i18n( "Shift selected lines +%1 milliseconds" ).arg( shiftTimeMillis ) );
// 	shiftSelectedLinesFwdAction->setShortcut( "Shift++", KActionExt::DefaultShortcut );
	connect( shiftSelectedLinesFwdAction, SIGNAL( activated() ), this, SLOT( shiftSelectedLinesForwards() ) );
	m_actionManager.addAction( new UserAction( shiftSelectedLinesFwdAction, UserAction::HasSelection|UserAction::FullScreenOff) );


	KActionExt* shiftSelectedLinesBwdAction = new KActionExt( "Shift+-", actionCollection, ACT_SHIFT_SELECTED_LINES_BACKWARDS );
	shiftSelectedLinesBwdAction->setText( i18n( "Shift -%1 Milliseconds" ).arg( shiftTimeMillis ) );
	shiftSelectedLinesBwdAction->setToolTip( i18n( "Shift selected lines -%1 milliseconds" ).arg( shiftTimeMillis ) );
// 	shiftSelectedLinesBwdAction->setShortcut( "Shift+-", KActionExt::DefaultShortcut );
	connect( shiftSelectedLinesBwdAction, SIGNAL( activated() ), this, SLOT( shiftSelectedLinesBackwards() ) );
	m_actionManager.addAction( new UserAction( shiftSelectedLinesBwdAction, UserAction::HasSelection|UserAction::FullScreenOff) );


	KActionExt* adjustAction = new KActionExt( "Ctrl+J", actionCollection, ACT_ADJUST );
	adjustAction->setText( i18n( "Adjust Lines..." ) );
	adjustAction->setToolTip( i18n( "Linearly adjust lines to a specified interval" ) );
// 	adjustAction->setShortcut( "Ctrl+J", KActionExt::DefaultShortcut );
	connect( adjustAction, SIGNAL( activated() ), this, SLOT( adjustLines() ) );
	m_actionManager.addAction( new UserAction( adjustAction, UserAction::SubHasLines|UserAction::FullScreenOff ) );


	KActionExt* sortLinesAction = new KActionExt( actionCollection, ACT_SORT_LINES );
	sortLinesAction->setText( i18n( "Sort Lines..." ) );
	sortLinesAction->setToolTip( i18n( "Sort lines based on their show time" ) );
	connect( sortLinesAction, SIGNAL( activated() ), this, SLOT( sortLines() ) );
	m_actionManager.addAction( new UserAction( sortLinesAction, UserAction::SubHasLines|UserAction::FullScreenOff ) );


	KActionExt* changeFrameRateAction = new KActionExt( actionCollection, ACT_CHANGE_FRAME_RATE );
	changeFrameRateAction->setText( i18n( "Change Frame Rate..." ) );
	changeFrameRateAction->setToolTip( i18n( "Retime subtitle lines by reinterpreting its frame rate" ) );
	connect( changeFrameRateAction, SIGNAL( activated() ), this, SLOT( changeFrameRate() ) );
	m_actionManager.addAction( new UserAction( changeFrameRateAction, UserAction::SubOpened|UserAction::FullScreenOff ) );


	KActionExt* durationLimitsAction = new KActionExt( "Ctrl+L", actionCollection, ACT_DURATION_LIMITS );
	durationLimitsAction->setText( i18n( "Duration Limits..." ) );
	durationLimitsAction->setToolTip( i18n( "Enforce lines minimun and/or maximun duration limits" ) );
// 	durationLimitsAction->setShortcut( "Ctrl+L", KActionExt::DefaultShortcut );
	connect( durationLimitsAction, SIGNAL( activated() ), this, SLOT( durationLimits() ) );
	m_actionManager.addAction( new UserAction( durationLimitsAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* autoDurationsAction = new KActionExt( actionCollection, ACT_AUTOMATIC_DURATIONS );
	autoDurationsAction->setText( i18n( "Automatic Durations..." ) );
	autoDurationsAction->setToolTip( i18n( "Set lines durations based on amount of words and/or line breaks" ) );
	connect( autoDurationsAction, SIGNAL( activated() ), this, SLOT( autoDurations() ) );
	m_actionManager.addAction( new UserAction( autoDurationsAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* maximizeDurationsAction = new KActionExt( actionCollection, ACT_MAXIMIZE_DURATIONS );
	maximizeDurationsAction->setText( i18n( "Maximize Durations..." ) );
	maximizeDurationsAction->setToolTip( i18n( "Extend lines durations up to their next lines show time" ) );
	connect( maximizeDurationsAction, SIGNAL( activated() ), this, SLOT( maximizeDurations() ) );
	m_actionManager.addAction( new UserAction( maximizeDurationsAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* fixOverlappingLinesAction = new KActionExt( actionCollection, ACT_FIX_OVERLAPPING_LINES );
	fixOverlappingLinesAction->setText( i18n( "Fix Overlapping Lines..." ) );
	fixOverlappingLinesAction->setToolTip( i18n( "Fix lines overlapping times" ) );
	connect( fixOverlappingLinesAction, SIGNAL( activated() ), this, SLOT( fixOverlappingLines() ) );
	m_actionManager.addAction( new UserAction( fixOverlappingLinesAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* syncWithSubtitleAction = new KActionExt( actionCollection, ACT_SYNC_WITH_SUBTITLE );
	syncWithSubtitleAction->setText( i18n( "Synchronize With Subtitle..." ) );
	syncWithSubtitleAction->setToolTip( i18n( "Copy timing information from another subtitle" ) ); 
	connect( syncWithSubtitleAction, SIGNAL( activated() ), this, SLOT( syncWithSubtitle() ) );
	m_actionManager.addAction( new UserAction( syncWithSubtitleAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* adjustTextsAction = new KActionExt( actionCollection, ACT_ADJUST_TEXTS );
	adjustTextsAction->setText( i18n( "Smart Lines Adjust..." ) );
	adjustTextsAction->setToolTip( i18n( "Automatically set line breaks" ) );
	connect( adjustTextsAction, SIGNAL( activated() ), this, SLOT( adjustTexts() ) );
	m_actionManager.addAction( new UserAction( adjustTextsAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* unbreakTextsAction = new KActionExt( actionCollection, ACT_UNBREAK_TEXTS );
	unbreakTextsAction->setText( i18n( "Unbreak Lines..." ) );
	unbreakTextsAction->setToolTip( i18n( "Remove line breaks from lines" ) );
	connect( unbreakTextsAction, SIGNAL( activated() ), this, SLOT( unbreakTexts() ) );
	m_actionManager.addAction( new UserAction( unbreakTextsAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* simplifySpacesAction = new KActionExt( actionCollection, ACT_SIMPLIFY_SPACES );
	simplifySpacesAction->setText( i18n( "Simplify Spaces..." ) );
	simplifySpacesAction->setToolTip( i18n( "Remove unneeded spaces from lines" ) );
	connect( simplifySpacesAction, SIGNAL( activated() ), this, SLOT( simplifySpaces() ) );
	m_actionManager.addAction( new UserAction( simplifySpacesAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* changeCaseAction = new KActionExt( actionCollection, ACT_CHANGE_CASE );
	changeCaseAction->setText( i18n( "Change Case..." ) );
	changeCaseAction->setToolTip( i18n( "Change lines text to upper, lower, title or sentence case" ) );
	connect( changeCaseAction, SIGNAL( activated() ), this, SLOT( changeCase() ) );
	m_actionManager.addAction( new UserAction( changeCaseAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* fixPunctuationAction = new KActionExt( actionCollection, ACT_FIX_PUNCTUATION );
	fixPunctuationAction->setText( i18n( "Fix Punctuation..." ) );
	fixPunctuationAction->setToolTip( i18n( "Fix punctuation errors in lines" ) );
	connect( fixPunctuationAction, SIGNAL( activated() ), this, SLOT( fixPunctuation() ) );
	m_actionManager.addAction( new UserAction( fixPunctuationAction, UserAction::SubHasLine|UserAction::FullScreenOff ) );


	KActionExt* editCurrentLinePrimaryInPlaceAction = new KActionExt( "F2", actionCollection, ACT_EDIT_CURRENT_LINE_IN_PLACE );
	editCurrentLinePrimaryInPlaceAction->setText( i18n( "Edit Text In Place" ) );
	editCurrentLinePrimaryInPlaceAction->setToolTip( i18n( "Edit current line text in place" ) );
// 	editCurrentLinePrimaryInPlaceAction->setShortcut( "F2", KActionExt::DefaultShortcut );
	connect( editCurrentLinePrimaryInPlaceAction, SIGNAL( activated() ), m_linesWidget, SLOT( editCurrentLinePrimaryInPlace() ) );
	m_actionManager.addAction( new UserAction( editCurrentLinePrimaryInPlaceAction, UserAction::HasSelection|UserAction::FullScreenOff ) );


	KActionExt* editCurrentLineSecondaryInPlaceAction = new KActionExt( "F6", actionCollection, ACT_EDIT_CURRENT_LINE_TR_IN_PLACE );
	editCurrentLineSecondaryInPlaceAction->setText( i18n( "Edit Translation In Place" ) );
	editCurrentLineSecondaryInPlaceAction->setToolTip( i18n( "Edit current line translation text in place" ) );
// 	editCurrentLineSecondaryInPlaceAction->setShortcut( "F6", KActionExt::DefaultShortcut );
	connect( editCurrentLineSecondaryInPlaceAction, SIGNAL(activated()), m_linesWidget, SLOT(editCurrentLineSecondaryInPlace()) );
	m_actionManager.addAction( new UserAction( editCurrentLineSecondaryInPlaceAction, UserAction::HasSelection|UserAction::SubTrOpened|UserAction::FullScreenOff ) );


	KActionExt* openVideoAction = new KActionExt( actionCollection, ACT_OPEN_VIDEO );
	openVideoAction->setIcon( "fileopen" );
	openVideoAction->setText( i18n( "Open Video..." ) );
	openVideoAction->setToolTip( i18n( "Open video file" ) );
	connect( openVideoAction, SIGNAL( activated() ), this, SLOT( openVideo() ) );


	m_recentVideosAction = new KRecentFilesActionExt( actionCollection, ACT_RECENT_VIDEOS );
	m_recentVideosAction->setMaxItems( 5 );
	m_recentVideosAction->setIcon( "fileopen" );
	m_recentVideosAction->setText( i18n( "Open &Recent Video" ) );
	m_recentVideosAction->setToolTip( i18n( "Open video file" ) );
	connect( m_recentVideosAction, SIGNAL(urlSelected(const KURL&)), this, SLOT(openVideo(const KURL&)) );


	KActionExt* closeVideoAction = new KActionExt( actionCollection, ACT_CLOSE_VIDEO );
	closeVideoAction->setIcon( "fileclose" );
	closeVideoAction->setText( i18n( "Close Video" ) );
	closeVideoAction->setToolTip( i18n( "Close video file" ) );
	connect( closeVideoAction, SIGNAL( activated() ), m_player, SLOT( closeFile() ) );
	m_actionManager.addAction( new UserAction( closeVideoAction, UserAction::VideoOpened ) );


	KToggleActionExt* fullScreenAction = new KToggleActionExt( KStdAccel::fullScreen(), actionCollection, ACT_TOGGLE_FULL_SCREEN );
// 	fullScreenAction->setChecked( false )
	fullScreenAction->setIcon( "window_fullscreen" );
	fullScreenAction->setText( i18n( "Full Screen Mode" ) );
	fullScreenAction->setToolTip( i18n( "Toggle full screen mode" ) );
// 	fullScreenAction->setShortcut( KStdAccel::fullScreen(), KActionExt::DefaultShortcut );
	connect( fullScreenAction, SIGNAL( toggled(bool) ), this, SLOT( setFullScreenMode(bool) ) );


	KActionExt* stopAction = new KActionExt( actionCollection, ACT_STOP );
	stopAction->setIcon( "player_stop" );
	stopAction->setText( i18n( "Stop" ) );
	stopAction->setToolTip( i18n( "Stop video playback" ) );
	connect( stopAction, SIGNAL( activated() ), m_player, SLOT( stop() ) );
	m_actionManager.addAction( new UserAction( stopAction, UserAction::VideoPlaying ) );


	KActionExt* playPauseAction = new KActionExt( actionCollection, ACT_PLAY_PAUSE );
	playPauseAction->setIcon( "player_play" );
	playPauseAction->setText( i18n( "Play" ) );
	playPauseAction->setToolTip( i18n( "Toggle video playing/paused" ) );
	connect( playPauseAction, SIGNAL( activated() ), m_player, SLOT( togglePlayPaused() ) );
	m_actionManager.addAction( new UserAction( playPauseAction, UserAction::VideoOpened ) );


	KActionExt* seekBackwardsAction = new KActionExt( "Left", actionCollection, ACT_SEEK_BACKWARDS );
	seekBackwardsAction->setIcon( "player_rew" );
	seekBackwardsAction->setText( i18n( "Seek Backwards" ) );
	seekBackwardsAction->setToolTip( i18n(
		"Seek backwards 1 second",
		"Seek backwards %n seconds",
		playerConfig()->optionAsInt( OPT_JUMP_ON_SEEK_SECS )
	) );
// 	seekBackwardsAction->setShortcut( "Left", KActionExt::DefaultShortcut );
	connect( seekBackwardsAction, SIGNAL( activated() ), this, SLOT( seekBackwards() ) );
	m_actionManager.addAction( new UserAction( seekBackwardsAction, UserAction::VideoPlaying ) );


	KActionExt* seekForwardsAction = new KActionExt( "Right", actionCollection, ACT_SEEK_FORWARDS );
	seekForwardsAction->setIcon( "player_fwd" );
	seekForwardsAction->setText( i18n( "Seek Forwards" ) );
	seekForwardsAction->setToolTip( i18n(
		"Seek forwards 1 second",
		"Seek forwards %n seconds",
		playerConfig()->optionAsInt( OPT_JUMP_ON_SEEK_SECS )
	) );
// 	seekForwardsAction->setShortcut( "Right", KActionExt::DefaultShortcut );
	connect( seekForwardsAction, SIGNAL( activated() ), this, SLOT( seekForwards() ) );
	m_actionManager.addAction( new UserAction( seekForwardsAction, UserAction::VideoPlaying ) );


	KActionExt* seekToPrevLineAction = new KActionExt( "Shift+Left", actionCollection, ACT_SEEK_TO_PREVIOUS_LINE );
	seekToPrevLineAction->setIcon( "back" );
	seekToPrevLineAction->setText( i18n( "Jump to Previous Line" ) );
	seekToPrevLineAction->setToolTip( i18n( "Seek to previous subtitle line show time" ) );
// 	seekToPrevLineAction->setShortcut( "Shift+Left", KActionExt::DefaultShortcut );
	connect( seekToPrevLineAction, SIGNAL( activated() ), this, SLOT( seekToPrevLine() ) );
	m_actionManager.addAction( new UserAction( seekToPrevLineAction, UserAction::SubHasLine|UserAction::VideoPlaying ) );


	KActionExt* seekToNextLineAction = new KActionExt( "Shift+Right", actionCollection, ACT_SEEK_TO_NEXT_LINE );
	seekToNextLineAction->setIcon( "forward" );
	seekToNextLineAction->setText( i18n( "Jump to Next Line" ) );
	seekToNextLineAction->setToolTip( i18n( "Seek to next subtitle line show time" ) );
// 	seekToNextLineAction->setShortcut( "Shift+Right", KActionExt::DefaultShortcut );
	connect( seekToNextLineAction, SIGNAL( activated() ), this, SLOT( seekToNextLine() ) );
	m_actionManager.addAction( new UserAction( seekToNextLineAction, UserAction::SubHasLine|UserAction::VideoPlaying ) );


	KActionExt* setCurrentLineShowTimeFromVideoAction = new KActionExt( "Shift+Z", actionCollection, ACT_SET_CURRENT_LINE_SHOW_TIME );
	setCurrentLineShowTimeFromVideoAction->setIconSet( GUI::customIconSet( "set-show-time", KIcon::Toolbar, 16 ) );
	setCurrentLineShowTimeFromVideoAction->setText( i18n( "Set Current Line Show Time" ) );
	setCurrentLineShowTimeFromVideoAction->setToolTip( i18n( "Set current line show time to video position" ) );
// 	setCurrentLineShowTimeFromVideoAction->setShortcut( "Shift+Z", KActionExt::DefaultShortcut );
	connect( setCurrentLineShowTimeFromVideoAction, SIGNAL( activated() ), this, SLOT( setCurrentLineShowTimeFromVideo() ) );
	m_actionManager.addAction( new UserAction( setCurrentLineShowTimeFromVideoAction, UserAction::HasSelection|UserAction::VideoPlaying ) );


	KActionExt* setCurrentLineHideTimeFromVideoAction = new KActionExt( "Shift+X", actionCollection, ACT_SET_CURRENT_LINE_HIDE_TIME );
	setCurrentLineHideTimeFromVideoAction->setIconSet( GUI::customIconSet( "set-hide-time", KIcon::Toolbar, 16 ) );
	setCurrentLineHideTimeFromVideoAction->setText( i18n( "Set Current Line Hide Time" ) );
	setCurrentLineHideTimeFromVideoAction->setToolTip( i18n( "Set current line hide time to video position" ) );
// 	setCurrentLineHideTimeFromVideoAction->setShortcut( "Shift+X", KActionExt::DefaultShortcut );
	connect( setCurrentLineHideTimeFromVideoAction, SIGNAL( activated() ), this, SLOT( setCurrentLineHideTimeFromVideo() ) );
	m_actionManager.addAction( new UserAction( setCurrentLineHideTimeFromVideoAction, UserAction::HasSelection|UserAction::VideoPlaying ) );


	KToggleActionExt* currentLineFollowsVideoAction = new KToggleActionExt( actionCollection, ACT_CURRENT_LINE_FOLLOWS_VIDEO );
	currentLineFollowsVideoAction->setIcon( "current-line-follows-video" );
	currentLineFollowsVideoAction->setText( i18n( "Current Line Follows Video" ) );
	currentLineFollowsVideoAction->setToolTip( i18n( "Make current line follow the playing video position" ) );
	connect( currentLineFollowsVideoAction, SIGNAL( toggled(bool) ), this, SLOT( onLinkCurrentLineToVideoToggled(bool) ) );


	KToggleActionExt* toggleMuteAction = new KToggleActionExt( "/", actionCollection, ACT_TOGGLE_MUTED );
	toggleMuteAction->setIcon( "audio-volume-muted" );
	toggleMuteAction->setText( i18n( "Mute" ), i18n( "Unmute" ) );
	toggleMuteAction->setToolTip( i18n( "Enable/disable playback sound" ) );
// 	toggleMuteAction->setShortcut( "/", KActionExt::DefaultShortcut );
	connect( toggleMuteAction, SIGNAL( toggled(bool) ), m_player, SLOT( toggleMuted() ) );


	KActionExt* increaseVolumeAction = new KActionExt( Qt::Key_Plus, actionCollection, ACT_INCREASE_VOLUME );
	increaseVolumeAction->setIcon( "audio-volume-high" );
	increaseVolumeAction->setText( i18n( "Increase Volume" ) );
	increaseVolumeAction->setToolTip( i18n( "Increase volume by 5%" ) );
// 	increaseVolumeAction->setShortcut( Qt::Key_Plus, KActionExt::DefaultShortcut );
	connect( increaseVolumeAction, SIGNAL( activated() ), m_player, SLOT( increaseVolume() ) );


	KActionExt* decreaseVolumeAction = new KActionExt( Qt::Key_Minus, actionCollection, ACT_DECREASE_VOLUME );
	decreaseVolumeAction->setIcon( "audio-volume-low" );
	decreaseVolumeAction->setText( i18n( "Decrease Volume" ) );
	decreaseVolumeAction->setToolTip( i18n( "Decrease volume by 5%" ) );
// 	decreaseVolumeAction->setShortcut( Qt::Key_Minus, KActionExt::DefaultShortcut );
	connect( decreaseVolumeAction, SIGNAL( activated() ), m_player, SLOT( decreaseVolume() ) );


	KSelectAction* setActiveAudioStreamAction = new KSelectAction( actionCollection, ACT_SET_ACTIVE_AUDIO_STREAM );
	setActiveAudioStreamAction->setIconSet( GUI::customIconSet( "flags", KIcon::Toolbar, 16 ) );
	setActiveAudioStreamAction->setText( i18n( "Audio Streams" ) );
	setActiveAudioStreamAction->setToolTip( i18n( "Select active audio stream" ) );
	connect( setActiveAudioStreamAction, SIGNAL(activated(int)), m_player, SLOT(setActiveAudioStream(int)) );
	m_actionManager.addAction( new UserAction( setActiveAudioStreamAction, UserAction::VideoOpened ) );


	KActionExt* increaseSubtitleFontAction = new KActionExt( "Alt++", actionCollection, ACT_INCREASE_SUBTITLE_FONT );
	increaseSubtitleFontAction->setIcon( "fontsizeup" );
	increaseSubtitleFontAction->setText( i18n( "Increase Font Size" ) );
	increaseSubtitleFontAction->setToolTip( i18n( "Increase subtitles font size by 1 point" ) );
// 	increaseSubtitleFontAction->setShortcut( "Alt++", KActionExt::DefaultShortcut );
	connect( increaseSubtitleFontAction, SIGNAL( activated() ), m_playerWidget, SLOT( increaseFontSize() ) );


	KActionExt* decreaseSubtitleFontAction = new KActionExt( "Alt+-", actionCollection, ACT_DECREASE_SUBTITLE_FONT );
	decreaseSubtitleFontAction->setIcon( "fontsizedown" );
	decreaseSubtitleFontAction->setText( i18n( "Decrease Font Size" ) );
	decreaseSubtitleFontAction->setToolTip( i18n( "Decrease subtitles font size by 1 point" ) );
// 	decreaseSubtitleFontAction->setShortcut( "Alt+-", KActionExt::DefaultShortcut );
	connect( decreaseSubtitleFontAction, SIGNAL( activated() ), m_playerWidget, SLOT( decreaseFontSize() ) );


	KSelectAction* setActiveSubtitleStreamAction = new KSelectAction( actionCollection, ACT_SET_ACTIVE_SUBTITLE_STREAM );
	setActiveSubtitleStreamAction->setIconSet( GUI::customIconSet( "flags", KIcon::Toolbar, 16 ) );
	setActiveSubtitleStreamAction->setText( i18n( "Subtitle Streams" ) );
	setActiveSubtitleStreamAction->setToolTip( i18n( "Select active subtitle stream" ) );
	connect( setActiveSubtitleStreamAction, SIGNAL(activated(int)), this, SLOT(setActiveSubtitleStream(int)) );
	m_actionManager.addAction( new UserAction( setActiveSubtitleStreamAction, UserAction::SubTrOpened ) );


	KActionExt* shiftToVideoPositionAction = new KActionExt( "Shift+A", actionCollection, ACT_SHIFT_TO_VIDEO_POSITION );
	shiftToVideoPositionAction->setText( i18n( "Shift to Video Position" ) );
	shiftToVideoPositionAction->setToolTip( i18n("Set current line show time to video position by equally shifting all lines" ) );
// 	shiftToVideoPositionAction->setShortcut( "Shift+A", KActionExt::DefaultShortcut );
	connect( shiftToVideoPositionAction, SIGNAL( activated() ), this, SLOT( shiftToVideoPosition() ) );
	m_actionManager.addAction( new UserAction( shiftToVideoPositionAction, UserAction::HasSelection|UserAction::VideoPlaying|UserAction::FullScreenOff ) );


	KActionExt* adjustToVideoPositionAnchorLastAction = new KActionExt( "Alt+Z", actionCollection, ACT_ADJUST_TO_VIDEO_POSITION_A_L );
	adjustToVideoPositionAnchorLastAction->setText( i18n( "Adjust to Video Position (Anchor Last Line)" ) );
	adjustToVideoPositionAnchorLastAction->setToolTip( i18n( "Set current line to video position by fixing the last line and linearly adjusting the other ones" ) );
// 	adjustToVideoPositionAnchorLastAction->setShortcut( "Alt+Z", KActionExt::DefaultShortcut );
	connect( adjustToVideoPositionAnchorLastAction, SIGNAL( activated() ), this, SLOT( adjustToVideoPositionAnchorLast() ) );
	m_actionManager.addAction( new UserAction( adjustToVideoPositionAnchorLastAction, UserAction::HasSelection|UserAction::VideoPlaying|UserAction::FullScreenOff ) );


	KActionExt* adjustToVideoPositionAnchorFirstAction = new KActionExt( "Alt+X", actionCollection, ACT_ADJUST_TO_VIDEO_POSITION_A_F );
	adjustToVideoPositionAnchorFirstAction->setText( i18n( "Adjust to Video Position (Anchor First Line)" ) );
	adjustToVideoPositionAnchorFirstAction->setToolTip( i18n( "Set current line to video position by fixing the first line and linearly adjusting the other ones" ) );
// 	adjustToVideoPositionAnchorFirstAction->setShortcut( "Alt+X", KActionExt::DefaultShortcut );
	connect( adjustToVideoPositionAnchorFirstAction, SIGNAL( activated() ), this, SLOT( adjustToVideoPositionAnchorFirst() ) );
	m_actionManager.addAction( new UserAction( adjustToVideoPositionAnchorFirstAction, UserAction::HasSelection|UserAction::VideoPlaying|UserAction::FullScreenOff ) );


	KActionExt* openAudioLevelsAction = new KActionExt( actionCollection, ACT_OPEN_WAVEFORM );
	openAudioLevelsAction->setIcon( "fileopen" );
	openAudioLevelsAction->setText( i18n( "Open Levels..." ) );
	openAudioLevelsAction->setToolTip( i18n( "Open audio levels file" ) );
	connect( openAudioLevelsAction, SIGNAL( activated() ), this, SLOT( openAudioLevels() ) );


	m_recentAudioLevelssAction = new KRecentFilesActionExt( actionCollection, ACT_RECENT_WAVEFORMS );
	m_recentAudioLevelssAction->setMaxItems( 5 );
	m_recentAudioLevelssAction->setIcon( "fileopen" );
	m_recentAudioLevelssAction->setText( i18n( "Open &Recent Levels" ) );
	m_recentAudioLevelssAction->setToolTip( i18n( "Open audio levels file" ) );
	connect( m_recentAudioLevelssAction, SIGNAL(urlSelected(const KURL&)), this, SLOT(openAudioLevels(const KURL&)) );


	KActionExt* saveAudioLevelsAsAction = new KActionExt( actionCollection, ACT_SAVE_LEVELS_AS );
	saveAudioLevelsAsAction->setIcon( "filesaveas" );
	saveAudioLevelsAsAction->setText( i18n( "Save Levels As..." ) );
	saveAudioLevelsAsAction->setToolTip( i18n( "Save audio levels file" ) );
	connect( saveAudioLevelsAsAction, SIGNAL( activated() ), this, SLOT( saveAudioLevelsAs() ) );
	m_actionManager.addAction( new UserAction( saveAudioLevelsAsAction, UserAction::AudioLevelsOpened ) );


	KActionExt* closeAudioLevelsAction = new KActionExt( actionCollection, ACT_CLOSE_WAVEFORM );
	closeAudioLevelsAction->setIcon( "fileclose" );
	closeAudioLevelsAction->setText( i18n( "Close Levels" ) );
	closeAudioLevelsAction->setToolTip( i18n( "Close audio levels file" ) );
	connect( closeAudioLevelsAction, SIGNAL( activated() ), this, SLOT( closeAudioLevels() ) );
	m_actionManager.addAction( new UserAction( closeAudioLevelsAction, UserAction::AudioLevelsOpened ) );


	KActionExt* increaseAudioLevelsVZoomAction = new KActionExt( actionCollection, ACT_INCREASE_LEVELS_V_ZOOM );
	increaseAudioLevelsVZoomAction->setIcon( "viewmag+" );
	increaseAudioLevelsVZoomAction->setText( i18n( "Increase Vertical Zoom" ) );
	increaseAudioLevelsVZoomAction->setToolTip( i18n( "Increase audio levels vertical zoom" ) );
	connect( increaseAudioLevelsVZoomAction, SIGNAL( activated() ), this, SLOT( increaseAudioLevelsVZoom() ) );
	m_actionManager.addAction( new UserAction( increaseAudioLevelsVZoomAction, UserAction::AudioLevelsOpened ) );


	KActionExt* decreaseAudioLevelsVZoomAction = new KActionExt( actionCollection, ACT_DECREASE_LEVELS_V_ZOOM );
	decreaseAudioLevelsVZoomAction->setIcon( "viewmag-" );
	decreaseAudioLevelsVZoomAction->setText( i18n( "Decrease Vertical Zoom" ) );
	decreaseAudioLevelsVZoomAction->setToolTip( i18n( "Decrease audio levels vertical zoom" ) );
	connect( decreaseAudioLevelsVZoomAction, SIGNAL( activated() ), this, SLOT( decreaseAudioLevelsVZoom() ) );
	m_actionManager.addAction( new UserAction( decreaseAudioLevelsVZoomAction, UserAction::AudioLevelsOpened ) );


	KActionExt* increaseAudioLevelsHZoomAction = new KActionExt( actionCollection, ACT_INCREASE_LEVELS_H_ZOOM );
	increaseAudioLevelsHZoomAction->setIcon( "viewmag+" );
	increaseAudioLevelsHZoomAction->setText( i18n( "Increase Horizontal Zoom" ) );
	increaseAudioLevelsHZoomAction->setToolTip( i18n( "Increase audio levels horizontal zoom" ) );
	connect( increaseAudioLevelsHZoomAction, SIGNAL( activated() ), this, SLOT( increaseAudioLevelsHZoom() ) );
	m_actionManager.addAction( new UserAction( increaseAudioLevelsHZoomAction, UserAction::AudioLevelsOpened ) );


	KActionExt* decreaseAudioLevelsHZoomAction = new KActionExt( actionCollection, ACT_DECREASE_LEVELS_H_ZOOM );
	decreaseAudioLevelsHZoomAction->setIcon( "viewmag-" );
	decreaseAudioLevelsHZoomAction->setText( i18n( "Decrease Horizontal Zoom" ) );
	decreaseAudioLevelsHZoomAction->setToolTip( i18n( "Decrease audio levels horizontal zoom" ) );
	connect( decreaseAudioLevelsHZoomAction, SIGNAL( activated() ), this, SLOT( decreaseAudioLevelsHZoom() ) );
	m_actionManager.addAction( new UserAction( decreaseAudioLevelsHZoomAction, UserAction::AudioLevelsOpened ) );
}

/// BEGIN ACTION HANDLERS

Time Application::videoPosition( bool compensate )
{
	if ( compensate && ! m_player->isPaused() )
		return Time( (long)(m_player->position()*1000) - generalConfig()->optionAsInt( OPT_VIDEO_POS_COMP_MSECS ) );
	else
		return Time( (long)(m_player->position()*1000) );
}

void Application::undo()
{
	if ( m_subtitle )
		m_subtitle->actionManager().undo();
}

void Application::redo()
{
	if ( m_subtitle )
		m_subtitle->actionManager().redo();
}

bool Application::checkDifferentNames( const QString& subtitlePath, const QString& subtitleTrPath )
{
	if ( subtitlePath != subtitleTrPath || subtitlePath.isEmpty() || subtitleTrPath.isEmpty() )
		return true;

	return KMessageBox::Continue == KMessageBox::warningContinueCancel(
		mainWidget(),
		i18n( "The requested action would make the subtitle and it's translation share the same file, possibly resulting in loss of information when saving.\nAre you sure you want to continue?" ),
		i18n( "Conflicting Subtitle Files" )
	);
}

void Application::newSubtitle()
{
	if ( ! closeSubtitle() )
		return;

	m_subtitle = new Subtitle();

	emit subtitleOpened( m_subtitle );

	connect( m_subtitle, SIGNAL( primaryDirtyStateChanged(bool) ), this, SLOT( updateTitle() ) );
	connect( m_subtitle, SIGNAL( secondaryDirtyStateChanged(bool) ), this, SLOT( updateTitle() ) );

	updateTitle();
}

void Application::openSubtitle()
{
	OpenSubtitleDialog openDlg( true, m_lastSubtitleDir, generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING ) );

	if ( openDlg.exec() == QDialog::Accepted )
	{
		if ( ! checkDifferentNames( openDlg.selectedFile(), m_subtitleTrPath ) )
			return;

		if ( ! closeSubtitle() )
			return;

		m_lastSubtitleDir = openDlg.selectedFile();

		KURL fileURL;
		fileURL.setProtocol( "file" );
		fileURL.setPath( openDlg.selectedFile() );
		fileURL.addQueryItem( "encoding", openDlg.selectedEncoding() );
		openSubtitle( fileURL );
	}
}

void Application::openSubtitle( const KURL& url )
{
	if ( ! closeSubtitle() )
		return;

	if ( url.protocol() != "file" )
		return;

	QString path = url.path();

	QString encoding = url.queryItem( "encoding" );
	if ( encoding.isEmpty() )
		encoding = generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING );

	bool codecFound = true;
	QTextCodec* codec = KGlobal::charsets()->codecForName( encoding, codecFound );
	if ( ! codecFound )
		codec = KGlobal::locale()->codecForEncoding();

	m_subtitle = new Subtitle();

	if ( FormatsManager::readSubtitle( *m_subtitle, true, path, codec, (Format::NewLine*)&m_subtitleEOL, &m_subtitleFormat ) )
	{
		// The loading of the subtitle shouldn't be an undoable action and there's no state before it
		m_subtitle->actionManager().clearHistory();
		m_subtitle->clearPrimaryDirty();
		m_subtitle->clearSecondaryDirty();

		emit subtitleOpened( m_subtitle );

		KURL recentURL;
		recentURL.setProtocol( "file" );
		recentURL.setPath( path );
		recentURL.addQueryItem( "encoding", codec->name() );
		m_recentSubtitlesAction->add( recentURL );

		m_subtitlePath = path;
		m_subtitleFileName = QFileInfo( m_subtitlePath ).fileName();
		m_subtitleEncoding = codec->name();

		m_encodingsComboBox->setCurrentItem( m_subtitleEncoding.upper() );

		connect( m_subtitle, SIGNAL( primaryDirtyStateChanged(bool) ), this, SLOT( updateTitle() ) );
		connect( m_subtitle, SIGNAL( secondaryDirtyStateChanged(bool) ), this, SLOT( updateTitle() ) );

		updateTitle();

		if ( generalConfig()->optionAsBool( OPT_AUTO_VIDEO_LOAD ) )
		{
			static QStringList videoExtensions( QStringList::split( ' ', "avi ogm mkv mpeg mpg mp4 rv wmv" ) );

			QFileInfo subtitleFileInfo( m_subtitlePath );

			QString subtitleFileName = m_subtitleFileName.lower();
			QString videoFileName = QFileInfo( m_player->filePath() ).baseName( true ).lower();

			if ( videoFileName.isEmpty() || subtitleFileName.find( videoFileName ) != 0 )
			{
				QStringList subtitleDirFiles = subtitleFileInfo.dir().entryList( QDir::Files|QDir::Readable );
				for ( QStringList::ConstIterator it = subtitleDirFiles.begin(), end = subtitleDirFiles.end(); it != end; ++it )
				{
					QFileInfo fileInfo( *it );
					if ( videoExtensions.contains( fileInfo.extension( false ).lower() ) )
					{
						if ( subtitleFileName.find( fileInfo.baseName( true ).lower() ) == 0 )
						{
							KURL auxURL;
							auxURL.setProtocol( "file" );
							auxURL.setPath( subtitleFileInfo.dir().filePath( *it ) );
							openVideo( auxURL );
							break;
						}
					}
				}
			}
		}
	}
	else
	{
		delete m_subtitle;
		m_subtitle = 0;

		KMessageBox::sorry( mainWidget(), i18n( "<qt>Couldn't parse the subtitle file.<br/>This may have been caused by usage of the wrong encoding.</qt>" ) );
	}
}

void Application::openSubtitle( const QString& encoding )
{
	if ( m_subtitlePath )
	{
		if ( encoding != m_subtitleEncoding )
		{
			KURL fileURL;
			fileURL.setProtocol( "file" );
			fileURL.setPath( m_subtitlePath );
			fileURL.addQueryItem( "encoding", encoding );
			openSubtitle( fileURL );
		}
	}
	else
		m_subtitleEncoding = encoding;
}


bool Application::saveSubtitle()
{
	if ( m_subtitlePath.isEmpty() || m_subtitleEncoding.isEmpty() || ! FormatsManager::hasOutputFormat( m_subtitleFormat ) )
		return saveSubtitleAs();

	bool codecFound = true;
	QTextCodec* codec = KGlobal::charsets()->codecForName( m_subtitleEncoding, codecFound );
	if ( ! codecFound )
		codec = KGlobal::locale()->codecForEncoding();

	if ( FormatsManager::writeSubtitle( *m_subtitle, true, m_subtitlePath, codec, (Format::NewLine)m_subtitleEOL, m_subtitleFormat, true ) )
	{
		m_subtitle->clearPrimaryDirty();

		KURL recentURL;
		recentURL.setProtocol( "file" );
		recentURL.setPath( m_subtitlePath );
		recentURL.addQueryItem( "encoding", codec->name() );
		m_recentSubtitlesAction->add( recentURL );

		m_encodingsComboBox->setCurrentItem( QString( codec->name() ).upper() );

		updateTitle();

		return true;
	}
	else
	{
		KMessageBox::sorry( mainWidget(), i18n( "There was an error saving the subtitle." ) );
		return false;
	}
}

bool Application::saveSubtitleAs()
{
	SaveSubtitleDialog saveDlg(
		true,
		m_subtitlePath,
		m_subtitleEncoding.isEmpty() ?
			generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING ) :
			m_subtitleEncoding,
		m_subtitleEOL,
		m_subtitleFormat
	);

	if ( saveDlg.exec() == QDialog::Accepted )
	{
		if ( QFile::exists( saveDlg.selectedFile() ) )
		{
			if ( KMessageBox::warningContinueCancel(
					mainWidget(),
					i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" )
						.arg( QFileInfo( saveDlg.selectedFile() ).fileName() ),
					i18n( "Overwrite File?" ),
					i18n( "Overwrite" )
				) != KMessageBox::Continue )
				return false;
		}

		m_subtitlePath = saveDlg.selectedFile();
		m_subtitleFileName = QFileInfo( m_subtitlePath ).baseName( true );
		m_subtitleEncoding = saveDlg.selectedEncoding();
		m_subtitleFormat = saveDlg.selectedFormat();
		m_subtitleEOL = saveDlg.selectedNewLine();

		return saveSubtitle();
	}

	return false;
}

bool Application::closeSubtitle()
{
	if ( m_subtitle )
	{
		if ( m_translationMode && m_subtitle->isSecondaryDirty() )
		{
			int result = KMessageBox::warningYesNoCancel(
				0,
				i18n( "Currently opened translation subtitle has unsaved changes.\nDo you want to save them?" ),
				i18n( "Close Translation Subtitle" ) + " - SubtitleComposer" );
			if ( result == KMessageBox::Cancel )
				return false;
			else if ( result == KMessageBox::Yes )
				if ( ! saveTrSubtitle() )
					return false;
		}

		if ( m_subtitle->isPrimaryDirty() )
		{
			int result = KMessageBox::warningYesNoCancel(
				0,
				i18n( "Currently opened subtitle has unsaved changes.\nDo you want to save them?" ),
				i18n( "Close Subtitle" ) + " - SubtitleComposer" );
			if ( result == KMessageBox::Cancel )
				return false;
			else if ( result == KMessageBox::Yes )
				if ( ! saveSubtitle() )
					return false;
		}

		if ( m_translationMode )
		{
			m_translationMode = false;
			emit translationModeChanged( false );
		}

		emit subtitleClosed();

		delete m_subtitle;
		m_subtitle = 0;

		m_subtitlePath = QString::null;
		m_subtitleFileName = QString::null;
		m_subtitleEncoding = QString::null;
		m_subtitleFormat = QString::null;

		m_subtitleTrPath = QString::null;
		m_subtitleTrFileName = QString::null;
		m_subtitleTrEncoding = QString::null;
		m_subtitleTrFormat = QString::null;

		updateTitle();
	}

	return true;
}


void Application::newTrSubtitle()
{
	if ( ! closeTrSubtitle() )
		return;

	m_translationMode = true;
	emit translationModeChanged( true );

	updateTitle();
}

void Application::openTrSubtitle()
{
	if ( ! m_subtitle )
		return;

	OpenSubtitleDialog openDlg( false, m_lastSubtitleDir, generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING ) );

	if ( openDlg.exec() == QDialog::Accepted )
	{
		if ( ! checkDifferentNames( m_subtitlePath, openDlg.selectedFile() ) )
			return;

		if ( ! closeTrSubtitle() )
			return;

		m_lastSubtitleDir = openDlg.selectedFile();

		KURL fileURL;
		fileURL.setProtocol( "file" );
		fileURL.setPath( openDlg.selectedFile() );
		fileURL.addQueryItem( "encoding", openDlg.selectedEncoding() );
		openTrSubtitle( fileURL );
	}
}

void Application::openTrSubtitle( const KURL& url )
{
	if ( ! m_subtitle )
		return;

	//if ( ! checkDifferentNames( m_subtitlePath, openDlg.selectedFile() ) )
	//	return;

	if ( ! closeTrSubtitle() )
		return;

	if ( url.protocol() != "file" )
		return;

	QString path = url.path();

	QString encoding = url.queryItem( "encoding" );
	if ( encoding.isEmpty() )
		encoding = generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING );

	bool codecFound = true;
	QTextCodec* codec = KGlobal::charsets()->codecForName( encoding, codecFound );
	if ( ! codecFound )
		codec = KGlobal::locale()->codecForEncoding();

	m_linesWidget->setCurrentFollowsLinesChanges( false );

	{ // NOTE: needed scope for the composite action executor variable

		SubtitleCompositeActionExecutor executor( *m_subtitle, QString::null );

		if ( FormatsManager::readSubtitle( *m_subtitle, false, path, codec, (Format::NewLine*)&m_subtitleTrEOL, &m_subtitleTrFormat ) )
		{
			KURL recentURL;
			recentURL.setProtocol( "file" );
			recentURL.setPath( path );
			recentURL.addQueryItem( "encoding", codec->name() );
			m_recentTrSubtitlesAction->add( recentURL );

			m_subtitleTrPath = path;
			m_subtitleTrFileName = QFileInfo( m_subtitleTrPath ).fileName();
			m_subtitleTrEncoding = codec->name();

			QStringList subtitleStreams;
			subtitleStreams.append( i18n( "Primary" ) );
			subtitleStreams.append( i18n( "Translation" ) );
			KSelectAction* activeSubtitleStreamAction = (KSelectAction*)action( ACT_SET_ACTIVE_SUBTITLE_STREAM );
			activeSubtitleStreamAction->setItems( subtitleStreams );
			activeSubtitleStreamAction->setCurrentItem( 0 );

			m_translationMode = true;

			updateTitle();
		}
		else
			KMessageBox::sorry( mainWidget(), i18n( "Couldn't parse the subtitle file." ) );
	}

	m_linesWidget->setCurrentFollowsLinesChanges( true );

	// After the loading of the translation subtitle we must clear the history or (from
	// a user POV) it would be posible to execute (undo) actions wich would result in an
	// unexpected state.
	m_subtitle->actionManager().clearHistory();

	// We don't clear the primary dirty state because the loading of the translation
	// only changes it when actually needed (i.e., when the tranlation had more lines)
	m_subtitle->clearSecondaryDirty();

	if ( m_translationMode )
		emit translationModeChanged( true );
}

bool Application::saveTrSubtitle()
{
	if ( m_subtitleTrPath.isEmpty() || m_subtitleTrEncoding.isEmpty() || ! FormatsManager::hasOutputFormat( m_subtitleTrFormat ) )
		return saveTrSubtitleAs();

	bool codecFound = true;
	QTextCodec* codec = KGlobal::charsets()->codecForName( m_subtitleTrEncoding, codecFound );
	if ( ! codecFound )
		codec = KGlobal::locale()->codecForEncoding();

	if ( FormatsManager::writeSubtitle( *m_subtitle, false, m_subtitleTrPath, codec, (Format::NewLine)m_subtitleTrEOL, m_subtitleTrFormat, true ) )
	{
		m_subtitle->clearSecondaryDirty();

		KURL recentURL;
		recentURL.setProtocol( "file" );
		recentURL.setPath( m_subtitleTrPath );
		recentURL.addQueryItem( "encoding", codec->name() );
		m_recentTrSubtitlesAction->add( recentURL );

		updateTitle();

		return true;
	}
	else
	{
		KMessageBox::sorry( mainWidget(), i18n( "There was an error saving the translation subtitle." ) );
		return false;
	}
}

bool Application::saveTrSubtitleAs()
{
	SaveSubtitleDialog saveDlg(
		false,
		m_subtitleTrPath,
		m_subtitleTrEncoding.isEmpty() ?
			generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING ) :
			m_subtitleTrEncoding,
		m_subtitleTrEOL,
		m_subtitleTrFormat
	);

	if ( saveDlg.exec() == QDialog::Accepted )
	{
		if ( ! checkDifferentNames( m_subtitlePath, saveDlg.selectedFile() ) )
			return false;

		if ( QFile::exists( saveDlg.selectedFile() ) )
		{
			if ( KMessageBox::warningContinueCancel(
					mainWidget(),
					i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" )
						.arg( QFileInfo( saveDlg.selectedFile() ).fileName() ),
					i18n( "Overwrite File?" ),
					i18n( "Overwrite" )
				) != KMessageBox::Continue )
				return false;
		}

		m_subtitleTrPath = saveDlg.selectedFile();
		m_subtitleTrFileName = QFileInfo( m_subtitleTrPath ).baseName( true );
		m_subtitleTrEncoding = saveDlg.selectedEncoding();
		m_subtitleTrFormat = saveDlg.selectedFormat();
		m_subtitleTrEOL = saveDlg.selectedNewLine();

		return saveTrSubtitle();
	}

	return false;
}

bool Application::closeTrSubtitle()
{
	if ( m_subtitle && m_translationMode )
	{
		if ( m_translationMode && m_subtitle->isSecondaryDirty() )
		{
			int result = KMessageBox::warningYesNoCancel(
				0,
				i18n( "Currently opened translation subtitle has unsaved changes.\nDo you want to save them?" ),
				i18n( "Close Translation Subtitle" ) + " - SubtitleComposer" );
			if ( result == KMessageBox::Cancel )
				return false;
			else if ( result == KMessageBox::Yes )
				if ( ! saveTrSubtitle() )
					return false;
		}

		m_translationMode = false;
		emit translationModeChanged( false );

		updateTitle();

		m_linesWidget->setUpdatesEnabled( false );

		{ // NOTE: needed scope for the composite action executor variable
			SubtitleCompositeActionExecutor executor( *m_subtitle, QString::null );
			for ( SubtitleIterator it( *m_subtitle ); it.current(); ++it )
			{
				it.current()->setSecondaryText( QString::null );
				it.current()->setErrorFlags( SubtitleLine::SecondaryErrors, false );
			}
		}

		// The cleaning of the translations texts shouldn't be an undoable action
		m_subtitle->actionManager().popUndo();

		m_linesWidget->setUpdatesEnabled( true );
	}

	return true;
}


void Application::joinSubtitles()
{
	static QString defaultEncoding( generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING ) );
	static JoinSubtitlesDialog* dlg = new JoinSubtitlesDialog( defaultEncoding, mainWidget() );

	dlg->setTranslationMode( m_translationMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		bool codecFound;
		QTextCodec* codec = KGlobal::charsets()->codecForName( dlg->subtitleEncoding(), codecFound );
		if ( ! codecFound )
			codec = KGlobal::locale()->codecForEncoding();

		Subtitle secondSubtitle;
		bool primary = dlg->textTarget() == JoinSubtitlesDialog::Primary;
		if ( FormatsManager::readSubtitle( secondSubtitle, primary, dlg->subtitlePath(), codec ) )
		{
// 			m_linesWidget->setCurrentFollowsLinesChanges( false );

			m_subtitle->appendSubtitle( secondSubtitle, dlg->shiftTime().toMillis() );

// 			m_linesWidget->setCurrentFollowsLinesChanges( true );
		}
		else
			KMessageBox::sorry( mainWidget(), i18n( "Couldn't read the subtitle file to append." ) );
	}
}


KURL Application::saveSplittedSubtitle( const Subtitle& subtitle, const QString& srcPath, QString encoding, QString format, bool primary )
{
	KURL dstFileURL;

	if ( ! subtitle.linesCount() )
		return dstFileURL;

	QFileInfo auxFileInfo(
		srcPath.isEmpty() ?
			QFileInfo( QDir( System::tempDir() ), primary ? "untitled.srt" : "untitled-translation.srt" ) :
			srcPath
	);

	QString dstFilePath( System::newFile(
		auxFileInfo.dirPath(),
		auxFileInfo.baseName( true ) + " - splitted",
		auxFileInfo.extension( false )
	) );

	if ( encoding.isEmpty() )
		encoding = "UTF-8";

	bool codecFound;
	QTextCodec* codec = KGlobal::charsets()->codecForName( encoding, codecFound );
	if ( ! codecFound )
		codec = KGlobal::locale()->codecForEncoding();

	if ( format.isEmpty() )
		format = "SubRip";

	if ( FormatsManager::writeSubtitle( subtitle, primary, dstFilePath, codec, (Format::NewLine)(primary ? m_subtitleEOL : m_subtitleTrEOL), format, true ) )
	{
		dstFileURL.setProtocol( "file" );
		dstFileURL.setPath( dstFilePath );
		dstFileURL.addQueryItem( "encoding", codec->name() );
	}
	else
		KMessageBox::sorry(
			mainWidget(),
			primary ?
				i18n( "Couldn't write the splitted subtitle file." ) :
				i18n( "Couldn't write the splitted subtitle translation file." )
		);

	return dstFileURL;
}

void Application::splitSubtitle()
{
	static SplitSubtitleDialog* dlg = new SplitSubtitleDialog( mainWidget() );

	if ( dlg->exec() != QDialog::Accepted )
		return;

	Subtitle newSubtitle;
	m_subtitle->splitSubtitle( newSubtitle, dlg->splitTime().toMillis(), dlg->shiftNewSubtitle() );
	if ( ! newSubtitle.linesCount() )
	{
		KMessageBox::information( mainWidget(), i18n( "The specified time does not split the subtitles." ) );
		return;
	}

	KURL splittedURL = saveSplittedSubtitle(
		newSubtitle,
		m_subtitlePath,
		m_subtitleEncoding,
		m_subtitleFormat,
		true
	);

	if ( splittedURL.path().isEmpty() )
	{
		m_subtitle->actionManager().undo();
		KMessageBox::sorry( mainWidget(), i18n( "Couldn't write the splitted subtitle file." ) );
		return;
	}

	KURL splittedTrURL;
	if ( m_translationMode )
	{
		splittedTrURL = saveSplittedSubtitle(
			newSubtitle,
			m_subtitleTrPath,
			m_subtitleTrEncoding,
			m_subtitleTrFormat,
			false
		);

		if ( splittedTrURL.path().isEmpty() )
		{
			m_subtitle->actionManager().undo();
			KMessageBox::sorry( mainWidget(), i18n( "Couldn't write the splitted subtitle translation file." ) );
			return;
		}
	}

	QProcess process( QString( aboutData()->appName() ) );
	process.addArgument( splittedURL.prettyURL() );
	if ( m_translationMode )
		process.addArgument( splittedTrURL.prettyURL() );

	if ( ! process.start() )
		KMessageBox::sorry(
			mainWidget(),
			m_translationMode ? 
				i18n( "Couldn't open a new Subtitle Composer window.\nThe splitted part was saved as %1." )
					.arg( splittedURL.path() ) :
				i18n( "Couldn't open a new Subtitle Composer window.\nThe splitted parts were saved as %1 and %2." )
					.arg( splittedURL.path() )
					.arg( splittedTrURL.path() )
		);
}


void Application::insertBeforeCurrentLine()
{
	static InsertLineDialog* dlg = new InsertLineDialog( true, mainWidget() );

	dlg->setTranslationMode( m_translationMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_linesWidget->setCurrentFollowsLinesChanges( false );

		SubtitleLine* currentLine = m_linesWidget->currentLine();
		SubtitleLine* newLine = m_subtitle->insertNewLine( currentLine ? currentLine->index() : 0, false, dlg->textTarget() );

		m_linesWidget->setCurrentFollowsLinesChanges( true );

		m_linesWidget->setCurrentLine( newLine );
	}
}

void Application::insertAfterCurrentLine()
{
	static InsertLineDialog* dlg = new InsertLineDialog( false, mainWidget() );

	dlg->setTranslationMode( m_translationMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_linesWidget->setCurrentFollowsLinesChanges( false );

		SubtitleLine* currentLine = m_linesWidget->currentLine();
		SubtitleLine* newLine = m_subtitle->insertNewLine( currentLine ? currentLine->index() + 1 : 0, true, dlg->textTarget() );

		m_linesWidget->setCurrentFollowsLinesChanges( true );

		m_linesWidget->setCurrentLine( newLine, true );
	}
}

void Application::removeSelectedLines()
{
	static RemoveLinesDialog* dlg = new RemoveLinesDialog( false, mainWidget() );

	dlg->setTranslationMode( m_translationMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_linesWidget->setCurrentFollowsLinesChanges( false );

		RangeList selectionRanges = m_linesWidget->selectionRanges();
		m_subtitle->removeLines( selectionRanges, dlg->textTarget() );

		m_linesWidget->setCurrentFollowsLinesChanges( true );

		unsigned firstIndex = selectionRanges.firstIndex();
		if ( firstIndex < m_subtitle->linesCount() )
			m_linesWidget->setCurrentLine( m_subtitle->line( firstIndex ), true );
		else if ( firstIndex - 1 < m_subtitle->linesCount() )
			m_linesWidget->setCurrentLine( m_subtitle->line( firstIndex - 1 ), true );
	}
}

void Application::joinSelectedLines()
{
	const RangeList& ranges = m_linesWidget->selectionRanges();

// 	if ( ranges.count() > 1 && KMessageBox::Continue != KMessageBox::warningContinueCancel(
// 		mainWidget(),
// 		i18n( "Current selection has multiple ranges.\nContinuing will join them all." ),
// 		i18n( "Join Lines" ) ) )
// 		return;

	m_subtitle->joinLines( ranges );
}

void Application::splitSelectedLines()
{
	m_subtitle->splitLines( m_linesWidget->selectionRanges() );
}


void Application::selectAllLines()
{
	m_linesWidget->selectAll( true );
}

void Application::gotoLine()
{
	GotoLineDialog gotoLineDlg( m_linesWidget->currentIndex() + 1, m_linesWidget->childCount() );
	if ( gotoLineDlg.exec() == QDialog::Accepted )
		m_linesWidget->setCurrentLine( m_subtitle->line( gotoLineDlg.lineNumber() - 1 ), true );
}

void Application::find()
{
	m_lastFoundLine = 0;
	m_find->find( m_linesWidget->selectionRanges(), m_linesWidget->currentIndex() );
}

void Application::findNext()
{
	if ( ! m_find->findNext() )
	{
		m_lastFoundLine = 0;
		m_find->find( m_linesWidget->selectionRanges(), m_linesWidget->currentIndex() );
	}
}

void Application::findPrevious()
{
	if ( ! m_find->findPrevious() )
	{
		m_lastFoundLine = 0;
		m_find->find( m_linesWidget->selectionRanges(), m_linesWidget->currentIndex(), true );
	}
}

void Application::replace()
{
	m_replace->replace( m_linesWidget->selectionRanges(), m_linesWidget->currentIndex() );
}

void Application::spellCheck()
{
	m_spelling->spellCheck( m_linesWidget->currentIndex() );
}

void Application::findNextError()
{
	m_lastFoundLine = 0;
	m_findError->find( m_linesWidget->currentIndex(), false );
}

void Application::findPreviousError()
{
	m_lastFoundLine = 0;
	m_findError->find( m_linesWidget->currentIndex(), true );
}

void Application::checkErrors()
{
	static CheckErrorsDialog* dlg = new CheckErrorsDialog( mainWidget() );

	dlg->setTranslationMode( m_translationMode );
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Check Lines Errors" ) );

		RangeList targetRanges( m_linesWidget->targetRanges( dlg->linesTarget() ) );

		if ( dlg->clearOtherErrors() )
			m_subtitle->clearErrors( targetRanges, SubtitleLine::AllErrors & ~dlg->errorFlagsToCheck() );

		if ( dlg->clearMarks() )
			m_subtitle->setMarked( targetRanges, false );

		m_subtitle->checkErrors(
			targetRanges,
			dlg->errorFlagsToCheck(),
			errorsConfig()->optionAsInt( OPT_MIN_DURATION ),
			errorsConfig()->optionAsInt( OPT_MAX_DURATION ),
			errorsConfig()->optionAsInt( OPT_MIN_DURATION_PER_CHARACTER ),
			errorsConfig()->optionAsInt( OPT_MAX_DURATION_PER_CHARACTER ),
			errorsConfig()->optionAsInt( OPT_MAX_CHARACTERS ),
			errorsConfig()->optionAsInt( OPT_MAX_LINES )
		);
	}
}

void Application::recheckAllErrors()
{
	m_subtitle->recheckErrors(
		Range::full(),
		errorsConfig()->optionAsInt( OPT_MIN_DURATION ),
		errorsConfig()->optionAsInt( OPT_MAX_DURATION ),
		errorsConfig()->optionAsInt( OPT_MIN_DURATION_PER_CHARACTER ),
		errorsConfig()->optionAsInt( OPT_MAX_DURATION_PER_CHARACTER ),
		errorsConfig()->optionAsInt( OPT_MAX_CHARACTERS ),
		errorsConfig()->optionAsInt( OPT_MAX_LINES )
	);
}

void Application::recheckSelectedErrors()
{
	SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Check Lines Errors" ) );

	for ( SubtitleIterator it( *m_subtitle, m_errorsWidget->selectionRanges() ); it.current(); ++it )
	{
		it.current()->check(
			m_errorsWidget->lineSelectedErrorFlags( it.current()->index() ),
			errorsConfig()->optionAsInt( OPT_MIN_DURATION ),
			errorsConfig()->optionAsInt( OPT_MAX_DURATION ),
			errorsConfig()->optionAsInt( OPT_MIN_DURATION_PER_CHARACTER ),
			errorsConfig()->optionAsInt( OPT_MAX_DURATION_PER_CHARACTER ),
			errorsConfig()->optionAsInt( OPT_MAX_CHARACTERS ),
			errorsConfig()->optionAsInt( OPT_MAX_LINES )
		);
	}
}

void Application::clearErrors()
{
	static ClearErrorsDialog* dlg = new ClearErrorsDialog( mainWidget() );

	dlg->setTranslationMode( m_translationMode );
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Clear Lines Errors" ) );

		RangeList targetRanges( m_linesWidget->targetRanges( dlg->linesTarget() ) );

		if ( dlg->clearMarks() )
			m_subtitle->setMarked( targetRanges, false );

		if ( dlg->errorFlagsToClear() )
			m_subtitle->clearErrors( targetRanges, dlg->errorFlagsToClear() );
	}
}

void Application::clearAllErrors()
{
	m_subtitle->clearErrors( Range::full(), SubtitleLine::AllErrors );
}

void Application::clearSelectedErrors()
{
	SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Check Lines Errors" ) );

	for ( SubtitleIterator it( *m_subtitle, m_errorsWidget->selectionRanges() ); it.current(); ++it )
	{
		SubtitleLine* line = it.current();
		int errorFlags = m_errorsWidget->lineSelectedErrorFlags( line->index() );

		line->setErrorFlags( errorFlags, false );
	}
}

void Application::clearAllMarks()
{
	m_subtitle->setMarked( Range::full(), false );
}

void Application::clearSelectedMarks()
{
	SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Clear Lines Marks" ) );

	for ( SubtitleIterator it( *m_subtitle, m_errorsWidget->selectionRanges() ); it.current(); ++it )
		it.current()->setMarked( false );
}


void Application::showErrors()
{
	if ( ! m_errorsDialog->isShown() )
		m_errorsDialog->show();
}

void Application::showErrorsConfig()
{
	m_configDialog->show();
	m_configDialog->showPage( ConfigDialog::Errors );
}

void Application::toggleSelectedLinesMark()
{
	m_subtitle->toggleMarked( m_linesWidget->selectionRanges() );
}

void Application::toggleSelectedLinesBold()
{
	m_subtitle->toggleStyleFlag( m_linesWidget->selectionRanges(), SubtitleLine::Bold );
}

void Application::toggleSelectedLinesItalic()
{
	m_subtitle->toggleStyleFlag( m_linesWidget->selectionRanges(), SubtitleLine::Italic );
}

void Application::toggleSelectedLinesUnderline()
{
	m_subtitle->toggleStyleFlag( m_linesWidget->selectionRanges(), SubtitleLine::Underline );
}

void Application::shiftLines()
{
	static ShiftTimesDialog* dlg = new ShiftTimesDialog( mainWidget() );

	dlg->resetShiftTime();
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
		m_subtitle->shiftLines( m_linesWidget->targetRanges( dlg->linesTarget() ), dlg->shiftTimeMillis() );
}

void Application::shiftSelectedLinesForwards()
{
	m_subtitle->shiftLines(
		m_linesWidget->selectionRanges(),
		generalConfig()->optionAsInt( OPT_SHIFT_SELECTED_LINES_MSECS )
	);
}

void Application::shiftSelectedLinesBackwards()
{
	m_subtitle->shiftLines(
		m_linesWidget->selectionRanges(),
		- generalConfig()->optionAsInt( OPT_SHIFT_SELECTED_LINES_MSECS )
	);
}


void Application::adjustLines()
{
	static AdjustTimesDialog* dlg = new AdjustTimesDialog( mainWidget() );

	dlg->setFirstLineTime( m_subtitle->firstLine()->showTime() );
	dlg->setLastLineTime( m_subtitle->lastLine()->showTime() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->adjustLines(
			Range::full(),
			dlg->firstLineTime().toMillis(),
			dlg->lastLineTime().toMillis()
		);
	}
}

void Application::sortLines()
{
	static FinalTargetableLinesActionDialog* dlg = new FinalTargetableLinesActionDialog(
		i18n( "Sort Lines" ),
		mainWidget()
	);

	dlg->setLinesTargetEnabled( TargetableLinesActionDialog::Selection, true );

	if ( m_linesWidget->selectionHasMultipleRanges() )
	{
		if ( m_linesWidget->showingContextMenu() )
		{
			KMessageBox::sorry( mainWidget(), i18n( "Can't perform sort on selection with multiple ranges." ) );
			return;
		}
		else
			dlg->setLinesTargetEnabled( TargetableLinesActionDialog::Selection, false );
	}

	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		//m_linesWidget->setCurrentFollowsLinesChanges( false );

		RangeList targetRanges( m_linesWidget->targetRanges( dlg->linesTarget() ) );
		if ( ! targetRanges.empty() )
			m_subtitle->sortLines( *targetRanges.begin() );

		//m_linesWidget->setCurrentFollowsLinesChanges( true );
	}
}


void Application::changeFrameRate()
{
	static ChangeFrameRateDialog* dlg = new ChangeFrameRateDialog( m_subtitle->framesPerSecond(), mainWidget() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->changeFramesPerSecond(
			dlg->toFramesPerSecond(),
			dlg->fromFramesPerSecond()
		);
	}
}

void Application::durationLimits()
{
	static DurationLimitsDialog* dlg = new DurationLimitsDialog(
		errorsConfig()->optionAsInt( OPT_MIN_DURATION ),
		errorsConfig()->optionAsInt( OPT_MAX_DURATION ),
		mainWidget()
	);

	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->applyDurationLimits(
			m_linesWidget->targetRanges( dlg->linesTarget() ),
			dlg->enforceMinDuration() ? dlg->minDuration() : 0,
			dlg->enforceMaxDuration() ? dlg->maxDuration() : Time::MAX_MSECONDS,
			! dlg->preventOverlap()
		);
	}
}

void Application::autoDurations()
{
	static AutoDurationsDialog* dlg = new AutoDurationsDialog( 60, 50, 50, mainWidget() );

	dlg->setTranslationMode( m_translationMode );
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->setAutoDurations(
			m_linesWidget->targetRanges( dlg->linesTarget() ),
			dlg->charMillis(),
			dlg->wordMillis(),
			dlg->lineMillis(),
			! dlg->preventOverlap(),
			dlg->textTarget()
		);
	}
}

void Application::maximizeDurations()
{
	static FinalTargetableLinesActionDialog* dlg = new FinalTargetableLinesActionDialog(
		i18n( "Maximize Durations" ),
		mainWidget()
	);

	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
		m_subtitle->setMaximunDurations( m_linesWidget->targetRanges( dlg->linesTarget() ) );
}

void Application::fixOverlappingLines()
{
	static FixOverlappingTimesDialog* dlg = new FixOverlappingTimesDialog( mainWidget() );

	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
		m_subtitle->fixOverlappingLines( m_linesWidget->targetRanges( dlg->linesTarget() ), dlg->minimumInterval() );
}

void Application::syncWithSubtitle()
{
	static SyncSubtitlesDialog* dlg = new SyncSubtitlesDialog(
		generalConfig()->option( OPT_DEFAULT_SUBS_ENCODING ),
		mainWidget()
	);

	if ( dlg->exec() == QDialog::Accepted )
	{

		bool codecFound = true;
		QTextCodec* codec = KGlobal::charsets()->codecForName( dlg->subtitleEncoding(), codecFound );
		if ( ! codecFound )
			codec = KGlobal::locale()->codecForEncoding();

		Subtitle referenceSubtitle;
		if ( FormatsManager::readSubtitle( referenceSubtitle, true, dlg->subtitlePath(), codec ) )
		{
			if ( dlg->adjustToReferenceSubtitle() )
			{
				if ( referenceSubtitle.linesCount() <= 1 )
					KMessageBox::sorry( mainWidget(), i18n( "The reference subtitle must have more than one line to proceed." ) );
				else
					m_subtitle->adjustLines(
						Range::full(),
						referenceSubtitle.firstLine()->showTime().toMillis(),
						referenceSubtitle.lastLine()->showTime().toMillis()
					);
			}
			else // if ( dlg->synchronizeToReferenceTimes() )
				m_subtitle->syncToSubtitle( referenceSubtitle );
		}
		else
			KMessageBox::sorry( mainWidget(), i18n( "Couldn't parse the reference subtitle file." ) );
	}
}

void Application::adjustTexts()
{
	static SmartTextsAdjustDialog* dlg = new SmartTextsAdjustDialog( 30, mainWidget() );

	dlg->setTranslationMode( m_translationMode );
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
		m_subtitle->adjustTexts(
			m_linesWidget->targetRanges( dlg->linesTarget() ),
			dlg->minLengthForLineBreak(),
			dlg->textTarget()
		);
}

void Application::unbreakTexts()
{
	static FinalTargetableTextsActionDialog* dlg = new FinalTargetableTextsActionDialog(
		i18n( "Unbreak Lines" ),
		mainWidget()
	);

	dlg->setTranslationMode( m_translationMode );
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
		m_subtitle->unbreakTexts(
			m_linesWidget->targetRanges( dlg->linesTarget() ),
			dlg->textTarget()
		);
}

void Application::simplifySpaces()
{
	static FinalTargetableTextsActionDialog* dlg = new FinalTargetableTextsActionDialog(
		i18n( "Simplify Spaces" ),
		mainWidget()
	);

	dlg->setTranslationMode( m_translationMode );
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
		m_subtitle->simplifySpaces(
			m_linesWidget->targetRanges( dlg->linesTarget() ),
			dlg->textTarget()
		);
}

void Application::changeCase()
{
	static ChangeTextsCaseDialog* dlg = new ChangeTextsCaseDialog( mainWidget() );

	dlg->setTranslationMode( m_translationMode );
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		switch ( dlg->caseOperation() )
		{
			case ChangeTextsCaseDialog::Upper:
				m_subtitle->upperCase(
					m_linesWidget->targetRanges( dlg->linesTarget() ),
					dlg->textTarget()
				);
				break;
			case ChangeTextsCaseDialog::Lower:
				m_subtitle->lowerCase(
					m_linesWidget->targetRanges( dlg->linesTarget() ),
					dlg->textTarget()
				);
				break;
			case ChangeTextsCaseDialog::Title:
				m_subtitle->titleCase(
					m_linesWidget->targetRanges( dlg->linesTarget() ),
					dlg->lowerFirst(),
					dlg->textTarget()
				);
				break;
			case ChangeTextsCaseDialog::Sentence:
				m_subtitle->sentenceCase(
					m_linesWidget->targetRanges( dlg->linesTarget() ),
					dlg->lowerFirst(),
					dlg->textTarget()
				);
				break;
		}
	}
}

void Application::fixPunctuation()
{
	static FixPunctuationDialog* dlg = new FixPunctuationDialog( mainWidget() );

	dlg->setTranslationMode( m_translationMode );
	dlg->setSelectionTargetOnlyMode( m_linesWidget->showingContextMenu() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->fixPunctuation(
			m_linesWidget->targetRanges( dlg->linesTarget() ),
			dlg->spaces(),
			dlg->quotes(),
			dlg->englishI(),
			dlg->ellipisis(),
			dlg->textTarget()
		);
	}
}

void Application::openVideo( const KURL& url )
{
	if ( url.protocol() != "file" )
		return;

	m_player->closeFile();

	m_player->openFile( url.path() );
}

void Application::openVideo()
{
	KFileDialog openDlg( m_lastVideoDir, buildMediaFilesFilter(), mainWidget(), 0, true );
	openDlg.setCaption( i18n( "Open Video" ) );

	if ( openDlg.exec() == QDialog::Accepted )
	{
		m_lastVideoDir = openDlg.selectedFile();
		openVideo( openDlg.selectedURL() );
	}
}

void Application::toggleFullScreenMode()
{
	setFullScreenMode( ! m_playerWidget->fullScreenMode() );
}

void Application::setFullScreenMode( bool enabled )
{
	if ( enabled != m_playerWidget->fullScreenMode() )
	{
		m_playerWidget->setFullScreenMode( enabled );

		KToggleActionExt* toggleFullScreenAction = (KToggleActionExt*)action( ACT_TOGGLE_FULL_SCREEN );
		toggleFullScreenAction->setChecked( enabled );

		emit fullScreenModeChanged( enabled );
	}
}

void Application::seekBackwards()
{
	double position = m_player->position() - playerConfig()->optionAsInt( OPT_JUMP_ON_SEEK_SECS );
	m_player->seek( position > 0.0 ? position : 0.0, false );
}

void Application::seekForwards()
{
	double position = m_player->position() + playerConfig()->optionAsInt( OPT_JUMP_ON_SEEK_SECS );
	m_player->seek( position <= m_player->length() ? position : m_player->length(), false );
}

void Application::seekToPrevLine()
{
	SubtitleLine* overlayLine = m_playerWidget->overlayLine();
	if ( overlayLine )
	{
		SubtitleLine* prevLine = overlayLine->prevLine();
		if ( prevLine )
		{
			m_player->seek( (prevLine->showTime().toMillis()) / 1000.0, true );
			m_linesWidget->setCurrentLine( prevLine );
			m_curLineWidget->setCurrentLine( prevLine );
		}
	}
}

void Application::seekToNextLine()
{
	SubtitleLine* overlayLine = m_playerWidget->overlayLine();
	if ( overlayLine )
	{
		SubtitleLine* nextLine = overlayLine->nextLine();
		if ( nextLine )
		{
			m_player->seek( (nextLine->showTime().toMillis()) / 1000.0, true );
			m_linesWidget->setCurrentLine( nextLine );
			m_curLineWidget->setCurrentLine( nextLine );
		}
	}
}

void Application::setCurrentLineShowTimeFromVideo()
{
	SubtitleLine* currentLine = m_linesWidget->currentLine();
	if ( currentLine )
		currentLine->setShowTime( videoPosition( true ) );
}

void Application::setCurrentLineHideTimeFromVideo()
{
	SubtitleLine* currentLine = m_linesWidget->currentLine();
	if ( currentLine )
	{
		currentLine->setHideTime( videoPosition( true ) );
		SubtitleLine* nextLine = currentLine->nextLine();
		if ( nextLine )
			m_linesWidget->setCurrentLine( nextLine, true );
	}
}

void Application::setActiveSubtitleStream( int subtitleStream )
{
	KSelectAction* activeSubtitleStreamAction = (KSelectAction*)action( ACT_SET_ACTIVE_SUBTITLE_STREAM );
 	activeSubtitleStreamAction->setCurrentItem( subtitleStream );

	m_playerWidget->setShowTranslation( subtitleStream ? true : false );
}

void Application::shiftToVideoPosition()
{
	SubtitleLine* currentLine = m_linesWidget->currentLine();
	if ( currentLine )
	{
		m_subtitle->shiftLines(
			Range::full(),
			videoPosition( true ).toMillis() - currentLine->showTime().toMillis()
		);
	}
}

void Application::adjustToVideoPositionAnchorLast()
{
	SubtitleLine* currentLine = m_linesWidget->currentLine();
	if ( currentLine )
	{
		long lastLineTime = m_subtitle->lastLine()->showTime().toMillis();

		long oldCurrentLineTime = currentLine->showTime().toMillis();
		long oldDeltaTime = lastLineTime - oldCurrentLineTime;

		if ( ! oldDeltaTime )
			return;

		long newCurrentLineTime = videoPosition( true ).toMillis();
		long newDeltaTime = lastLineTime - newCurrentLineTime;

		double scaleFactor = (double)newDeltaTime/oldDeltaTime;
		long shiftTime = (long)(newCurrentLineTime-scaleFactor*oldCurrentLineTime);

		long newFirstLineTime = (long)(shiftTime + m_subtitle->firstLine()->showTime().toMillis()*scaleFactor);

		if ( newFirstLineTime < 0 )
		{
			if ( KMessageBox::warningContinueCancel(
					mainWidget(),
					i18n( "Continuing would result in loss of timing information for some lines.\nAre you sure you want to continue?" )
				) != KMessageBox::Continue )
				return;
		}

		m_subtitle->adjustLines(
			Range::full(),
			newFirstLineTime,
			lastLineTime
		);
	}
}

void Application::adjustToVideoPositionAnchorFirst()
{
	SubtitleLine* currentLine = m_linesWidget->currentLine();
	if ( currentLine )
	{
		long firstLineTime = m_subtitle->firstLine()->showTime().toMillis();

		long oldCurrentLineTime = currentLine->showTime().toMillis();
		long oldDeltaTime = oldCurrentLineTime - firstLineTime;

		if ( ! oldDeltaTime )
			return;

		long newCurrentLineTime = videoPosition( true ).toMillis();
		long newDeltaTime = newCurrentLineTime - firstLineTime;

		double scaleFactor = (double)newDeltaTime/oldDeltaTime;
		long shiftTime = (long)(firstLineTime-scaleFactor*firstLineTime);

		long newLastLineTime = (long)(shiftTime + m_subtitle->lastLine()->showTime().toMillis()*scaleFactor);

		if ( newLastLineTime > Time::MAX_MSECONDS )
		{
			if ( KMessageBox::warningContinueCancel(
					mainWidget(),
					i18n( "Continuing would result in loss of timing information for some lines.\nAre you sure you want to continue?" )
				) != KMessageBox::Continue )
				return;
		}

		m_subtitle->adjustLines(
			Range::full(),
			firstLineTime,
			newLastLineTime
		);
	}
}

void Application::openAudioLevels()
{
	KFileDialog openDlg( m_lastAudioLevelsDir, buildLevelsFilesFilter(), mainWidget(), 0, true );
	openDlg.setCaption( i18n( "Open AudioLevels" ) );

	if ( openDlg.exec() == QDialog::Accepted )
	{
		m_lastAudioLevelsDir = openDlg.selectedFile();
		openAudioLevels( openDlg.selectedURL() );
	}

}

void Application::openAudioLevels( const KURL& url )
{
	if ( url.protocol() != "file" )
		return;

	closeAudioLevels();

	m_audiolevels = new AudioLevels();
// 	if ( m_audiolevels->loadFromMedia( url.path(), true ) )
	if ( m_audiolevels->load( url.path() ) )
	{
		m_recentAudioLevelssAction->add( url );

		emit audiolevelsOpened( m_audiolevels );
	}
	else
	{
		delete m_audiolevels;
		m_audiolevels = 0;

		KMessageBox::sorry( mainWidget(), i18n( "There was an error opening the audiolevels." ) );
	}
}

void Application::saveAudioLevelsAs()
{
	KFileDialog saveDlg( m_lastAudioLevelsDir, QString::null, mainWidget(), 0, true );
	saveDlg.setCaption( i18n( "Save AudioLevels" ) );
	saveDlg.setOperationMode( KFileDialog::Saving );

	if ( saveDlg.exec() == QDialog::Accepted )
	{
		if ( QFile::exists( saveDlg.selectedFile() ) )
		{
			if ( KMessageBox::warningContinueCancel(
					mainWidget(),
					i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" )
						.arg( QFileInfo( saveDlg.selectedFile() ).fileName() ),
					i18n( "Overwrite File?" ),
					i18n( "Overwrite" )
				) != KMessageBox::Continue )
				return;
		}

		if ( ! m_audiolevels->save( saveDlg.selectedFile(), true ) )
			KMessageBox::sorry( mainWidget(), i18n( "There was an error saving the audiolevels." ) );
	}
}

void Application::closeAudioLevels()
{
	delete m_audiolevels;
	m_audiolevels = 0;

	emit audiolevelsClosed();
}

void Application::increaseAudioLevelsVZoom()
{
}

void Application::decreaseAudioLevelsVZoom()
{
}

void Application::increaseAudioLevelsHZoom()
{
}

void Application::decreaseAudioLevelsHZoom()
{
}



/// END ACTION HANDLERS


void Application::updateTitle()
{
	if ( m_subtitle )
	{
		if ( m_translationMode )
		{
			static const QString titleBuilder( "%1%2 | %3%4" );
			static const QString modified = QString::fromUtf8( " [" ) + i18n( "modified" ) + QString::fromUtf8( "]" );

			m_mainWindow->setCaption(
				titleBuilder
					.arg( m_subtitlePath.isEmpty() ? i18n( "Untitled" ) : m_subtitleFileName )
					.arg( m_subtitle->isPrimaryDirty() ? modified : QString::null )
					.arg( m_subtitleTrPath.isEmpty() ? i18n( "Untitled Translation" ) : m_subtitleTrFileName )
					.arg( m_subtitle->isSecondaryDirty() ? modified : QString::null ),
				false
			);
		}
		else
		{
			m_mainWindow->setCaption(
				m_subtitlePath.isEmpty() ? i18n( "Untitled" ) : m_subtitlePath,
				m_subtitle->isPrimaryDirty()
			);
		}
	}
	else
		m_mainWindow->setCaption( QString::null );
}

void Application::onLineDoubleClicked( SubtitleLine* line )
{
	if ( m_player->state() == Player::Ready )
		m_player->play();

	int mseconds = line->showTime().toMillis() - generalConfig()->optionAsInt( OPT_JUMP_TO_POSITION );
	m_player->seek( mseconds > 0 ? mseconds / 1000.0 : 0.0, true );

	if ( m_player->state() == Player::Paused )
		m_player->play();
}

void Application::onHighlightLine( SubtitleLine* line, bool primary, int firstIndex, int lastIndex )
{
	if ( m_playerWidget->fullScreenMode() )
	{
		if ( m_lastFoundLine != line )
		{
			m_lastFoundLine = line;

			m_player->seek( line->showTime().toMillis() / 1000.0, true );
		}
	}
	else
	{
		m_linesWidget->setCurrentLine( line, true );

		if ( firstIndex >= 0 && lastIndex >= 0 )
		{
			if ( primary )
				m_curLineWidget->highlightPrimary( firstIndex, lastIndex );
			else
				m_curLineWidget->highlightSecondary( firstIndex, lastIndex );
		}
	}
}

void Application::onPlayingLineChanged( SubtitleLine* line )
{
	m_linesWidget->setPlayingLine( line );

	if ( m_linkCurrentLineToPosition )
		m_linesWidget->setCurrentLine( line, true );
}

void Application::onLinkCurrentLineToVideoToggled( bool value )
{
	if ( m_linkCurrentLineToPosition != value )
	{
		m_linkCurrentLineToPosition = value;

		if ( m_linkCurrentLineToPosition )
			m_linesWidget->setCurrentLine( m_playerWidget->playingLine(), true );
	}
}

void Application::onPlayerFileOpened( const QString& filePath )
{
	m_recentVideosAction->add( KURL( filePath ) );
}

void Application::onPlayerPlaying()
{
	KAction* playPauseAction = action( ACT_PLAY_PAUSE );
	playPauseAction->setIcon( "player_pause" );
	playPauseAction->setText( i18n( "Pause" ) );
}

void Application::onPlayerPaused()
{
	KAction* playPauseAction = action( ACT_PLAY_PAUSE );
	playPauseAction->setIcon( "player_play" );
	playPauseAction->setText( i18n( "Play" ) );
}

void Application::onPlayerStopped()
{
	KAction* playPauseAction = action( ACT_PLAY_PAUSE );
	playPauseAction->setIcon( "player_play" );
	playPauseAction->setText( i18n( "Play" ) );
}

void Application::onPlayerAudioStreamsChanged( const QStringList& audioStreams )
{
	KSelectAction* activeAudioStreamAction = (KSelectAction*)action( ACT_SET_ACTIVE_AUDIO_STREAM );
	activeAudioStreamAction->setItems( audioStreams );
}

void Application::onPlayerActiveAudioStreamChanged( int audioStream )
{
	KSelectAction* activeAudioStreamAction = (KSelectAction*)action( ACT_SET_ACTIVE_AUDIO_STREAM );
	if ( audioStream >= 0 )
		activeAudioStreamAction->setCurrentItem( audioStream );
}

void Application::onPlayerMutedChanged( bool muted )
{
	KToggleActionExt* toggleMutedAction = (KToggleActionExt*)action( ACT_TOGGLE_MUTED );
	toggleMutedAction->setChecked( muted );
}

void Application::onPlayerOptionChanged( const QString& option, const QString& value )
{
	if ( option == OPT_JUMP_ON_SEEK_SECS )
	{
		action( "seek_backwards" )->setToolTip( i18n(
			"Seek backwards 1 second",
			"Seek backwards %n seconds",
			value.toInt()
		) );

		action( "seek_forwards" )->setToolTip( i18n(
			"Seek forwards 1 second",
			"Seek forwards %n seconds",
			value.toInt()
		) );
	}
}

void Application::showConfirmQuit()
{
	if ( ! closeSubtitle() )
		return;

	saveConfig();
	quit();
}

void Application::updateConfigFromDialog()
{
	m_config = m_configDialog->config();

	// We have to manually update the player backends configurations because the have their own copies
	QStringList backendNames( m_player->backendNames() );
	for ( QStringList::ConstIterator it = backendNames.begin(), end = backendNames.end(); it != end; ++it )
		m_player->backend( *it )->setConfigGroup( *m_config.group( *it ) );

	m_config.writeTo( KGlobal::config() );
}

#include "application.moc"
