/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "configdialog.h"
#include "configdefs/generalconfigwidget.h"
#include "configdefs/errorsconfigwidget.h"
#include "configdefs/playerconfigwidget.h"
#include <player.h>
#include <playerbackend.h>
#include <commondefs.h>

#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qtooltip.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qscrollview.h>

#include <klocale.h>
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <kdebug.h>

using namespace SubtitleComposer;

ConfigDialog::ConfigDialog( const AppConfig& config, QWidget* parent ):
	KDialogBase::KDialogBase(	KDialogBase::IconList,	// dialogFace
								0,						// flags
								parent,					// parent
								0,						// name
								false,					// modal
								"Configure",			// caption
								Ok|Apply|Cancel|Default,// buttonMask
								Ok, 					// defaultButton
								true ),					// separator
	m_acceptedConfig( config ),
	m_config( config )
{
	showButton( Default, true );

	KIconLoader iconLoader;

	m_configWidgets.insert( General, new GeneralConfigWidget( *m_config.group( "General" ) ) );
	addPage(
		m_configWidgets.at( General ),
		i18n( "General" ),
		 iconLoader.loadIcon( "misc", KIcon::Desktop ),
		i18n( "General Settings" )
	);

	m_configWidgets.insert( Errors, new ErrorsConfigWidget( *m_config.group( "Errors" ) ) );
	addPage(
		m_configWidgets.at( Errors ),
		i18n( "Errors" ),
		 iconLoader.loadIcon( "button_ok", KIcon::Desktop ),
		i18n( "Error Checks Settings" )
	);

	m_configWidgets.insert( Player, new PlayerConfigWidget( *m_config.group( "Player" ) ) );
	addPage(
		m_configWidgets.at( Player ),
		i18n( "Player" ),
// 		iconLoader.loadIcon( "player", KIcon::Desktop ),
		GUI::customIcon( QString( "player" ), 48 ),
		i18n( "Player Settings" )
	);

	unsigned idx = FirstBackend;
	QStringList backendNames( Player::instance()->backendNames() );
	for ( QStringList::ConstIterator it = backendNames.begin(), end = backendNames.end(); it != end; ++it, ++idx )
	{
		m_configWidgets.append( Player::instance()->backend( *it )->newAppConfigGroupWidget( 0 ) );
		addPage(
			m_configWidgets.at( idx ),
			*it,
			GUI::customIcon( QString( "logo-" ) + (*it).lower(), 48 ),
			i18n( "%1 Backend Settings" ).arg( *it )
		);
	}

	setControlsFromConfig();
	enableButtonApply( false );

	connect( &config, SIGNAL( optionChanged( const QString&, const QString&, const QString& ) ),
			 this, SLOT( onOptionChanged( const QString&, const QString&, const QString& ) ) );

	connect( this, SIGNAL( applyClicked(void) ), this, SLOT( acceptConfig(void) ) );
	connect( this, SIGNAL( okClicked(void) ), this, SLOT( acceptConfigAndClose(void) ) );
	connect( this, SIGNAL( cancelClicked(void) ), this, SLOT( rejectConfigAndClose(void) ) );
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( setActivePageControlsFromDefaults(void) ) );
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( enableApply(void) ) );

	// Apply button connections/setup
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( enableApply(void) ) );
	for ( QPtrListIterator<AppConfigGroupWidget> it( m_configWidgets ); it.current(); ++it )
		connect( it.current(), SIGNAL( settingsChanged() ), this, SLOT( enableApply(void) ) );

	QSize size = minimumSizeHint();
	resize( size.width() + 35, size.height() + 35 );
}

ConfigDialog::~ConfigDialog()
{
	m_configWidgets.setAutoDelete( false );
}

const AppConfig& ConfigDialog::config() const
{
	return m_acceptedConfig;
}

void ConfigDialog::setConfig( const AppConfig& config )
{
	bool visible = isVisible();

	if ( visible )
		hide();

	m_acceptedConfig = m_config = config;

	setControlsFromConfig();
	enableButtonApply( false );

	if ( visible )
		show();
}

void ConfigDialog::show()
{
	KDialogBase::show();
}

void ConfigDialog::hide()
{
	KDialogBase::hide();
}

void ConfigDialog::setControlsFromConfig()
{
	for ( unsigned pageIndex = 0; pageIndex < m_configWidgets.count(); ++pageIndex )
		setControlsFromConfig( pageIndex );
}

void ConfigDialog::setControlsFromConfig( unsigned pageIndex )
{
	if ( pageIndex < m_configWidgets.count() )
	{
		AppConfigGroup* configGroup = m_config.group( m_configWidgets.at( pageIndex )->configGroup().name() );
		if ( configGroup )
		{
			m_configWidgets.at( pageIndex )->setConfigGroup( *configGroup );
			m_configWidgets.at( pageIndex )->setControlsFromConfig();
		}
	}
}

void ConfigDialog::setActivePageControlsFromDefaults()
{
	setControlsFromDefaults( activePageIndex() );
}

void ConfigDialog::setControlsFromDefaults( unsigned pageIndex )
{
	if ( pageIndex < m_configWidgets.count() )
		m_configWidgets.at( pageIndex )->setControlsFromDefaults();
}


void ConfigDialog::setConfigFromControls()
{
	for ( unsigned pageIndex = 0; pageIndex < m_configWidgets.count(); ++pageIndex )
		setConfigFromControls( pageIndex );
}

void ConfigDialog::setConfigFromControls( unsigned pageIndex )
{
	if ( pageIndex < m_configWidgets.count() )
	{
		m_configWidgets.at( pageIndex )->setConfigFromControls();
		m_config.setGroup( m_configWidgets.at( pageIndex )->configGroup() ); // replace the group
	}
}

void ConfigDialog::acceptConfig()
{
	setConfigFromControls();
	m_acceptedConfig = m_config;

	enableButtonApply( false );

	emit accepted();
}

void ConfigDialog::acceptConfigAndClose()
{
	acceptConfig();
	hide();
}

void ConfigDialog::rejectConfig()
{
	m_config = m_acceptedConfig;
	setControlsFromConfig();
	enableButtonApply( false );
}

void ConfigDialog::rejectConfigAndClose()
{
	hide();
	rejectConfig();
}

void ConfigDialog::addPage( QWidget* page, const QString& itemName, const QPixmap& pixmap, const QString& header )
{
	QVBox *frame = addVBoxPage( itemName, header, pixmap );
	frame->setSpacing( 0 );
	frame->setMargin( 0 );
	page->reparent( (QWidget*)frame, 0, QPoint() );
}

void ConfigDialog::addPage( QWidget* page, const QString& itemName, const QString& pixmapName, const QString& header )
{
	QVBox *frame = addVBoxPage( itemName, header, pixmapName );
	frame->setSpacing( 0 );
	frame->setMargin( 0 );
	page->reparent( (QWidget*)frame, 0, QPoint() );
}

void ConfigDialog::enableApply()
{
	enableButtonApply( true );
}

void ConfigDialog::onOptionChanged( const QString& groupName, const QString& optionName, const QString& value )
{
	if ( ! m_config.group( groupName ) )
		return;

	m_config.group( groupName )->setOption( optionName, value );
	m_acceptedConfig.group( groupName )->setOption( optionName, value );

	bool applyButtonEnabled = actionButton( KDialogBase::Apply )->isEnabled();

	for ( unsigned pageIndex = 0; pageIndex < m_configWidgets.count(); ++pageIndex )
	{
		AppConfigGroup* configGroup = m_acceptedConfig.group( m_configWidgets.at( pageIndex )->configGroup().name() );
		if ( configGroup )
		{
			m_configWidgets.at( pageIndex )->setConfigGroup( *configGroup );
			m_configWidgets.at( pageIndex )->setControlsFromConfig();
		}
	}

	if ( ! applyButtonEnabled )
		enableButtonApply( false );
}
