/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "currentlinewidget.h"
#include <timeedit.h>
#include <textedit.h>
#include <qxtsignalwaiter.h>

#include <kdebug.h>
#include <klocale.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qgroupbox.h>

using namespace SubtitleComposer;

CurrentLineWidget::CurrentLineWidget( QWidget* parent ):
	QWidget( parent ),
	m_currentLine( 0 ),
	m_translationMode( false ),
	m_updateCurrentLine( true ),
	m_updateControls( true )
{
	m_primaryTextLabel = new QLabel( this );
	m_primaryTextLabel->setAlignment( Qt::AlignLeft|Qt::WordBreak );
	m_primaryTextLabel->setTextFormat( Qt::PlainText );
	m_primaryTextEdit = new TextEdit( this );
	m_primaryTextEdit->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Maximum );
	m_primaryTextEdit->setCheckSpellingEnabled( false );
	//m_primaryTextEdit->setUndoRedoEnabled( false );

	m_secondaryTextLabel = new QLabel( this );
	m_secondaryTextLabel->setAlignment( Qt::AlignLeft|Qt::WordBreak );
	m_secondaryTextLabel->setTextFormat( Qt::PlainText );
	m_secondaryTextEdit = new TextEdit( this );
	m_secondaryTextEdit->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Maximum );
	m_secondaryTextEdit->setCheckSpellingEnabled( false );
	//m_secondaryTextEdit->setUndoRedoEnabled( false );

	QFont font = m_primaryTextLabel->font();
	font.setPointSize( font.pointSize() - 1 );
	m_primaryTextLabel->setFont( font );
	m_secondaryTextLabel->setFont( font );

	QGroupBox* timesControlsGroupBox = new QGroupBox( this );
	timesControlsGroupBox->setColumnLayout( 0, Qt::Vertical );
	timesControlsGroupBox->layout()->setSpacing( 5 );
	timesControlsGroupBox->layout()->setMargin( 6 );

	QLabel* showLabel = new QLabel( i18n( "<b>Show</b>" ), timesControlsGroupBox );
	m_showTimeEdit = new TimeEdit( timesControlsGroupBox );

	QLabel* hideLabel = new QLabel( i18n( "<b>Hide</b>" ), timesControlsGroupBox );
	m_hideTimeEdit = new TimeEdit( timesControlsGroupBox );

	QLabel* durationLabel = new QLabel( i18n( "<b>Duration</b>" ), timesControlsGroupBox );
	m_durationTimeEdit = new TimeEdit( timesControlsGroupBox ) ;

	QGridLayout* timesControlsLayout = new QGridLayout( timesControlsGroupBox->layout() );
	timesControlsLayout->addWidget( showLabel, 0, 0 );
	timesControlsLayout->addWidget( m_showTimeEdit, 0, 1 );
	timesControlsLayout->addWidget( hideLabel, 1, 0 );
	timesControlsLayout->addWidget( m_hideTimeEdit, 1, 1 );
	timesControlsLayout->addWidget( durationLabel, 2, 0 );
	timesControlsLayout->addWidget( m_durationTimeEdit, 2, 1 );

	m_mainLayout = new QGridLayout( this );
	m_mainLayout->setMargin( 0 );
	m_mainLayout->setSpacing( 0 );
	m_mainLayout->setColSpacing( 1, 5 );
	m_mainLayout->setColSpacing( 3, 0 );

	m_mainLayout->addMultiCellWidget( timesControlsGroupBox, 0, 2, 0, 0 );
	m_mainLayout->addWidget( m_primaryTextLabel, 0, 2 );
	m_mainLayout->addWidget( m_primaryTextEdit, 2, 2 );
	m_mainLayout->addWidget( m_secondaryTextLabel, 0, 4 );
	m_mainLayout->addWidget( m_secondaryTextEdit, 2, 4 );

	int maxTextHeight = minimumSizeHint().height() - m_primaryTextLabel->minimumSizeHint().height() + 4;
	m_primaryTextEdit->setMaximumHeight( maxTextHeight );
	m_secondaryTextEdit->setMaximumHeight( maxTextHeight );

	connect( m_primaryTextEdit, SIGNAL( textChanged() ), this, SLOT( onPrimaryTextEditChanged() ) ) ;
	connect( m_secondaryTextEdit, SIGNAL( textChanged() ), this, SLOT( onSecondaryTextEditChanged() ) ) ;
	connect( m_showTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( onShowTimeEditChanged(int) ) ) ;
	connect( m_hideTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( onHideTimeEditChanged(int) ) ) ;
	connect( m_durationTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( onDurationTimeEditChanged(int) ) ) ;

	setTranslationMode( m_translationMode );
}

CurrentLineWidget::~CurrentLineWidget()
{
}

void CurrentLineWidget::loadConfig()
{
}

void CurrentLineWidget::saveConfig()
{
}

void CurrentLineWidget::setSubtitle( Subtitle* subtitle )
{
	if ( ! subtitle )
		setCurrentLine( 0 );
}

void CurrentLineWidget::setCurrentLine( SubtitleLine* line )
{
	if ( m_currentLine )
	{
		disconnect( m_currentLine, SIGNAL( primaryTextChanged( const QString& ) ),
					this, SLOT( onLinePrimaryTextChanged( const QString& ) ) );
		disconnect( m_currentLine, SIGNAL( secondaryTextChanged( const QString& ) ),
					this, SLOT( onLineSecondaryTextChanged( const QString& ) ) );
		disconnect( m_currentLine, SIGNAL( showTimeChanged( const Time& ) ),
					this, SLOT( onLineShowTimeChanged( const Time& ) ) );
		disconnect( m_currentLine, SIGNAL( hideTimeChanged( const Time& ) ),
					this, SLOT( onLineHideTimeChanged( const Time& ) ) );
		disconnect( m_currentLine, SIGNAL( styleFlagsChanged( int ) ),
					this, SLOT( onLineStyleFlagsChanged( int ) ) );
	}

	m_currentLine = line;

	if ( m_currentLine )
	{
		connect( m_currentLine, SIGNAL( primaryTextChanged( const QString& ) ),
				 this, SLOT( onLinePrimaryTextChanged( const QString& ) ) );
		connect( m_currentLine, SIGNAL( secondaryTextChanged( const QString& ) ),
				 this, SLOT( onLineSecondaryTextChanged( const QString& ) ) );
		connect( m_currentLine, SIGNAL( showTimeChanged( const Time& ) ),
				 this, SLOT( onLineShowTimeChanged( const Time& ) ) );
		connect( m_currentLine, SIGNAL( hideTimeChanged( const Time& ) ),
				 this, SLOT( onLineHideTimeChanged( const Time& ) ) );
		connect( m_currentLine, SIGNAL( styleFlagsChanged( int ) ),
				 this, SLOT( onLineStyleFlagsChanged( int ) ) );
	}

	bool enabled = m_currentLine != 0;

	onLineShowTimeChanged( enabled ? m_currentLine->showTime() : 0 );
	onLineHideTimeChanged( enabled ? m_currentLine->hideTime() : 0 );
	onLineStyleFlagsChanged( enabled ? m_currentLine->styleFlags() : 0 );

	m_primaryTextLabel->setText(
		enabled ?
			buildTextDescription( m_currentLine->primaryText() ) :
			i18n( "No current line" )
	);

	if ( m_translationMode )
		m_secondaryTextLabel->setText(
			enabled ?
				buildTextDescription( m_currentLine->secondaryText() ) :
				i18n( "No current line" )
		);

	m_primaryTextEdit->setEnabled( enabled );
	m_secondaryTextEdit->setEnabled( enabled );
	m_showTimeEdit->setEnabled( enabled );
	m_hideTimeEdit->setEnabled( enabled );
	m_durationTimeEdit->setEnabled( enabled );
}

void CurrentLineWidget::setTranslationMode( bool enabled )
{
	m_translationMode = enabled;

	if ( m_translationMode )
	{
		m_secondaryTextLabel->show();
		m_secondaryTextEdit->show();

		m_mainLayout->setColSpacing( 3, 5 );
	}
	else
	{
		m_secondaryTextLabel->hide();
		m_secondaryTextEdit->hide();

		m_mainLayout->setColSpacing( 3, 0 );
	}

	setCurrentLine( m_currentLine );
}

void CurrentLineWidget::onPrimaryTextEditChanged()
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;

		QString primaryText( m_primaryTextEdit->text() );
		m_currentLine->setPrimaryText( primaryText );
		m_primaryTextLabel->setText( buildTextDescription( primaryText ) );

		m_updateControls = true;
	}
}

void CurrentLineWidget::onSecondaryTextEditChanged()
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;

		QString secondaryText( m_secondaryTextEdit->text() );
		m_currentLine->setSecondaryText( secondaryText );
		m_secondaryTextLabel->setText( buildTextDescription( secondaryText ) );

		m_updateControls = true;
	}
}

void CurrentLineWidget::onShowTimeEditChanged( int showTime )
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;
		m_currentLine->setShowTime( showTime );
		m_durationTimeEdit->setValue( m_hideTimeEdit->value() - showTime );
		m_updateControls = true;
	}
}

void CurrentLineWidget::onHideTimeEditChanged( int hideTime )
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;
		m_currentLine->setHideTime( hideTime );
		m_durationTimeEdit->setValue( hideTime - m_showTimeEdit->value() );
		m_updateControls = true;
	}
}

void CurrentLineWidget::onDurationTimeEditChanged( int durationTime )
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;
		m_currentLine->setDurationTime( durationTime );
		m_hideTimeEdit->setValue( m_showTimeEdit->value() + durationTime );
		m_updateControls = true;
	}
}

QString CurrentLineWidget::buildTextDescription( const QString& text )
{
	// we directly use this version of i18n (instead of the one for numbers)
	// for (presumed) performance reasons (don't want to go check the dict
	// for every key pressed).

	static QString loadedText( i18n( "Current line: %1 characters" ) );

	static QString plus( " + " );
	static QString equals( " = " );

	QStringList lines( QStringList::split( "\n", text, true ) );

	QString characters;
	int totalChars = 0, lineChars;

	for ( int index = 0, lastIndex = lines.size()-2; index <= lastIndex; ++index )
	{
		lineChars = lines[index].length();
		totalChars += lineChars;
		characters += QString::number( lineChars ) + plus;
	}

	if ( lines.size() > 1 )
	{
		lineChars = lines.last().length();
		totalChars += lineChars;
		characters += QString::number( lineChars ) + equals + QString::number( totalChars );
	}
	else
	{
		if ( lines.size() )
			totalChars += lines.last().length();
		characters = QString::number( totalChars );
	}

	return QString( loadedText ).arg( characters );
}

void CurrentLineWidget::onLinePrimaryTextChanged( const QString& primaryText )
{
	if ( m_updateControls )
	{
		m_updateCurrentLine = false;

		m_primaryTextLabel->setText( buildTextDescription( primaryText ) );

		//int styleFlags = m_currentLine->styleFlags();

		m_primaryTextEdit->clear();
		//m_primaryTextEdit->setBold( styleFlags & SubtitleLine::Bold );
		//m_primaryTextEdit->setItalic( styleFlags & SubtitleLine::Italic );
		//m_primaryTextEdit->setUnderline( styleFlags & SubtitleLine::Underline );
		m_primaryTextEdit->setText( primaryText );

		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::onLineSecondaryTextChanged( const QString& secondaryText )
{
	if ( m_updateControls && m_translationMode )
	{
		m_updateCurrentLine = false;

		m_primaryTextLabel->setText( buildTextDescription( secondaryText ) );

		//int styleFlags = m_currentLine->styleFlags();

		m_secondaryTextEdit->clear();
		//m_secondaryTextEdit->setBold( styleFlags & SubtitleLine::Bold );
		//m_secondaryTextEdit->setItalic( styleFlags & SubtitleLine::Italic );
		//m_secondaryTextEdit->setUnderline( styleFlags & SubtitleLine::Underline );
		m_secondaryTextEdit->setText( secondaryText );

		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::onLineShowTimeChanged( const Time& showTime )
{
	if ( m_updateControls )
	{
		m_updateCurrentLine = false;
		m_showTimeEdit->setValue( showTime.toMillis() );
		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::onLineHideTimeChanged( const Time& hideTime )
{
	if ( m_updateControls )
	{
		m_updateCurrentLine = false;
		m_hideTimeEdit->setValue( hideTime.toMillis() );
		m_durationTimeEdit->setValue( m_hideTimeEdit->value() - m_showTimeEdit->value() );
		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::onLineStyleFlagsChanged( int styleFlags )
{
	if ( m_updateControls )
	{
		m_updateCurrentLine = false;

		m_primaryTextEdit->clear();
		m_primaryTextEdit->setBold( styleFlags & SubtitleLine::Bold );
		m_primaryTextEdit->setItalic( styleFlags & SubtitleLine::Italic );
		m_primaryTextEdit->setUnderline( styleFlags & SubtitleLine::Underline );
		m_primaryTextEdit->setText( m_currentLine ? m_currentLine->primaryText() : QString::null );

		if ( m_translationMode )
		{
			m_secondaryTextEdit->clear();
			m_secondaryTextEdit->setBold( styleFlags & SubtitleLine::Bold );
			m_secondaryTextEdit->setItalic( styleFlags & SubtitleLine::Italic );
			m_secondaryTextEdit->setUnderline( styleFlags & SubtitleLine::Underline );
			m_secondaryTextEdit->setText( m_currentLine ? m_currentLine->secondaryText() : QString::null );
		}

		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::highlightPrimary( int startIndex, int endIndex )
{
	m_primaryTextEdit->setSelection( startIndex, endIndex );
}

void CurrentLineWidget::highlightSecondary( int startIndex, int endIndex )
{
	m_secondaryTextEdit->setSelection( startIndex, endIndex );
}

#include "currentlinewidget.moc"
