/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "application.h"
#include "errorswidget.h"
#include "errorswidgetitem.h"
#include "configdefs/errorsconfigdefs.h"

#include <subtitle.h>
#include <subtitleline.h>
#include <subtitleiterator.h>

#include <qheader.h>
#include <qlayout.h>

#include <kpopupmenu.h>
#include <kdebug.h>
#include <klocale.h>

using namespace SubtitleComposer;

ErrorsWidget::ErrorsWidget( QWidget* parent ):
	KListView( parent, "ErrorsWidget" ),
	m_subtitle( 0 ),
	m_linesCount( 0 ),
	m_errorsCount( 0 ),
	m_marksCount( 0 )
{
	addColumn( i18n( "Errors Information" ) );

	setColumnWidthMode( 0, QListView::Maximum );

	header()->setClickEnabled( false );
	setSorting( -1 );

	setAllColumnsShowFocus( true );
	setSelectionMode( QListView::Extended );
	setResizeMode( QListView::LastColumn );
	setRootIsDecorated( true );
// 	setTreeStepSize( 10 );

	connect( this, SIGNAL( currentChanged(QListViewItem*) ), this, SLOT( onCurrentChanged(QListViewItem*) ) );
}

ErrorsWidget::~ErrorsWidget()
{
}

unsigned ErrorsWidget::linesCount()
{
	return m_linesCount;
}

unsigned ErrorsWidget::errorsCount()
{
	return m_errorsCount;
}

unsigned ErrorsWidget::marksCount()
{
	return m_marksCount;
}

void ErrorsWidget::setSubtitle( Subtitle* subtitle )
{
	if ( m_subtitle )
	{
		disconnect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( onLinesInserted(unsigned,unsigned) ) );
		disconnect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( onLinesRemoved(unsigned,unsigned) ) );

		clear();
	}

	m_subtitle = subtitle;

	if ( m_subtitle )
	{
		connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( onLinesInserted(unsigned,unsigned) ) );
		connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( onLinesRemoved(unsigned,unsigned) ) );

		setUpdatesEnabled( false );

		ErrorsWidgetItem* after = 0;
		for ( SubtitleIterator it( *m_subtitle ); it.current(); ++it )
			after = createItem( it.current(), after, it.index() + 1 );

		setUpdatesEnabled( true );
	}
}

void ErrorsWidget::contentsMouseDoubleClickEvent( QMouseEvent* e )
{
	QListViewItem* item = itemAt( viewport()->mapFromGlobal( e->globalPos() ) );
	if ( item != 0 )
		emit lineDoubleClicked( itemLine( item ) );
}

void ErrorsWidget::contextMenuEvent( QContextMenuEvent* e )
{
	int lines = 0, errors = 0, marks = 0;

	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ); it.current(); ++it )
	{
		if ( it.current()->parent() )
		{
			if ( (static_cast<ErrorsWidgetNestedItem*>( it.current() ))->errorFlags() )
				errors++;
			else
				marks++;
		}
		else
		{
			if ( (static_cast<ErrorsWidgetItem*>( it.current() ))->line()->errorFlags() )
				errors++;
			else
				marks++;
			lines++;
		}

		if ( lines > 1 && errors > 1 && lines > 1 )
			break;
	}

	if ( lines || errors || marks )
	{
		KPopupMenu menu;

		if ( (lines > 1 && marks) || marks > 1 )
			menu.insertItem( i18n( "Clear Marks" ), app(), SLOT( clearSelectedMarks() ) );
		else if ( marks )
			menu.insertItem( i18n( "Clear Mark" ), app(), SLOT( clearSelectedMarks() ) );

		if ( lines && errors || errors > 1 )
			menu.insertItem( i18n( "Clear Errors" ), app(), SLOT( clearSelectedErrors() ) );
		else if ( errors )
			menu.insertItem( i18n( "Clear Error" ), app(), SLOT( clearSelectedErrors() ) );

		if ( ! app()->errorsConfig()->optionAsBool( OPT_AUTO_CLEAR_FIXED ) )
		{
			if ( lines && errors || errors )
				menu.insertItem( i18n( "Recheck and Clear Fixed" ), app(), SLOT( recheckSelectedErrors() ) );
		}

		menu.exec( e->globalPos() );
		e->ignore();
	}

	KListView::contextMenuEvent( e );
}

void ErrorsWidget::setCurrentItem( QListViewItem* newCurrentItem, bool clearSelection )
{
	if ( ! newCurrentItem || ! newCurrentItem->isVisible() )
		return;

	QListViewItem* oldCurrentItem = currentItem();

	if ( clearSelection )
		this->clearSelection();
	newCurrentItem->setSelected( true );

	ensureItemVisible( newCurrentItem );
	QListViewItem* lastChild = newCurrentItem->firstChild();
	if ( lastChild )
	{
		while ( lastChild->nextSibling() )
			lastChild = lastChild->nextSibling();
		ensureItemVisible( lastChild );
	}

	if ( oldCurrentItem != newCurrentItem && newCurrentItem != oldCurrentItem->parent() )
	{
		KListView::setCurrentItem( newCurrentItem );
		emit currentChanged( newCurrentItem );
		emit selectionChanged();
	}
}

SubtitleLine* ErrorsWidget::itemLine( QListViewItem* item )
{
	return item ? (
		item->parent() ?
			(static_cast<ErrorsWidgetItem*>( item->parent() ))->line() :
			(static_cast<ErrorsWidgetItem*>( item ))->line()
		) :
		0;
}

SubtitleLine* ErrorsWidget::currentLine()
{
	return itemLine( currentItem() );
}

int ErrorsWidget::lineSelectedErrorFlags( unsigned index )
{
	ErrorsWidgetItem* item = this->item(  index );
	if ( ! item )
		return 0;

	if ( item->isSelected() )
		return item->line()->errorFlags();

	int selectedErrorFlags = 0;
	for ( QListViewItem* errorItem = item->firstChild(); errorItem != 0; errorItem = errorItem->nextSibling() )
		if ( errorItem->isSelected() )
			selectedErrorFlags |= (static_cast<ErrorsWidgetNestedItem*>(errorItem))->errorFlags();

	return selectedErrorFlags;
}

RangeList ErrorsWidget::selectionRanges() const
{
	RangeList ranges;

	int firstIndex = -1, index = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++index )
	{
		bool isSelected = item->isSelected();

		for ( QListViewItem* errorItem = item->firstChild(); errorItem != 0; errorItem = errorItem->nextSibling() )
		{
			if ( errorItem->isSelected() )
			{
				isSelected = true;
				break;
			}
		}

		if ( isSelected )
		{
			if ( firstIndex == -1 ) // mark start of selected range
				firstIndex = index;
		}
		else
		{
			if ( firstIndex != -1 )
			{
				ranges << Range( firstIndex, index - 1 );
				firstIndex = -1;
			}
		}
	}

	if ( firstIndex != -1 )
		ranges << Range( firstIndex, index - 1 );

	return ranges;
}

void ErrorsWidget::setCurrentLine( SubtitleLine* line )
{
	if ( line && line != currentLine() )
	{
		QListViewItem* newCurrentItem = item( line->index() );
		if ( ! newCurrentItem )
			return;

		setCurrentItem( newCurrentItem, true );
	}
}

ErrorsWidgetItem* ErrorsWidget::item( unsigned targetIndex )
{
	unsigned itemIndex = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++itemIndex )
	{
		if ( itemIndex == targetIndex )
			return static_cast<ErrorsWidgetItem*>( item );
	}
	return 0;
}

ErrorsWidgetItem* ErrorsWidget::createItem( SubtitleLine* line, ErrorsWidgetItem* after, unsigned number )
{
	ErrorsWidgetItem* item = new ErrorsWidgetItem( line, this, after );
	item->setNumber( number ? number : (after ? (after->line()->number() + 1) : 1) );

	return item;
}

void ErrorsWidget::onLinesInserted( unsigned firstIndex, unsigned lastIndex )
{
	unsigned index = firstIndex;
	ErrorsWidgetItem* item = index ? this->item( index - 1 ) : 0;
	for ( ; index <= lastIndex; ++index )
		item = createItem( m_subtitle->line( index ), item, index + 1 );

	// update the remaining lines numbers
	for ( item = item->nextSiblingCasted(); item != 0; item = item->nextSiblingCasted() )
		item->setNumber( ++index );
}

void ErrorsWidget::onLinesRemoved( unsigned firstIndex, unsigned lastIndex )
{
	ErrorsWidgetItem *item = this->item( firstIndex ), *itemAux;
	unsigned index = firstIndex;

	for ( ; index <= lastIndex; ++index )
	{
		itemAux = item->nextSiblingCasted();
		delete item;
		item = itemAux;
	}

	QListViewItem* currentItem( this->currentItem() );
	if ( currentItem )
	{
		if ( ! currentItem->isSelected() )
		{
			QListViewItem* itemAbove = currentItem->itemAbove();
			if ( itemAbove && itemAbove->isSelected() )
				setCurrentItem( itemAbove, false );
			else
			{
				QListViewItem* itemBelow = currentItem->itemBelow();
				if ( itemBelow && itemBelow->isSelected() )
					setCurrentItem( itemBelow, false );
			}
		}
	}

	// update the remaining lines numbers
	for ( ; item != 0; item = item->nextSiblingCasted() )
		item->setNumber( ++firstIndex );
}

void ErrorsWidget::onCurrentChanged( QListViewItem* currentItem )
{
	if ( currentItem != 0 )
	{
		if ( ! currentItem->parent() )
			emit currentLineChanged( (static_cast<ErrorsWidgetItem*>( currentItem ))->line() );
		else
			emit currentLineChanged( (static_cast<ErrorsWidgetItem*>( currentItem->parent() ))->line() );
	}
}

void ErrorsWidget::keyPressEvent( QKeyEvent* event )
{
	if ( event->key() == Qt::Key_Delete )
	{
		SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Check Lines Errors" ) );

		app()->clearSelectedErrors();
		app()->clearSelectedMarks();

		event->accept();
	}
	else
		KListView::keyPressEvent( event );
}

void ErrorsWidget::updateLinesCount( bool increment )
{
	m_linesCount += (increment ? 1 : -1);
	emit linesCountChanged( m_linesCount );
}

void ErrorsWidget::updateErrorsCount( bool increment )
{
	m_errorsCount += (increment ? 1 : -1);
	emit errorsCountChanged( m_linesCount );
}

void ErrorsWidget::updateMarksCount( bool increment )
{
	m_marksCount += (increment ? 1 : -1);
	emit marksCountChanged( m_marksCount );
}

#include "errorswidget.moc"
