/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "lineswidget.h"
#include "lineswidgetitem.h"
#include "application.h"
#include "configdefs/generalconfigdefs.h"
#include "dialogs/targetablelinesactiondialog.h"
#include <subtitleiterator.h>

#include <qheader.h>

#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kpopupmenu.h>

using namespace SubtitleComposer;

LinesWidget::LinesWidget( QWidget* parent ):
	KListView( parent, "ListView" ),
	m_subtitle( 0 ),
	m_translationMode( false ),
	m_playingItem( 0 ),
	m_nextItemCurrentOnDelete( true ),
	m_currentFollowsLinesChanges( true ),
	m_showingContextMenu( false )
{
	addColumn( i18n( "Num" ) );
	addColumn( i18n( "Show Time" ) );
	addColumn( i18n( "Hide Time" ) );
	addColumn( i18n( "Text" ) );
	addColumn( i18n( "Translation" ) );

	setColumnWidthMode( Number, QListView::Maximum );
	setColumnWidthMode( ShowTime, QListView::Maximum );
	setColumnAlignment( ShowTime, Qt::AlignHCenter );
	setColumnWidthMode( HideTime, QListView::Maximum );
	setColumnAlignment( HideTime, Qt::AlignHCenter );
	setColumnWidthMode( Text, QListView::Manual );
	setColumnWidthMode( Translation, QListView::Manual );

	header()->setClickEnabled( false );
	setSorting( -1 );

	setAllColumnsShowFocus( true );
	setSelectionMode( QListView::Extended );
	setResizeMode( QListView::NoColumn );
	setShowToolTips( true );

	connect( this, SIGNAL( currentChanged(QListViewItem*) ), this, SLOT( onCurrentChanged(QListViewItem*) ) );

	setAcceptDrops( true );
	viewport()->installEventFilter( this );

	setTranslationMode( m_translationMode );
}

LinesWidget::~LinesWidget()
{
}

void LinesWidget::editCurrentLinePrimaryInPlace()
{
	QListViewItem* currentItem = this->currentItem();
	if ( currentItem )
	{
		currentItem->setRenameEnabled( Text, true );
		currentItem->startRename( Text );
	}
}

void LinesWidget::editCurrentLineSecondaryInPlace()
{
	if ( m_translationMode )
	{
		QListViewItem* currentItem = this->currentItem();
		if ( currentItem )
		{
			currentItem->setRenameEnabled( Translation, true );
			currentItem->startRename( Translation );
		}
	}
}

bool LinesWidget::showingContextMenu()
{
	return m_showingContextMenu;
}

bool LinesWidget::currentFollowsLinesChanges()
{
	return m_currentFollowsLinesChanges;
}

void LinesWidget::setCurrentFollowsLinesChanges( bool value )
{
	m_currentFollowsLinesChanges = value;
}

LinesWidgetItem* LinesWidget::item( unsigned targetIndex )
{
	unsigned itemIndex = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++itemIndex )
	{
		if ( itemIndex == targetIndex )
			return static_cast<LinesWidgetItem*>( item );
	}
	return 0;
}

int LinesWidget::itemIndex( QListViewItem* targetItem )
{
	int itemIndex = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++itemIndex )
	{
		if ( item == targetItem )
			return itemIndex;
	}
	return -1;
}

SubtitleLine* LinesWidget::currentLine()
{
	QListViewItem* currentItem = this->currentItem();
	return currentItem ? (static_cast<LinesWidgetItem*>( currentItem ))->line() : 0;
}

int LinesWidget::currentIndex()
{
	QListViewItem* currentItem = this->currentItem();
	return currentItem ? itemIndex( currentItem ) : -1;
}

int LinesWidget::firstSelectedIndex() const
{
	int index = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++index )
		if ( item->isSelected() )
			return index;
	return -1;
}

int LinesWidget::lastSelectedIndex() const
{
	int index = 0, lastSelectedIndex = -1;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++index )
		if ( item->isSelected() )
			lastSelectedIndex = index;
	return lastSelectedIndex;
}

bool LinesWidget::selectionHasMultipleRanges() const
{
	int ranges = 0;

	int firstIndex = -1, index = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++index )
	{
		if ( item->isSelected() )
		{
			if ( firstIndex == -1 ) // mark start of selected range
				firstIndex = index;
		}
		else
		{
			if ( firstIndex != -1 )
			{
				ranges++;
				if ( ranges > 1 )
					return true;
				firstIndex = -1;
			}
		}
	}

	if ( firstIndex != -1 )
		ranges++;

	return ranges > 1;
}

RangeList LinesWidget::selectionRanges() const
{
	RangeList ranges;

	int firstIndex = -1, index = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++index )
	{
		if ( item->isSelected() )
		{
			if ( firstIndex == -1 ) // mark start of selected range
				firstIndex = index;
		}
		else
		{
			if ( firstIndex != -1 )
			{
				ranges << Range( firstIndex, index - 1 );
				firstIndex = -1;
			}
		}
	}

	if ( firstIndex != -1 )
		ranges << Range( firstIndex, index - 1 );

	return ranges;
}

RangeList LinesWidget::targetRanges( int target ) const
{
	switch ( target )
	{
		case TargetableLinesActionDialog::AllLines:
			return Range::full();
		case TargetableLinesActionDialog::Selection:
			return selectionRanges();
		case TargetableLinesActionDialog::FromSelected:
		{
			int index = firstSelectedIndex();
			return index < 0 ? RangeList() : Range::upper( index );
		}
		case TargetableLinesActionDialog::UpToSelected:
		{
			int index = lastSelectedIndex();
			return index < 0 ? RangeList() : Range::lower( index );
		}
		default:
			return RangeList();
	}
}

void LinesWidget::loadConfig()
{
	KConfigGroupSaver saver( kapp->config(), "ListView Settings" );

	restoreLayout( kapp->config(), "ListView Settings" );

	setTranslationMode( m_translationMode );
}

void LinesWidget::saveConfig()
{
	KConfigGroupSaver saver( kapp->config(), "ListView Settings" );

	saveLayout( kapp->config(), "ListView Settings" );
}


void LinesWidget::setSubtitle( Subtitle* subtitle )
{
	m_playingItem = 0;

	if ( m_subtitle )
	{
		disconnect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( onLinesInserted(unsigned,unsigned) ) );
		disconnect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( onLinesRemoved(unsigned,unsigned) ) );

		clear();
	}

	m_subtitle = subtitle;

	if ( m_subtitle )
	{
		connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( onLinesInserted(unsigned,unsigned) ) );
		connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( onLinesRemoved(unsigned,unsigned) ) );

		setUpdatesEnabled( false );

		LinesWidgetItem* after = 0;
		for ( SubtitleIterator it( *m_subtitle ); it.current(); ++it )
			after = createItem( it.current(), after, it.index() + 1 );

		setUpdatesEnabled( true );

		setCurrentItem( firstChild(), true, true );
	}
}

void LinesWidget::setTranslationMode( bool enabled )
{
	m_translationMode = enabled;

	if ( m_translationMode )
	{
		int width = columnWidth( Text );
		setColumnWidth( Text, width / 2 );
		setColumnWidth( Translation, width - columnWidth( Text ) );
		header()->setResizeEnabled( true, Translation );
	}
	else
	{
		setColumnWidth( Text, columnWidth( Text ) + columnWidth( Translation ) );
		setColumnWidth( Translation, 0 );
		header()->setResizeEnabled( false, Translation );
	}
}

void LinesWidget::setCurrentItem( QListViewItem* newCurrentItem, bool clearSelection, bool force )
{
	if ( ! newCurrentItem )
		return; // weird behavior when we change the current item and a modal dialog is shown

	QListViewItem* oldCurrentItem = currentItem();

	if ( clearSelection )
	{
		this->clearSelection();
/*		if ( oldCurrentItem )
		{
			kdDebug() << "setCurrentItem OLD " << oldCurrentItem->text( Text ) << endl;
			oldCurrentItem->setSelected( false );
		}*/
	}

	newCurrentItem->setSelected( true );

	ensureItemVisible( newCurrentItem );

	if ( force || newCurrentItem != oldCurrentItem )
	{
		KListView::setCurrentItem( newCurrentItem );
		emit currentChanged( newCurrentItem );
		emit selectionChanged();
	}
}

void LinesWidget::setCurrentLine( SubtitleLine* line, bool clearSelection )
{
	if ( line )
		setCurrentItem( item( line->index() ), clearSelection );
}


void LinesWidget::setPlayingLine( SubtitleLine* line )
{
	if ( m_playingItem && m_playingItem->line() == line )
		return;

	if ( m_playingItem )
		m_playingItem->setPlaying( false );

	m_playingItem = line ? item( line->number() - 1 ) : 0;
	if ( m_playingItem )
		m_playingItem->setPlaying( true );
}

void LinesWidget::contentsMouseDoubleClickEvent( QMouseEvent* e )
{
	QListViewItem* item = itemAt( viewport()->mapFromGlobal( e->globalPos() ) );
	if ( item != 0 )
		emit lineDoubleClicked( (static_cast<LinesWidgetItem*>( item ))->line() );
}

bool LinesWidget::eventFilter( QObject* object, QEvent* event )
{
	if ( object == viewport() )
	{
		if ( event->type() == QEvent::DragEnter )
		{
			QDragEnterEvent* dragEnterEvent = static_cast<QDragEnterEvent*>( event );
			dragEnterEvent->accept( KURLDrag::canDecode( dragEnterEvent ) );
			return true;
		}
		else if ( event->type() == QEvent::DragMove )
		{
			return true;
		}
		else if ( event->type() == QEvent::Drop )
		{
			QDropEvent* dropEvent = static_cast<QDropEvent*>( event );

			KURL::List urls;
			if ( KURLDrag::decode( dropEvent, urls ) && ! urls.isEmpty() )
			{
				for ( KURL::List::ConstIterator it = urls.begin(), end = urls.end(); it != end; ++it )
				{
					if ( (*it).protocol() != "file" )
						continue;

					app()->openSubtitle( *it );
					break;
				}
			}

			return true; // eat event
		}
		else
			return KListView::eventFilter( object, event ); // standard event processing
	}

	return KListView::eventFilter( object, event );
}

void LinesWidget::contextMenuEvent( QContextMenuEvent* e )
{
	SubtitleLine* referenceLine = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling() )
		if ( item->isSelected() )
		{
			referenceLine = (static_cast<LinesWidgetItem*>( item ))->line();
			break;
		}

	if ( ! referenceLine )
		return;

	static KIconLoader iconLoader;

	Application* app = Application::instance();

	KPopupMenu textsMenu;
	textsMenu.insertItem( i18n( "Smart Lines Adjust..." ), app, SLOT( adjustTexts() ) );
	textsMenu.insertItem( m_translationMode ? i18n( "Unbreak Lines..." ) : i18n( "Unbreak Lines" ), app, SLOT( unbreakTexts() ) );
	textsMenu.insertItem( m_translationMode ? i18n( "Simplify Spaces..." ) : i18n( "Simplify Spaces" ), app, SLOT( simplifySpaces() ) );
	textsMenu.insertItem( i18n( "Change Case..." ), app, SLOT( changeCase() ) );
	textsMenu.insertItem( i18n( "Fix Punctuation..." ), app, SLOT( fixPunctuation() ) );
	textsMenu.insertSeparator();
	textsMenu.insertItem( iconLoader.loadIcon( "spellcheck", KIcon::Small ), i18n( "Spelling..." ), app, SLOT( spellCheck() ) );

	KPopupMenu stylesMenu;
	int styleFlags = referenceLine->styleFlags();
	stylesMenu.setCheckable( true );
	stylesMenu.insertItem( iconLoader.loadIcon( "text_bold", KIcon::Small ), i18n( "Bold" ),
							app, SLOT( toggleSelectedLinesBold() ), 0, SubtitleLine::Bold );
	stylesMenu.setItemChecked( SubtitleLine::Bold, styleFlags & SubtitleLine::Bold );
	stylesMenu.insertItem( iconLoader.loadIcon( "text_italic", KIcon::Small ), i18n( "Italic" ),
							app, SLOT( toggleSelectedLinesItalic() ), 0, SubtitleLine::Italic );
	stylesMenu.setItemChecked( SubtitleLine::Italic, styleFlags & SubtitleLine::Italic );
	stylesMenu.insertItem( iconLoader.loadIcon( "text_under", KIcon::Small ), i18n( "Underline" ),
							app, SLOT( toggleSelectedLinesUnderline() ), 0, SubtitleLine::Underline );
	stylesMenu.setItemChecked( SubtitleLine::Underline, styleFlags & SubtitleLine::Underline );

	KPopupMenu timesMenu;
	QString shiftMillis( app->generalConfig()->option( OPT_SHIFT_SELECTED_LINES_MSECS ) );
	timesMenu.insertItem( i18n( "Shift +%1 Milliseconds" ).arg( shiftMillis ), app, SLOT( shiftSelectedLinesForwards() ) );
	timesMenu.insertItem( i18n( "Shift -%1 Milliseconds" ).arg( shiftMillis ), app, SLOT( shiftSelectedLinesBackwards() ) );
	timesMenu.insertSeparator();
	timesMenu.insertItem( i18n( "Shift Lines..." ), app, SLOT( shiftLines() ) );
	timesMenu.insertItem( i18n( "Sort Lines" ), app, SLOT( sortLines() ) );
	timesMenu.insertItem( i18n( "Duration Limits..." ), app, SLOT( durationLimits() ) );
	timesMenu.insertItem( i18n( "Automatic Durations..." ), app, SLOT( autoDurations() ) );
	timesMenu.insertItem( i18n( "Maximize Durations" ), app, SLOT( maximizeDurations() ) );
	timesMenu.insertItem( i18n( "Fix Overlapping Lines" ), app, SLOT( fixOverlappingLines() ) );

	KPopupMenu errorsMenu;
	errorsMenu.setCheckable( true );
	errorsMenu.insertItem( i18n( "Mark" ), app, SLOT( toggleSelectedLinesMark() ), 0, 0 );
	errorsMenu.setItemChecked( 0, referenceLine->marked() );
	errorsMenu.insertSeparator();
	errorsMenu.insertItem( i18n( "Check Errors..." ), app, SLOT( checkErrors() ) );
	errorsMenu.insertItem( i18n( "Clear Errors..." ), app, SLOT( clearErrors() ) );
	errorsMenu.insertSeparator();
	errorsMenu.insertItem( i18n( "Show Errors..." ), app, SLOT( showErrors() ) );

	KPopupMenu menu;
	menu.setCheckable( true );
	menu.insertItem( i18n( "Select All" ), app, SLOT( selectAllLines() ) );
	menu.insertSeparator();
	menu.insertItem( i18n( "Remove" ), app, SLOT( removeSelectedLines() ) );
	menu.insertItem( i18n( "Insert Before" ), app, SLOT( insertBeforeCurrentLine() ) );
	menu.insertItem( i18n( "Insert After" ), app, SLOT( insertAfterCurrentLine() ) );
	menu.insertSeparator();
	menu.insertItem( i18n( "Join Lines" ), app, SLOT( joinSelectedLines() ) );
	menu.insertItem( i18n( "Split Lines" ), app, SLOT( splitSelectedLines() ) );
	menu.insertSeparator();
	menu.insertItem( i18n( "Texts" ), &textsMenu );
	menu.insertItem( i18n( "Styles" ), &stylesMenu );
	menu.insertItem( i18n( "Times" ), &timesMenu );
	menu.insertItem( i18n( "Errors" ), &errorsMenu );

	m_showingContextMenu = true;
	menu.exec( e->globalPos() );
	m_showingContextMenu = false;

	e->ignore();

	KListView::contextMenuEvent( e );
}

LinesWidgetItem* LinesWidget::createItem( SubtitleLine* line, LinesWidgetItem* after, unsigned number )
{
	LinesWidgetItem* item = new LinesWidgetItem( line, this, after );
	item->updateNumber( number ? number : (after ? (after->line()->number() + 1) : 1) );
	return item;
}

void LinesWidget::onCurrentChanged( QListViewItem* currentItem )
{
// 	if ( m_currentFollowsLinesChanges )
		emit currentLineChanged( currentItem ? (static_cast<LinesWidgetItem*>( currentItem ))->line() : 0 );
}

void LinesWidget::onLinesInserted( unsigned firstIndex, unsigned lastIndex )
{
	unsigned index = firstIndex;
	LinesWidgetItem* item = index ? this->item( index - 1 ) : 0;
	for ( ; index <= lastIndex; ++index )
	{
		item = createItem( m_subtitle->line( index ), item, index + 1 );

		if ( m_currentFollowsLinesChanges )
		{
			setSelected( item, true );
			if ( index == lastIndex )
				setCurrentItem( item, false, true );
		}
	}

	// update the remaining lines numbers
	for ( item = item->nextSiblingCasted(); item != 0; item = item->nextSiblingCasted() )
		item->updateNumber( ++index );
}

void LinesWidget::onLinesRemoved( unsigned firstIndex, unsigned lastIndex )
{
	LinesWidgetItem *item = this->item( firstIndex ), *itemAux;
	unsigned index = firstIndex;

	for ( ; index <= lastIndex; ++index )
	{
		itemAux = item->nextSiblingCasted();
		if ( item == m_playingItem )
			m_playingItem = 0;

		delete item;
		item = itemAux;
	}

	if ( m_currentFollowsLinesChanges )
	{
		QListViewItem* currentItem( this->currentItem() );
		if ( currentItem )
		{
			if ( m_nextItemCurrentOnDelete )
				setCurrentItem( currentItem, false );
			else if ( ! currentItem->isSelected() )
			{
				QListViewItem* itemAbove = currentItem->itemAbove();
				if ( itemAbove && itemAbove->isSelected() )
					setCurrentItem( itemAbove, false );
				else
				{
					QListViewItem* itemBelow = currentItem->itemBelow();
					if ( itemBelow && itemBelow->isSelected() )
						setCurrentItem( itemBelow, false );
				}
			}
		}
		else
			emit selectionChanged();
	}
	else if ( ! childCount() )
		emit selectionChanged();

	// update the remaining lines numbers
	for ( ; item != 0; item = item->nextSiblingCasted() )
		item->updateNumber( ++firstIndex );
}

#include "lineswidget.moc"
