/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "replace.h"

#include <kdebug.h>
#include <klocale.h>

#include <kmessagebox.h>

#include <kreplacedialog.h>
#include <kreplace.h>

using namespace SubtitleComposer;

Replace::Replace( QWidget* parent ):
	QObject( parent ),
	m_subtitle( 0 ),
	m_translationMode( false ),
	m_feedPrimaryNext( true ),
	m_replace( 0 ),
	m_iterator( 0 )
{
	m_dialog = new KReplaceDialog( parent, "ReplaceDialog" );
	m_dialog->setHasSelection( true );
	m_dialog->setHasCursor( true );
	m_dialog->setOptions( m_dialog->options() | KFindDialog::FromCursor );
}

Replace::~Replace()
{
	delete m_dialog;

	setSubtitle( 0 );
}

void Replace::invalidate()
{
	delete m_replace;
	m_replace = 0;

	delete m_iterator;
	m_iterator = 0;

	m_feedPrimaryNext = true;
}

QWidget* Replace::parentWidget()
{
	return static_cast<QWidget*>( parent() );
}

void Replace::setSubtitle( Subtitle* subtitle )
{
	m_subtitle = subtitle;

	invalidate();
}

void Replace::setTranslationMode( bool enabled )
{
	m_translationMode = enabled;

	invalidate();
}

void Replace::replace( const RangeList& selectionRanges, unsigned currentIndex )
{
	invalidate();

	if ( ! m_subtitle || ! m_subtitle->linesCount() )
		return;

	if ( m_dialog->exec() != QDialog::Accepted )
		return;

	m_replace = new KReplace( m_dialog->pattern(), m_dialog->replacement(), m_dialog->options(), 0 );

	// Connect findNext signal - called when pressing the button in the dialog
	connect( m_replace, SIGNAL( findNext() ),
			 this, SLOT(advance()) );

	// Connect signals to code which handles highlighting of found text, and on-the-fly replacement.
	connect( m_replace, SIGNAL( highlight( const QString&, int, int ) ),
			 this, SLOT( onHighlight( const QString&, int, int ) ) );

	// Connect replace signal - called when doing a replacement
	connect( m_replace, SIGNAL( replace(const QString&, int, int, int) ),
			 this, SLOT( onReplace(const QString&, int, int, int) ) );

	if ( m_dialog->options() & KFindDialog::SelectedText )
	{
		m_iterator = new SubtitleIterator( *m_subtitle, selectionRanges );
		if ( m_iterator->index() < 0 ) // Invalid index means no lines in selectionRanges
			return;
	}
	else
		m_iterator = new SubtitleIterator( *m_subtitle );

	if ( m_dialog->options() & KFindDialog::FromCursor )
		m_iterator->toIndex( currentIndex );

	m_firstIndex = m_iterator->index();
	m_found = false;

	advance();
}

void Replace::advance()
{
	KFind::Result res = KFind::NoMatch;

	bool selection = m_replace->options() & KFindDialog::SelectedText;
	bool promptOnReplace =  m_replace->options() & KReplaceDialog::PromptOnReplace;

	if ( promptOnReplace )
	{
		KDialogBase* replaceNextDialog = m_replace->replaceNextDialog( true );
		if ( replaceNextDialog )
			replaceNextDialog->setModal( true );
	}

	do
	{
		if ( m_replace->needData() )
		{
			if ( m_translationMode )
			{
				m_replace->setData(
					m_feedPrimaryNext ?
						m_iterator->current()->primaryText() :
						m_iterator->current()->secondaryText()
				);
				m_feedPrimaryNext = ! m_feedPrimaryNext;
			}
			else
				m_replace->setData( m_iterator->current()->primaryText() );
		}

		res = m_replace->replace();

		if ( res == KFind::Match )
			m_found = true;
		else if ( ! m_translationMode || m_feedPrimaryNext )
		{
			++(*m_iterator);

			if ( m_firstIndex == m_iterator->index() ||
				 m_firstIndex == m_iterator->firstIndex() && m_iterator->index() == SubtitleIterator::AFTER_LAST )
			{
				if ( m_found ) // special case
					KMessageBox::information(
						parentWidget(),
						i18n( "1 replacement made.", "%n replacements made.", m_replace->numReplacements() ),
						i18n( "Replace" )
					);
				else
					KMessageBox::sorry(
						parentWidget(),
						i18n( "No instances of '%1' found!" ).arg( m_replace->pattern() ),
						i18n( "Replace" )
					);

				m_replace->resetCounts();
				m_replace->closeReplaceNextDialog();
				break;
			}

			if ( m_iterator->index() < 0 )
			{
				m_iterator->toFirst();

				int numReplacements = m_replace->numReplacements();

				m_replace->resetCounts();

				if ( KMessageBox::warningContinueCancel(
						parentWidget(),
						i18n( "1 replacement made.", "%n replacements made.", numReplacements ) + "\n\n" +
						(selection ?
							i18n( "End of selection reached.\nContinue from the beginning?" ) :
							i18n( "End of subtitle reached.\nContinue from the beginning?" )),
						i18n( "Replace" )
						) != KMessageBox::Continue )
				{
					m_replace->closeReplaceNextDialog();
					break;
				}
			}
		}
	}
	while ( res != KFind::Match );
}

void Replace::onHighlight( const QString&, int matchingIndex, int matchedLength )
{
	bool primary = ! m_translationMode || ! m_feedPrimaryNext;

	emit found( m_iterator->current(), primary, matchingIndex, matchingIndex + matchedLength - 1 );
}

void Replace::onReplace( const QString& text, int /*replacementIndex*/, int /*replacedLength*/, int /*matchedLength*/ )
{
	if ( m_translationMode && m_feedPrimaryNext )
		m_iterator->current()->setSecondaryText( text );
	else
		m_iterator->current()->setPrimaryText( text );
}

#include "replace.moc"
