/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "spelling.h"

#include <kdebug.h>
#include <klocale.h>

#include <kmessagebox.h>

#include <kspell.h>
#include <kspelldlg.h>
#include <ksconfig.h>

using namespace SubtitleComposer;

Spelling::Spelling( QWidget* parent ):
	QObject( parent ),
	m_subtitle( 0 ),
	m_translationMode( false ),
	m_feedPrimaryNext( true ),
	m_spell( 0 ),
	m_iterator( 0 )
{
}

Spelling::~Spelling()
{
	setSubtitle( 0 );
}

void Spelling::invalidate()
{
	delete m_iterator;
	m_iterator = 0;

	m_feedPrimaryNext = true;
}

QWidget* Spelling::parentWidget()
{
	return static_cast<QWidget*>( parent() );
}

void Spelling::setSubtitle( Subtitle* subtitle )
{
	m_subtitle = subtitle;

	invalidate();
}

void Spelling::setTranslationMode( bool enabled )
{
	m_translationMode = enabled;

	invalidate();
}

void Spelling::spellCheck( unsigned currentIndex )
{
	invalidate();

	if ( ! m_subtitle || ! m_subtitle->linesCount() )
		return;

	m_iterator = new SubtitleIterator( *m_subtitle );
	m_iterator->toIndex( currentIndex );
	m_firstIndex = m_iterator->index();

	KSpellConfig* config = new KSpellConfig( parentWidget(), "KSpellConfig" );
// 	config->setDictionary( m_generalConfig.option( OPT_SPELL_CHECK_DICT ) );

	m_spell = new KSpell( parentWidget(), i18n( "Spell Checking" ),
						  this, SLOT( onReady() ), config, true, true );

	connect( m_spell, SIGNAL( done(const QString&) ),
			 this, SLOT( onDone(const QString&) ) );
	connect( m_spell, SIGNAL( death() ),
			 this, SLOT( onDeath() ) );
	connect( m_spell, SIGNAL( misspelling(const QString&, const QStringList&, unsigned int) ),
			 this, SLOT( onMispelling(const QString&, const QStringList&, unsigned int) ) );
	connect( m_spell, SIGNAL( corrected(const QString&, const QString&, unsigned int) ),
			 this, SLOT( onCorrected(const QString&, const QString&, unsigned int) ) );
}

void Spelling::onReady()
{
	m_spell->check( m_iterator->current()->primaryText() );

	m_feedPrimaryNext = ! m_feedPrimaryNext;
}

void Spelling::onDone( const QString& )
{
	if ( m_spell->dlgResult() == KS_CANCEL || m_spell->dlgResult() == KS_STOP )
		m_spell->cleanUp();
	else
	{
		if ( advance() )
		{
			if ( m_translationMode )
			{
				m_feedPrimaryNext = ! m_feedPrimaryNext;
				m_spell->check(
					m_feedPrimaryNext ?
						m_iterator->current()->secondaryText() :
						m_iterator->current()->primaryText()
				);
			}
			else
				m_spell->check( m_iterator->current()->primaryText() );
		}
		else
			m_spell->cleanUp();
	}
}

bool Spelling::advance()
{
	if ( ! m_translationMode || m_feedPrimaryNext )
	{
		++(*m_iterator);

		if ( m_firstIndex == m_iterator->index() ||
			m_firstIndex == m_iterator->firstIndex() && m_iterator->index() == SubtitleIterator::AFTER_LAST )
			return false;

		if ( m_iterator->index() < 0 )
		{
			m_iterator->toFirst();

			if ( KMessageBox::warningContinueCancel(
					parentWidget(),
					i18n( "End of subtitle reached.\nContinue from the beginning?" ),
					i18n( "Spell Checking" )
					) != KMessageBox::Continue )
				return false;
		}
	}

	return true;
}

void Spelling::onMispelling( const QString& text, const QStringList&, unsigned int pos )
{
	bool primary = ! m_translationMode || ! m_feedPrimaryNext;

	emit mispelled( m_iterator->current(), primary, pos, pos + text.length() - 1 );
}

void Spelling::onCorrected( const QString& oldWord, const QString& newWord, unsigned int pos )
{
	if ( m_translationMode && m_feedPrimaryNext )
		m_iterator->current()->setSecondaryText(
			m_iterator->current()->secondaryText().replace( pos, oldWord.length(), newWord )
		);
	else
		m_iterator->current()->setPrimaryText(
			m_iterator->current()->primaryText().replace( pos, oldWord.length(), newWord )
		);
}

void Spelling::onDeath()
{
	delete m_spell;
	m_spell = 0;
}

#include "spelling.moc"
