/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *   based on smplayer by Ricardo Villalba                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mplayerbackend.h"
#include "mplayerconfigdefs.h"
#include "mplayerconfigwidget.h"
#include "mplayerprocess.h"
#include <qxtsignalwaiter.h>

#include <kdebug.h>
#include <klocale.h>
#include <kstandarddirs.h>

using namespace SubtitleComposer;

MPlayerBackend::MPlayerBackend():
	PlayerBackend( defaultsConfigGroup() ),
	m_process( new MPlayerProcess( m_configGroup, this ) ),
	m_position( 0.0 ),
	m_reportUpdates( true )
{
	connect( m_process, SIGNAL(mediaDataLoaded()), this, SLOT(onMediaDataLoaded()) );
	connect( m_process, SIGNAL(playingReceived()), this, SLOT(onPlayingReceived()) );
	connect( m_process, SIGNAL(pausedReceived()), this, SLOT(onPausedReceived()) );
	connect( m_process, SIGNAL(positionReceived(double)), this, SLOT(onPositionReceived(double)) );
	connect( m_process, SIGNAL(processExited()), this, SLOT(onProcessExited()) );
}

MPlayerBackend::~MPlayerBackend()
{
	if ( isInitialized() )
		__finalize();
}

VideoWidget* MPlayerBackend::_initialize( QWidget* videoWidgetParent )
{
	return new VideoWidget( videoWidgetParent );
}

void MPlayerBackend::_finalize()
{
	__finalize();
}

void MPlayerBackend::__finalize()
{
}

AppConfigGroup MPlayerBackend::defaultsConfigGroup()
{
	QMap<QString,QString> defaults;

	defaults[OPT_EXEC_PATH] = "mplayer";
	defaults[OPT_VO] = QString::null;
	defaults[OPT_AO] = QString::null;

	defaults[OPT_CACHE] = "5120";

	defaults[OPT_VOLUME_NORM] = "false";

	defaults[OPT_INPUT_CONF_PATH] = ::locate( "appdata", "input.conf" );

	defaults[OPT_FRAME_DROP] = "false";
	defaults[OPT_HARD_FRAME_DROP] = "false";
	defaults[OPT_AUTOSYNC_FACTOR] = QString::null;

	return AppConfigGroup( "MPlayer", defaults );
}

SubtitleComposer::AppConfigGroupWidget* MPlayerBackend::newAppConfigGroupWidget( QWidget* parent )
{
	return new MPlayerConfigWidget( m_configGroup, parent );
}


bool MPlayerBackend::_openFile( const QString& filePath, bool& playingAfterCall )
{
	m_position = 0.0;

	playingAfterCall = true;

	if ( ! m_process->start( filePath, (int)videoWidget()->videoLayer()->winId(), activeAudioStream(), audioStreams().count() ) )
		return false;

	return true;
}

void MPlayerBackend::_closeFile()
{
}

bool MPlayerBackend::_stop()
{
	if ( ! m_process->isRunning() )
		return true;

	if ( ! isApplicationClosingDown() )
	{
 		// NOTE the quit message is not processed when the application
		// is closing down so we don't even send it because it would be
		// just a waste of 3 seconds.
		m_process->sendQuit();
		m_process->waitForFinished( 3000 );
	}

	if ( m_process->isRunning() )
	{
		m_process->tryTerminate();
		m_process->waitForFinished( 3000 );
	}

	if ( m_process->isRunning() )
	{
		m_process->kill();
		m_process->waitForFinished( 3000 );
	}

	//return m_process->state() == QProcess::NotRunning;
	return true;
}

bool MPlayerBackend::_play()
{
	if ( ! m_process->isRunning() ) // player state was Ready
	{
		m_position = 0.0;

		if ( ! m_process->start( filePath(), (int)videoWidget()->videoLayer()->winId(), activeAudioStream(), audioStreams().count() ) )
			return false;

		if ( ! m_process->isRunning() )
			return false; // process ended prematurely
	}
	else // player state was Playing or Paused
		m_process->sendTogglePause();

	return true;
}

bool MPlayerBackend::_pause()
{
	if ( ! m_process->isRunning() ) // player state was Ready
	{
		m_position = 0.0;

		if ( ! m_process->start( filePath(), (int)videoWidget()->videoLayer()->winId(), activeAudioStream(), audioStreams().count() ) )
			return false;

		if ( ! m_process->isRunning() )
			return false; // process ended prematurely

		m_process->sendTogglePause();
	}
	else // player state was Playing or Paused
		m_process->sendTogglePause();

	return true;
}


bool MPlayerBackend::_seek( double seconds, bool accurate )
{
	if ( accurate )
	{
		bool wasPaused = isPaused();
		bool wasMuted = isMuted();

		m_reportUpdates = false;

		if ( ! wasPaused )
			m_process->sendTogglePause();

		if ( ! wasMuted )
			m_process->sendToggleMute();

		double seekPosition = seconds;
		do
		{
			m_process->sendSeek( seekPosition );

			if ( seekPosition > 0.0 )
			{
				seekPosition -= 1.0;
				if ( seekPosition < 0.0 )
					seekPosition = 0.0;
			}
			else
				break;
		}
		while ( m_position > seconds );

		if ( ! wasMuted )
			m_process->sendToggleMute();

		if ( ! wasPaused )
			m_process->sendTogglePause();

		m_reportUpdates = true;

		updatePosition( m_position );
	}
	else
		m_process->sendFastSeek( seconds );
	
	return true;
}

bool MPlayerBackend::_setActiveAudioStream( int audioStream, bool& supported )
{
	supported = true;
	m_process->sendAudioStream( audioStream );
	return true;
}

bool MPlayerBackend::_setVolume( double volume )
{
	m_process->sendVolume( volume );
	return true;
}


void MPlayerBackend::onMediaDataLoaded()
{
	const MediaData& mediaData = m_process->mediaData();

	QStringList audioStreams;

	for (QMap<int,TrackData>::ConstIterator it = mediaData.audioTracks.begin(), end = mediaData.audioTracks.end(); it != end; ++it)
	{
		QString audioStreamName;
		if ( ! it.data().language.isEmpty() )
			audioStreamName = it.data().language;
		if ( ! it.data().name.isEmpty() )
			audioStreamName += (audioStreamName.isEmpty() ? "" : " / ") + it.data().name;
		if ( audioStreamName.isEmpty() )
			audioStreamName = i18n( "Audio Stream #%1" ).arg( it.key() );

		audioStreams << audioStreamName;
	}

	if ( mediaData.videoWidth && mediaData.videoHeight )
	{
		videoWidget()->setVideoResolution(
			mediaData.videoWidth,
			mediaData.videoHeight,
			mediaData.videoAspect ?
				mediaData.videoAspect :
				0.0
		);
	}

	updateAudioStreams( audioStreams );

	if ( mediaData.duration )
		updateLength( mediaData.duration );

	if ( mediaData.videoFPS )
		updateFramesPerSecond( mediaData.videoFPS );
}

void MPlayerBackend::onPlayingReceived()
{
	if ( ! m_reportUpdates )
		return;

	updateState( Player::Playing );
}

void MPlayerBackend::onPausedReceived()
{
	if ( ! m_reportUpdates )
		return;

	updateState( Player::Paused );
}

void MPlayerBackend::onPositionReceived( double seconds )
{
	m_position = seconds;

	if ( ! m_reportUpdates )
		return;

	if ( ! isPlaying() )
		updateState( Player::Playing );

	updatePosition( seconds );
}

void MPlayerBackend::onProcessExited()
{
	updateState( Player::Ready );
}
