/***************************************************************************
 *   smplayer, GUI front-end for mplayer.                                  *
 *   Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>           *
 *                                                                         *
 *   modified for inclusion in Subtitle Composer                           *
 *   Copyright (C) 2007 by Sergio Pistone <sergio_pistone@yahoo.com.ar>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mplayerprocess.h"
#include "mplayerconfigdefs.h"
#include <qxtsignalwaiter.h>

#include <qdatetime.h>
#include <qapplication.h>
#include <qstringlist.h>
#include <qthread.h>

#include <kdebug.h>
#include <kstandarddirs.h>

using namespace SubtitleComposer;

class Sleeper : public QThread
{
	public:
		static void sleep( unsigned long secs )		{ QThread::sleep( secs ); }
		static void msleep( unsigned long msecs )	{ QThread::msleep( msecs ); }
		static void usleep( unsigned long usecs )	{ QThread::usleep(usecs); }
};

MPlayerProcess::MPlayerProcess( const AppConfigGroup& configGroup, QObject* parent ):
	QProcess( parent ),
	m_configGroup( configGroup ),
	m_mediaData(),
	m_incompleteLine(),
	m_isMediaDataLoaded( false ),
	m_isPaused( false ),
	m_emitPlaying( false ),
	m_positionRegExp( "^[AV]: *([0-9,:.-]+)" ),
	m_videoFrameRegExp( "^[AV]:.* (\\d+)\\/.\\d+" ),
	m_generalTagRegExp( "^(ID_.*)=(.*)" ),
	m_audioTagRegExp( "^ID_AID_(\\d+)_(LANG|NAME)=(.*)" ),
	m_pausedTagRegExp( "^ID_PAUSED" )
{
	connect( this, SIGNAL(readyReadStdout()), this, SLOT(onReadyReadStandardOutput()) );
	connect( this, SIGNAL(wroteToStdin()), this, SLOT(onWroteToStdin()) );
	connect( &m_commandsQueueTimer, SIGNAL(timeout()), this, SLOT(onTimeout()) );
}

MPlayerProcess::~MPlayerProcess()
{
}

bool MPlayerProcess::waitForFinished( int msecs )
{
	static const int sleepInterval = 50;

	int elapsed = 0;
	while ( isRunning() )
	{
		qApp->processEvents();
		Sleeper::msleep( sleepInterval );
		elapsed += sleepInterval;
		if ( msecs >= 0 && elapsed > msecs )
			break;
	}

	return isRunning();
}

const MediaData& MPlayerProcess::mediaData()
{
	return m_mediaData;
}

bool MPlayerProcess::start( const QString& filePath, int winId, int audioStream, int audioStreamCount )
{
	if ( KStandardDirs::findExe( m_configGroup.option( OPT_EXEC_PATH ) ).isEmpty() )
		return false;

	m_mediaData.reset();
	m_isMediaDataLoaded = false;
	m_isPaused = false;
	m_emitPlaying = false;
	m_incompleteLine = QCString();

	QStringList args;

	args << KStandardDirs::findExe( m_configGroup.option( OPT_EXEC_PATH ) );

	if ( audioStream >= 0 && audioStreamCount > 1 )
		args << "-aid" << QString::number( audioStream );

	args << "-noquiet";
	args << "-nofs"; // No mplayer fullscreen mode
	args << "-identify"; // makes mplayer emit all kinds of additional information
	args << "-slave"; // enable slave mode so we can sent commands to mplayer process

	if ( ! m_configGroup.option( OPT_VO ).isEmpty() )
		args << "-vo" << m_configGroup.option( OPT_VO );

	if ( ! m_configGroup.option( OPT_AO ).isEmpty() )
		args << "-ao" << m_configGroup.option( OPT_AO );

	args << "-zoom"; // allow software scaling where hardware scaling is unavaliable
	args << "-nokeepaspect"; // do not keep window aspect ratio when resizing windows

	if ( m_configGroup.optionAsBool( OPT_FRAME_DROP ) )
		args << "-framedrop";

	if ( m_configGroup.optionAsBool( OPT_HARD_FRAME_DROP ) )
		args << "-hardframedrop";

	if ( ! m_configGroup.option( OPT_AUTOSYNC_FACTOR ).isEmpty() )
		args << "-autosync" << m_configGroup.option( OPT_AUTOSYNC_FACTOR );

	if ( ! m_configGroup.option( OPT_INPUT_CONF_PATH ).isEmpty() )
		args << "-input" << "conf=" + m_configGroup.option( OPT_INPUT_CONF_PATH );

	args << "-wid" << QString::number( winId ); // set window id so that it gets embeded in our window
	args << "-noautosub"; // turn off automatic subtitle file loading

	if ( ! m_configGroup.option( OPT_CACHE ).isEmpty() )
	{
		args << "-cache" << m_configGroup.option( OPT_CACHE );
		args << "-cache-min" << QString::number( 99 );
		args << "-cache-seek-min" << QString::number( 99 );
	}

	args << "-osdlevel" << QString::number( 0 ); // no OSD

	if ( m_configGroup.optionAsBool( OPT_VOLUME_NORM ) )
		args << "-af" << "volnorm=2"; // set volume normalization

	args << "-softvol" << "-softvol-max" << QString::number( 100 ); // maximum software volume level

	args << filePath;

	setArguments( args );

	return QProcess::start();
}


void MPlayerProcess::sendTogglePause()
{
	if ( m_isPaused ) // set playing
		sendCommand( "pause", Playing, false );
	else // set paused
		sendCommand( "pause", Pausing, true );
}

void MPlayerProcess::sendSeek( double seconds )
{
	sendCommand( QString( "seek %1 2" ).arg( seconds ), PausingKeep, true );
}

void MPlayerProcess::sendFastSeek( double seconds )
{
	static const QString seek( "seek" );

	for ( QStringList::Iterator it = m_commandsQueue.begin(), end = m_commandsQueue.end(); it != end; ++it )
	{
		if ( (*it).contains( seek ) )
			it = m_commandsQueue.remove( it );
	}

	queueCommand( QString( "seek %1 2" ).arg( seconds ), PausingKeep );
}

void MPlayerProcess::sendToggleMute()
{
	sendCommand( "mute", PausingKeep, true );
}

void MPlayerProcess::sendVolume( double volume )
{
	sendCommand( QString( "volume %1 1" ).arg( volume ), PausingKeep, false );
}


void MPlayerProcess::sendAudioStream( int audioStream )
{
	sendCommand( QString( "switch_audio %1" ).arg( audioStream ), PausingKeep, false );
}

void MPlayerProcess::sendQuit()
{
	sendCommand( "quit", PausingKeep, false );
}

void MPlayerProcess::sendCommand( const char* cmd, MPlayerProcess::CommandMode mode, bool block )
{
	sendCommand( QString( cmd ), mode, block );
}

void MPlayerProcess::sendCommand( const QString& cmd, MPlayerProcess::CommandMode mode, bool block )
{
	static int count = 0;

	if ( count )
	{
		kdDebug() << "call to sendCommand already in progress" << endl;
		return;
	}

	if ( ! isRunning() )
		return;

	count++;

	if ( mode == Pausing || (mode == PausingKeep && m_isPaused) )
	{
// 		kdDebug() << "sending pausing " << cmd << endl;

		if ( block )
		{
			QxtSignalWaiter pauseWaiter( this, SIGNAL(pausedReceived()) );
			writeToStdin( "pausing " + cmd + "\n" );
// 			kdDebug() << "WAITING" << endl;
			if ( ! pauseWaiter.wait( 5000 ) )
				kdDebug() << ">>>>>>>TIMEDOUT<<<<<<<" << endl;
// 			kdDebug() << "WAITED" << endl;
		}
		else
		{
			writeToStdin( "pausing " + cmd + "\n" );
		}
	}
	else // if ( mode == Playing || (mode == PausingKeep && ! m_isPaused) )
	{
// 		kdDebug() << "sending " << cmd << endl;

		if ( block )
		{
			QxtSignalWaiter playingWaiter( this, SIGNAL(playingReceived()) );
			m_emitPlaying = true; // to make the playingReceived() signal be emmited again
			writeToStdin( cmd + "\n" );
// 			kdDebug() << "WAITING" << endl;
			if ( ! playingWaiter.wait( 5000 ) )
				kdDebug() << ">>>>>>TIMEDOUT<<<<<<<" << endl;
// 			kdDebug() << "WAITED" << endl;
		}
		else
		{
			writeToStdin( cmd + "\n" );
		}
	}

	count--;
}


void MPlayerProcess::queueCommand( const QString& cmd, MPlayerProcess::CommandMode mode )
{
	QString command;
	switch ( mode )
	{
		case Pausing:
			m_commandsQueue.append( "pausing " + cmd + "\n" );
			break;
		case PausingKeep:
			m_commandsQueue.append( "pausing_keep " + cmd + "\n" );
			break;
		case Playing:
			m_commandsQueue.append( cmd + "\n" );
			break;
	}

// 	kdDebug() << "queuing " << m_commandsQueue.last();

	if ( ! m_commandsQueueTimer.isActive() )
		m_commandsQueueTimer.start( 100 );
}

void MPlayerProcess::onReadyReadStandardOutput()
{
	QByteArray newData = readStdout();
	if ( ! newData.size() )
		return;
	newData.resize( newData.size() + 1 );
	newData[newData.size()-1] = 0;

	m_incompleteLine.append( newData );
	m_incompleteLine.replace( 0x0D, '\n' );

	for ( int idx = m_incompleteLine.find( '\n' ); idx > -1; idx = m_incompleteLine.find( '\n' ) )
	{
		parseLine( QString::fromLocal8Bit( m_incompleteLine.left( idx ).data() ) );
		m_incompleteLine = m_incompleteLine.mid( idx + 1 );
	}
}

void MPlayerProcess::onWroteToStdin()
{
	if ( m_commandsQueue.empty() )
		return;

	m_commandsQueue.pop_front();
}

void MPlayerProcess::onTimeout()
{
	if ( ! m_commandsQueue.empty() )
		writeToStdin( m_commandsQueue.first() );
	else
		m_commandsQueueTimer.stop();
}


void MPlayerProcess::parseLine( const QString& line )
{
	if ( line.isEmpty() )
		return;

// 	kdDebug() << line << endl;

	if ( m_mediaData.videoFPS != 0.0 && m_videoFrameRegExp.search( line ) > -1 )
	{
		// try to parse the position from the reported frame number

		if ( ! m_isMediaDataLoaded )
		{
			emit mediaDataLoaded();
			m_isMediaDataLoaded = true;
		}

		if ( m_isPaused || m_emitPlaying )
		{
			m_isPaused = false;
			m_emitPlaying = false;
			emit playingReceived();
		}

		double position = m_videoFrameRegExp.cap( 1 ).toInt() / m_mediaData.videoFPS;
		if ( m_positionRegExp.search( line ) > -1 )
		{
			double aux = m_positionRegExp.cap( 1 ).toDouble();
			if ( position - aux > 0.5 || position - aux < -0.5 )
				 // mplayer is reporting badly, use innacurate (but safe) value instead
				position = aux;
		}

		emit positionReceived( position );
	}
	else

	if ( m_positionRegExp.search( line ) > -1 )
	{
		// parse the reported position in seconds

		if ( ! m_isMediaDataLoaded )
		{
			emit mediaDataLoaded();
			m_isMediaDataLoaded = true;
		}

		if ( m_isPaused || m_emitPlaying )
		{
			m_isPaused = false;
			m_emitPlaying = false;
			emit playingReceived();
		}

		emit positionReceived( m_positionRegExp.cap( 1 ).toDouble() );
	}

	else
	{
		// pause toggled
		if ( m_pausedTagRegExp.search( line ) > -1 )
		{
			m_isPaused = ! m_isPaused;
			if ( m_isPaused )
				emit pausedReceived();
			else
				emit playingReceived();
		}

		// The following things are not sent when the file has started to play
		// (or if sent, smplayer will ignore anyway...)
		// So not process anymore, if video is playing to save some time
		if ( m_isMediaDataLoaded )
			return;

		// Matroska audio
		if ( m_audioTagRegExp.search( line ) > -1 )
		{
			int ID = m_audioTagRegExp.cap( 1 ).toInt();
			if ( m_audioTagRegExp.cap( 2 ) == "NAME" )
				m_mediaData.audioTracks[ID].name = m_audioTagRegExp.cap( 3 );
			else
				m_mediaData.audioTracks[ID].language = m_audioTagRegExp.cap( 3 );
		}
		else

		// Generic things
		if ( m_generalTagRegExp.search( line ) > -1 )
		{
			QString tag = m_generalTagRegExp.cap( 1 );
			QString value = m_generalTagRegExp.cap( 2 );

			if ( tag == "ID_AUDIO_ID" ) // Generic audio
			{
				int ID = value.toInt();
				if ( ! m_mediaData.audioTracks.contains( ID ) )
					m_mediaData.audioTracks.insert( ID, TrackData() );
			}
			else if ( tag == "ID_LENGTH" )
				m_mediaData.duration = value.toDouble();
			else if ( tag == "ID_VIDEO_WIDTH" )
			{
				m_mediaData.hasVideo = true;
				m_mediaData.videoWidth = value.toInt();
			}
			else if ( tag == "ID_VIDEO_HEIGHT" )
			{
				m_mediaData.hasVideo = true;
				m_mediaData.videoHeight = value.toInt();
			}
			else if ( tag == "ID_VIDEO_ASPECT" )
			{
				m_mediaData.hasVideo = true;
				m_mediaData.videoAspect = value.toDouble();
				if ( m_mediaData.videoAspect == 0.0 && m_mediaData.videoWidth != 0.0 && m_mediaData.videoHeight != 0 )
					m_mediaData.videoAspect = (double) m_mediaData.videoWidth / m_mediaData.videoHeight;
			}
			else if ( tag == "ID_VIDEO_FPS" )
			{
				bool ok;
				m_mediaData.videoFPS = value.toDouble( &ok );
				if ( ! ok )
					m_mediaData.videoFPS = 0.0;
			}
		}
	}
}
