/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "player.h"

#ifdef HAVE_GSTREAMER
#include "gstreamer/gstplayerbackend.h"
#endif
#include "mplayer/mplayerbackend.h"
#ifdef HAVE_XINE
#include "xine/xineplayerbackend.h"
#endif

#include <kdebug.h>

using namespace SubtitleComposer;

Player::Player():
	m_activeBackend( 0 )
{
#ifdef HAVE_GSTREAMER
	m_backendsMap["GStreamer"] = new GstPlayerBackend();
#endif

	m_backendsMap["MPlayer"] = new MPlayerBackend();

#ifdef HAVE_XINE
	m_backendsMap["Xine"] = new XinePlayerBackend();
#endif
}

Player::~Player()
{
}

Player* const Player::instance()
{
	static Player player;

	return &player;
}

void Player::initBackend( QWidget* videoWidgetParent, const QString& backendName )
{
	if ( m_activeBackend )
	{
		kdError() << "Player already has an initialized backend" << endl;
		return;
	}

	// we first try to set the requested backend as active
	setActiveBackend( backendName, videoWidgetParent );

	// if that fails, we set the first available backend as active
	if ( ! m_activeBackend )
	{
		for ( QMap<QString,PlayerBackend*>::ConstIterator it = m_backendsMap.begin(), end = m_backendsMap.end(); it != end; ++it )
			if ( setActiveBackend( it.key(), videoWidgetParent ) != 0 )
				break;
	}

	if ( ! m_activeBackend )
		kdError() << "Couldn't initialize any player backend" << endl;
}

void Player::deinitBackend()
{
	if ( ! m_activeBackend )
		return;

	for ( QMap<QString,PlayerBackend*>::ConstIterator it = m_backendsMap.begin(), end = m_backendsMap.end(); it != end; ++it )
		it.data()->finalize();

	m_activeBackend = 0;
}

QStringList Player::backendNames()
{
	return m_backendsMap.keys();
}

PlayerBackend* Player::backend( const QString& backendName )
{
	return m_backendsMap.contains( backendName ) ? m_backendsMap[backendName] : 0;
}

PlayerBackend* Player::activeBackend()
{
	return m_activeBackend;
}

PlayerBackend* Player::setActiveBackend( const QString& backendName, QWidget* videoWidgetParent )
{
	if ( ! m_backendsMap.contains( backendName ) )
	{
		kdDebug() << "Attempted to set unknown backend as active: " << backendName << endl;
		return m_activeBackend;
	}

	bool isMuted = false;
	double volume = 100.0;

	if ( m_activeBackend )
	{
		isMuted = m_activeBackend->isMuted();
		volume = m_activeBackend->volume();

		m_activeBackend->finalize();

		disconnectActiveBackend();
	}

	if ( m_backendsMap[backendName]->initialize( videoWidgetParent ) )
		m_activeBackend = m_backendsMap[backendName];
	else
	{
		kdDebug() << "Error initializing " << backendName << " backend" << endl;

		if ( ! m_activeBackend )
			return 0;

		kdDebug() << "Attempting to reinitialize previous backend" << endl;
		if ( ! m_activeBackend->initialize( videoWidgetParent ) )
		{
			kdDebug() << "Error initializing previous backend" << endl;
			return 0;
		}
	}

	m_activeBackend->setMuted( isMuted );
	m_activeBackend->setVolume( volume );

	connectActiveBackend();

	// NOTE: we connect the m_activeBackend signals after it's been
	// initialized so we have to call this methods for ourselves:
	connectActiveBackendVideoWidget();
	emit backendInitialized( m_activeBackend );

	return m_activeBackend;
}

void Player::setApplicationClosingDown()
{
	m_activeBackend->setApplicationClosingDown();
}

void Player::connectActiveBackend()
{
	if ( ! m_activeBackend )
		return;

	connect( m_activeBackend, SIGNAL( initialized( PlayerBackend* ) ),
			 this, SLOT( connectActiveBackendVideoWidget() ) );

	connect( m_activeBackend, SIGNAL( initialized( PlayerBackend* ) ),
			 this, SIGNAL( backendInitialized( PlayerBackend* ) ) );
	connect( m_activeBackend, SIGNAL( finalized( PlayerBackend* ) ),
			 this, SIGNAL( backendFinalized( PlayerBackend* ) ) );

	connect( m_activeBackend, SIGNAL( fileOpened( const QString& ) ),
			 this, SIGNAL( fileOpened( const QString& ) ) );
	connect( m_activeBackend, SIGNAL( fileOpenError( const QString& ) ),
			 this, SIGNAL( fileOpenError( const QString& ) ) );
	connect( m_activeBackend, SIGNAL( fileClosed() ),
			 this, SIGNAL( fileClosed() ) );
	connect( m_activeBackend, SIGNAL( playbackError() ),
			 this, SIGNAL( playbackError() ) );
	connect( m_activeBackend, SIGNAL( playing() ),
			 this, SIGNAL( playing() ) );
	connect( m_activeBackend, SIGNAL( paused() ),
			 this, SIGNAL( paused() ) );
	connect( m_activeBackend, SIGNAL( stopped() ),
			 this, SIGNAL( stopped() ) );
	connect( m_activeBackend, SIGNAL( positionChanged( double ) ),
			 this, SIGNAL( positionChanged( double ) ) );
	connect( m_activeBackend, SIGNAL( lengthChanged( double ) ),
			 this, SIGNAL( lengthChanged( double ) ) );
	connect( m_activeBackend, SIGNAL( framesPerSecondChanged( double ) ),
			 this, SIGNAL( framesPerSecondChanged( double ) ) );

	connect( m_activeBackend, SIGNAL( activeAudioStreamChanged( int ) ),
			 this, SIGNAL( activeAudioStreamChanged( int ) ) );
	connect( m_activeBackend, SIGNAL( audioStreamsChanged( const QStringList& ) ),
			 this, SIGNAL( audioStreamsChanged( const QStringList& ) ) );
	connect( m_activeBackend, SIGNAL( volumeChanged( double ) ),
			 this, SIGNAL( volumeChanged( double ) ) );
	connect( m_activeBackend, SIGNAL( mutedChanged( bool ) ),
			 this, SIGNAL( mutedChanged( bool ) ) );
}

void Player::connectActiveBackendVideoWidget()
{
	connect( m_activeBackend->videoWidget(), SIGNAL( doubleClicked( const QPoint& ) ),
			this, SIGNAL( doubleClicked( const QPoint& ) ) );
	connect( m_activeBackend->videoWidget(), SIGNAL( leftClicked( const QPoint& ) ),
			this, SIGNAL( leftClicked( const QPoint& ) ) );
	connect( m_activeBackend->videoWidget(), SIGNAL( rightClicked( const QPoint& ) ),
			this, SIGNAL( rightClicked( const QPoint& ) ) );
	connect( m_activeBackend->videoWidget(), SIGNAL( wheelUp() ),
			this, SIGNAL( wheelUp() ) );
	connect( m_activeBackend->videoWidget(), SIGNAL( wheelDown() ),
			this, SIGNAL( wheelDown() ) );
}

void Player::disconnectActiveBackend()
{
	if ( ! m_activeBackend )
		return;

	disconnect( m_activeBackend, SIGNAL( initialized( PlayerBackend* ) ),
				this, SLOT( connectActiveBackendVideoWidget() ) );

	disconnect( m_activeBackend, SIGNAL( initialized( PlayerBackend* ) ),
				this, SIGNAL( backendInitialized( PlayerBackend* ) ) );
	disconnect( m_activeBackend, SIGNAL( finalized( PlayerBackend* ) ),
				this, SIGNAL( backendFinalized( PlayerBackend* ) ) );

	disconnect( m_activeBackend, SIGNAL( fileOpened( const QString& ) ),
				this, SIGNAL( fileOpened( const QString& ) ) );
	disconnect( m_activeBackend, SIGNAL( fileOpenError( const QString& ) ),
				this, SIGNAL( fileOpenError( const QString& ) ) );
	disconnect( m_activeBackend, SIGNAL( fileClosed() ),
				this, SIGNAL( fileClosed() ) );
	disconnect( m_activeBackend, SIGNAL( playbackError() ),
				this, SIGNAL( playbackError() ) );
	disconnect( m_activeBackend, SIGNAL( playing() ),
				this, SIGNAL( playing() ) );
	disconnect( m_activeBackend, SIGNAL( paused() ),
				this, SIGNAL( paused() ) );
	disconnect( m_activeBackend, SIGNAL( stopped() ),
				this, SIGNAL( stopped() ) );
	disconnect( m_activeBackend, SIGNAL( positionChanged( double ) ),
				this, SIGNAL( positionChanged( double ) ) );
	disconnect( m_activeBackend, SIGNAL( lengthChanged( double ) ),
				this, SIGNAL( lengthChanged( double ) ) );
	disconnect( m_activeBackend, SIGNAL( framesPerSecondChanged( double ) ),
				this, SIGNAL( framesPerSecondChanged( double ) ) );

	disconnect( m_activeBackend, SIGNAL( activeAudioStreamChanged( int ) ),
				this, SIGNAL( activeAudioStreamChanged( int ) ) );
	disconnect( m_activeBackend, SIGNAL( audioStreamsChanged( const QStringList& ) ),
				this, SIGNAL( audioStreamsChanged( const QStringList& ) ) );
	disconnect( m_activeBackend, SIGNAL( volumeChanged( double ) ),
			 	this, SIGNAL( volumeChanged( double ) ) );
	disconnect( m_activeBackend, SIGNAL( mutedChanged( bool ) ),
			 	this, SIGNAL( mutedChanged( bool ) ) );
}

/// FORWARD CALLS TO THE ACTIVE BACKEND

const QString& Player::filePath() const
{
	return m_activeBackend->filePath();
}

double Player::length() const
{
	return m_activeBackend->length();
}

double Player::framesPerSecond() const
{
	return m_activeBackend->framesPerSecond();
}

bool Player::openFile( const QString& filePath )
{
	return m_activeBackend->openFile( filePath );
}

bool Player::closeFile()
{
	return m_activeBackend->closeFile();
}

Player::State Player::state() const
{
	return m_activeBackend->state();
}

bool Player::isPlaying() const
{
	return m_activeBackend->isPlaying();
}

bool Player::play()
{
	return m_activeBackend->play();
}

bool Player::isPaused() const
{
	return m_activeBackend->isPaused();
}

bool Player::pause()
{
	return m_activeBackend->pause();
}

bool Player::togglePlayPaused()
{
	return m_activeBackend->togglePlayPaused();
}

double Player::position() const
{
	return m_activeBackend->position();
}

bool Player::seek( double seconds, bool accurate )
{
	return m_activeBackend->seek( seconds, accurate );
}

bool Player::isStopped() const
{
	return m_activeBackend->isStopped();
}

bool Player::stop()
{
	return m_activeBackend->stop();
}

const QStringList& Player::audioStreams() const
{
	return m_activeBackend->audioStreams();
}

int Player::activeAudioStream() const
{
	return m_activeBackend->activeAudioStream();
}

bool Player::setActiveAudioStream( int audioStreamIndex )
{
	return m_activeBackend->setActiveAudioStream( audioStreamIndex );
}

double Player::volume() const
{
	return m_activeBackend->volume();
}

void Player::increaseVolume( double amount )
{
	return m_activeBackend->increaseVolume( amount );
}

void Player::decreaseVolume( double amount )
{
	return m_activeBackend->decreaseVolume( amount );
}

void Player::setVolume( double volume )
{
	return m_activeBackend->setVolume( volume );
}

bool Player::isMuted() const
{
	return m_activeBackend->isMuted();
}

void Player::toggleMuted()
{
	return m_activeBackend->toggleMuted();
}

void Player::setMuted( bool mute )
{
	return m_activeBackend->setMuted( mute );
}
