#ifndef PLAYER_H
#define PLAYER_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "player.h"

#include <qstring.h>
#include <qstringlist.h>
#include <qwidget.h>
#include <qmap.h>

namespace SubtitleComposer
{
	class PlayerBackend;

	class Player : public QObject
	{
		Q_OBJECT

		public:

			typedef enum {
				Unitialized=0,
				Closed, // same as Initialized
				Opening,
				// Opened, same as >Opening
				Playing,
				Paused,
				Ready // same as Stopped or Finished
			} State;

			static Player* const instance();

			const QString& filePath() const;

			State state() const;
			bool isPlaying() const;
			bool isPaused() const;
			double position() const;
			double length() const;
			double framesPerSecond() const;
			bool isStopped() const;

			double volume() const;
			bool isMuted() const;
			int activeAudioStream() const;
			const QStringList& audioStreams() const;

			void initBackend( QWidget* videoWidgetParent, const QString& backendName=QString::null );
			void deinitBackend();

			QStringList backendNames();
			PlayerBackend* backend( const QString& name );
			PlayerBackend* activeBackend();
			PlayerBackend* setActiveBackend( const QString& name, QWidget* videoWidgetParent );

			void setApplicationClosingDown();

		public slots:

			// return values of this functions don't imply that the operation was performed OK
			// but that it was allowed (a false return value means that nothing was attempted.

			bool openFile( const QString& filePath );
			bool closeFile();

			bool play();
			bool pause();
			bool togglePlayPaused();
			bool seek( double seconds, bool accurate );
			bool stop();
			bool setActiveAudioStream( int audioStreamIndex );

			void increaseVolume( double amount=3.0 );
			void decreaseVolume( double amount=3.0 );
			void setVolume( double volume ); // value from 0.0 to 100.0 (inclusive)

			void toggleMuted();
			void setMuted( bool mute );

		signals:

			void fileOpenError( const QString& filePath );
			void fileOpened( const QString& filePath );
			void fileClosed();

			void playbackError(); // unexpected error

			void playing();
			void positionChanged( double seconds );
			void lengthChanged( double seconds );
			void framesPerSecondChanged( double fps );
			void paused();
			void stopped();
			void activeAudioStreamChanged( int audioStreamIndex );
			void audioStreamsChanged( const QStringList& audioStreams );

			void volumeChanged( double volume );
			void mutedChanged( bool muted );

			void doubleClicked( const QPoint& point );
			void rightClicked( const QPoint& point );
			void leftClicked( const QPoint& point );
			void wheelUp();
			void wheelDown();

			void backendInitialized( PlayerBackend* playerBackend );
			void backendFinalized( PlayerBackend* playerBackend );

		private:

			Player();
			~Player();

			void connectActiveBackend();
			void disconnectActiveBackend();

		private slots:

			void connectActiveBackendVideoWidget();

		private:

			PlayerBackend* m_activeBackend;
			QMap<QString,PlayerBackend*> m_backendsMap;
	};
}

#endif
