/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "playerbackend.h"
#include "videowidget.h"

#include <qfileinfo.h>
#include <qtimer.h>

#include <kdebug.h>

#include <math.h>

using namespace SubtitleComposer;

#define DEFAULT_MIN_POSITION_DELTA 0.02

PlayerBackend::PlayerBackend( const AppConfigGroup& configGroup ):
	QObject( 0 ),
	m_filePath(),
	m_state( Player::Unitialized ),
	m_position( -1.0 ),
	m_length( -1.0 ),
	m_framesPerSecond( -1.0 ),
	m_minPositionDelta( DEFAULT_MIN_POSITION_DELTA ),
	m_activeAudioStream( -1 ),
	m_audioStreams(),
	m_muted( false ),
	m_logarithmicVolume( true ),
	m_userVolume( 100.0 ),
	m_volume( 100.0 ),
	m_videoWidget( 0 ),
	m_openFileTimer( new QTimer( this ) ),
	m_applicationClosingDown( false ),
	m_configGroup( configGroup )
{
	// the timeout might seem too much, but it only matters when the file couldn't be
	// opened, and it's better to have the user wait a bit in that case than showing
	// an error we there's nothing wrong with the file (a longer time might be needed
	// for example if the computer is under heavy load or is just slow)

	connect( m_openFileTimer, SIGNAL( timeout() ), this, SLOT( onOpenFileTimeout() ) );
}

PlayerBackend::~PlayerBackend()
{
}

SubtitleComposer::VideoWidget* PlayerBackend::videoWidget()
{
	return m_videoWidget;
}

bool PlayerBackend::isApplicationClosingDown() const
{
	return m_applicationClosingDown;
}

void PlayerBackend::setApplicationClosingDown()
{
	m_applicationClosingDown = true;
}

bool PlayerBackend::isInitialized() const
{
	return m_state != Player::Unitialized;
}

bool PlayerBackend::initialize( QWidget* videoWidgetParent )
{
	if ( m_state != Player::Unitialized )
		return true;

	if ( (m_videoWidget = _initialize( videoWidgetParent )) )
	{
		m_state = Player::Closed;

		connect( m_videoWidget, SIGNAL(destroyed()), this, SLOT(onVideoWidgetDestroyed()) );

		m_videoWidget->show();

		// FIXME there's got to be a better way to make videoWidgetParent update it's geometry than this...
		QRect geometry = videoWidgetParent->geometry();
		geometry.setHeight( geometry.height() + 1 );
		videoWidgetParent->setGeometry( geometry );

		emit initialized( this );
	}

	return m_state != Player::Unitialized;
}

void PlayerBackend::finalize()
{
	if ( m_state == Player::Unitialized )
		return;

	closeFile();

	_finalize();

	m_state = Player::Unitialized;

	m_videoWidget->disconnect();
	m_videoWidget->hide();
	delete m_videoWidget;

	m_videoWidget = 0;

	emit finalized( this );
}

bool PlayerBackend::reinitialize()
{
	if ( m_state == Player::Unitialized )
		return false;

	QWidget* videoWidgetParent = videoWidget()->parentWidget();

	finalize();

	return initialize( videoWidgetParent );
}

void PlayerBackend::onVideoWidgetDestroyed()
{
	if ( m_state == Player::Unitialized )
	{
		kdWarning() << "called on onVideoWidgetDestroyed() with player not initialized!";
		kdWarning() << "this shouldn't have happened!!!";
		return;
	}

	m_videoWidget = 0;

	closeFile();

	_finalize();

	m_state = Player::Unitialized;

	emit finalized( this );
}

const AppConfigGroup& PlayerBackend::configGroup()
{
	return m_configGroup;
}

void PlayerBackend::setConfigGroup( const AppConfigGroup& newConfigGroup )
{
	if ( m_configGroup.isCompatibleWith( newConfigGroup ) )
	{
		if ( m_configGroup != newConfigGroup )
		{
			m_configGroup = newConfigGroup;
			reinitialize();
		}
	}
}

const QString& PlayerBackend::filePath() const
{
	return m_filePath;
}

void PlayerBackend::resetState()
{
	if ( m_openFileTimer->isActive() )
		m_openFileTimer->stop();


	m_filePath = QString::null;

	m_position = -1.0;
	m_length = -1.0;
	m_framesPerSecond = -1.0;
	m_minPositionDelta = DEFAULT_MIN_POSITION_DELTA;

	m_activeAudioStream = -1;
	m_audioStreams.clear();

	m_state = Player::Closed;

	if ( m_videoWidget )
		m_videoWidget->videoLayer()->hide();
}

bool PlayerBackend::openFile( const QString& filePath )
{
	if ( m_state != Player::Closed )
		return false;

	QFileInfo fileInfo( filePath );
	if ( ! fileInfo.exists() || ! fileInfo.isFile() || ! fileInfo.isReadable() )
	{
		emit fileOpenError( filePath ); // operation will never succed
		return true;
	}

	m_filePath = filePath;
	m_state = Player::Opening;

	// FIXME add an option to set the error timeout when opening video
	m_openFileTimer->start( 6000, true );

	bool playingAfterCall = true;
	if ( ! _openFile( fileInfo.absFilePath(), playingAfterCall ) )
	{
		resetState();
		emit fileOpenError( filePath );
		return true;
	}

	if ( ! playingAfterCall )
		_play();

	return true;
}

bool PlayerBackend::closeFile()
{
	if ( m_state <= Player::Closed )
		return false;

	bool stop = m_state != Player::Ready;
	if ( stop )
		_stop(); // we can safely ignore the _stop() return value here as we're about to close the file

	_closeFile();

	resetState();

	if ( stop )
		emit stopped();

	emit fileClosed();

	return true;
}

Player::State PlayerBackend::state() const
{
	return m_state;
}

bool PlayerBackend::isPlaying() const
{
	return m_state == Player::Playing;
}

bool PlayerBackend::play()
{
	if ( m_state <= Player::Opening || m_state == Player::Playing )
		return false;

	if ( ! _play() )
	{
		resetState();
		emit playbackError();
	}

	return true;
}

bool PlayerBackend::isPaused() const
{
	return m_state == Player::Paused;
}

bool PlayerBackend::pause()
{
	if ( m_state <= Player::Opening || m_state == Player::Paused )
		return false;

	if ( ! _pause() )
	{
		resetState();
		emit playbackError();
	}

	return true;
}

bool PlayerBackend::togglePlayPaused()
{
	if ( m_state <= Player::Opening )
		return false;

	bool error;
	if ( m_state == Player::Playing )
		error = ! _pause();
	else
		error = ! _play();

	if ( error )
	{
		resetState();
		emit playbackError();
	}

	return true;
}

double PlayerBackend::position() const
{
	return m_state <= Player::Opening ? -1.0 : (m_state == Player::Ready ? 0.0 : m_position);
}

double PlayerBackend::length() const
{
	return m_state <= Player::Opening ? -1.0 : m_length;
}

double PlayerBackend::framesPerSecond() const
{
	return m_state <= Player::Opening ? -1.0 : m_framesPerSecond;
}

bool PlayerBackend::seek( double seconds, bool accurate )
{
	if ( (m_state != Player::Playing && m_state != Player::Paused) || seconds < 0 || seconds > m_length )
		return false;

	if ( seconds == m_position )
		return true;

	if ( ! _seek( seconds, accurate ) )
	{
		resetState();
		emit playbackError();
	}

	return true;
}

bool PlayerBackend::isStopped() const
{
	return m_state == Player::Ready;
}

bool PlayerBackend::stop()
{
	if ( m_state <= Player::Opening || m_state == Player::Ready )
		return false;

	if ( ! _stop() )
	{
		resetState();
		emit playbackError();
		return true;
	}

	return true;
}

double PlayerBackend::logarithmicVolume( double volume )
{
	static const double base = 4.0;
	static const double power = 1.0;
	static const double divisor = pow( base, power );

	double scaledVol = volume*power*power/100.0;
	double factor = pow( base, scaledVol/power );

	return (scaledVol*factor/divisor)*(100.0/(power*power));
}

bool PlayerBackend::logarithmicVolume()
{
	return m_logarithmicVolume;
}

void PlayerBackend::setLogarithmicVolume( bool logarithmic )
{
	if ( m_logarithmicVolume != logarithmic )
	{
		m_logarithmicVolume = logarithmic;

		m_volume = m_muted ? 0 : (m_logarithmicVolume ? logarithmicVolume( m_userVolume ) : m_userVolume);

		if ( ! m_muted && m_state == Player::Playing )
		{
			if ( ! _setVolume( m_volume ) )
			{
				resetState();
				emit playbackError();
			}
		}
	}
}

double PlayerBackend::volume() const
{
	return m_userVolume;
}

void PlayerBackend::increaseVolume( double amount )
{
	setVolume( m_userVolume + amount );
	setMuted( false );
}

void PlayerBackend::decreaseVolume( double amount )
{
	setVolume( m_userVolume - amount );
}

void PlayerBackend::setVolume( double volume )
{
	if ( volume < 0.0 )
		volume = 0.0;
	else if ( volume > 100.0 )
		volume = 100.0;

	if ( m_userVolume != volume )
	{
		m_userVolume = volume;

		m_volume = m_muted ? 0 : (m_logarithmicVolume ? logarithmicVolume( m_userVolume ) : m_userVolume);

		if ( ! m_muted && m_state == Player::Playing )
		{
			if ( ! _setVolume( m_volume ) )
			{
				resetState();
				emit playbackError();
				return;
			}
		}

		emit volumeChanged( m_userVolume );
	}
}

bool PlayerBackend::isMuted() const
{
	return m_muted;
}

void PlayerBackend::toggleMuted()
{
	setMuted( ! m_muted );
}

void PlayerBackend::setMuted( bool muted )
{
	if ( m_muted != muted )
	{
		m_muted = muted;

		m_volume = m_muted ? 0 : (m_logarithmicVolume ? logarithmicVolume( m_userVolume ) : m_userVolume);

		if ( m_state == Player::Playing )
		{
			if ( ! _setVolume( m_volume ) )
			{
				resetState();
				emit playbackError();
				return;
			}
		}

		emit mutedChanged( m_muted );
	}
}

int PlayerBackend::activeAudioStream() const
{
	return m_state <= Player::Opening ? -1 : m_activeAudioStream;
}

const QStringList& PlayerBackend::audioStreams() const
{
	static QStringList emptyList;

	return m_state <= Player::Opening ? emptyList : m_audioStreams;
}

bool PlayerBackend::setActiveAudioStream( int audioStreamIndex )
{
	if ( m_state <= Player::Opening || m_audioStreams.size() <= 1 )
		return false;

	if ( m_activeAudioStream == audioStreamIndex || audioStreamIndex < 0 || audioStreamIndex >= (signed)m_audioStreams.size() )
		return false;

	m_activeAudioStream = audioStreamIndex;

	if ( m_state != Player::Ready )
	{
		bool supported = false;
		if ( ! _setActiveAudioStream( audioStreamIndex, supported ) )
		{
			resetState();
			emit playbackError();
			return true;
		}

		if ( ! supported )
		{
			if ( ! _stop() )
			{
				resetState();
				emit playbackError();
				return true;
			}
			else if ( ! _play() )
			{
				resetState();
				emit playbackError();
				return true;
			}
		}
	}

	if ( m_state == Player::Playing )
	{
		if ( ! _setVolume( m_volume ) )
		{
			resetState();
			emit playbackError();
			return true;
		}
	}

	emit activeAudioStreamChanged( audioStreamIndex );
	return true;
}

void PlayerBackend::updatePosition( double position )
{
	if ( m_state <= Player::Closed )
		return;

	if ( position > m_length && m_length > 0 )
		updateLength( position );

	if ( m_position != position )
	{
		if ( m_position <= 0 || m_minPositionDelta <= 0.0 )
		{
			m_position = position;
			emit positionChanged( position );
		}
		else
		{
			double positionDelta = m_position - position;
			if ( positionDelta >= m_minPositionDelta || -positionDelta >= m_minPositionDelta )
			{
				m_position = position;
				emit positionChanged( position );
			}
		}
	}
}

void PlayerBackend::updateLength( double length )
{
	if ( m_state <= Player::Closed )
		return;

	if ( length >= 0 && m_length != length )
	{
		m_length = length;
		emit lengthChanged( length );
	}
}

void PlayerBackend::updateFramesPerSecond( double framesPerSecond )
{
	if ( m_state <= Player::Closed )
		return;

	if ( framesPerSecond > 0 && m_framesPerSecond != framesPerSecond )
	{
		m_framesPerSecond = framesPerSecond;
		m_minPositionDelta = 1.0/framesPerSecond;
		emit framesPerSecondChanged( framesPerSecond );
	}
}

void PlayerBackend::updateAudioStreams( const QStringList& audioStreams )
{
	if ( m_state <= Player::Closed )
		return;

	m_audioStreams = audioStreams;

	emit audioStreamsChanged( m_audioStreams );

	if ( m_activeAudioStream < 0 && audioStreams.count() )
	{
		m_activeAudioStream = 0;
		emit activeAudioStreamChanged( m_activeAudioStream );
	}
}

void PlayerBackend::onOpenFileTimeout()
{
	QString filePath( m_filePath );
	_stop();
	_closeFile();

	resetState();

	emit fileOpenError( filePath );
}

void PlayerBackend::updateState( Player::State state )
{
	if ( m_state == Player::Opening )
	{
		if ( state == Player::Playing )
		{
			m_openFileTimer->stop();

			m_state = Player::Playing;
			videoWidget()->videoLayer()->show();
			_setVolume( m_volume );

			emit fileOpened( m_filePath );

			// we emit this signals in case their values were already set
			emit lengthChanged( m_length );
			emit framesPerSecondChanged( m_framesPerSecond );
			emit audioStreamsChanged( m_audioStreams );
			emit activeAudioStreamChanged( m_activeAudioStream );

			emit playing();
		}
	}
	else if ( m_state > Player::Opening )
	{
		if ( m_state != state && state > Player::Opening )
		{
			m_state = state;
			switch ( m_state )
			{
				case Player::Playing:
					videoWidget()->videoLayer()->show();
					_setVolume( m_volume );
					emit playing();
					break;
				case Player::Paused:
					emit paused();
					break;
				case Player::Ready:
					videoWidget()->videoLayer()->hide();
					emit stopped();
					break;
				default:
					break;
			}
		}
	}
}
