#ifndef PLAYERBACKEND_H
#define PLAYERBACKEND_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "player.h"
#include "videowidget.h"
#include <appconfiggroup.h>
#include <appconfiggroupwidget.h>

#include <qobject.h>
#include <qwidget.h>
#include <qstring.h>

class QTimer;

namespace SubtitleComposer
{
	class PlayerBackend : public QObject
	{
		Q_OBJECT

		friend class Player;

		public:

			// FIXME there should be a method for backends to abort on error

			VideoWidget* videoWidget();

			const AppConfigGroup& configGroup();
			void setConfigGroup( const AppConfigGroup& configGroup );

			virtual AppConfigGroupWidget* newAppConfigGroupWidget( QWidget* parent ) = 0;

			const QString& filePath() const;

			Player::State state() const;
			bool isPlaying() const;
			bool isPaused() const;
			double position() const;
			double length() const;
			double framesPerSecond() const;
			bool isStopped() const;

			double volume() const;
			bool isMuted() const;
			int activeAudioStream() const;
			const QStringList& audioStreams() const;

		public slots:

			// return values of this functions don't imply that the operation was performed OK
			// but that it was allowed (a false return value means that nothing was attempted.

			bool openFile( const QString& filePath );
			bool closeFile();

			bool play();
			bool pause();
			bool togglePlayPaused();
			bool seek( double seconds, bool accurate );
			bool stop();
			bool setActiveAudioStream( int audioStreamIndex );

			void increaseVolume( double amount );
			void decreaseVolume( double amount );
			void setVolume( double volume ); // value from 0.0 to 100.0 (inclusive)

			void toggleMuted();
			void setMuted( bool mute );

		signals:

			void initialized( PlayerBackend* playerBackend );
			void finalized( PlayerBackend* playerBackend );

			void fileOpenError( const QString& filePath );
			void fileOpened( const QString& filePath );
			void fileClosed();

			void playbackError(); // unexpected error

			void playing();
			void positionChanged( double seconds );
			void lengthChanged( double seconds );
			void framesPerSecondChanged( double fps );
			void paused();
			void stopped();
			void activeAudioStreamChanged( int audioStreamIndex );
			void audioStreamsChanged( const QStringList& audioStreams );

			void volumeChanged( double volume );
			void mutedChanged( bool muted );

		protected:

			PlayerBackend( const AppConfigGroup& configGroup );
			virtual ~PlayerBackend();

			/**
			Returns true if _initialize() has been called (but not _finalize()) and
			false otherwise
			*/
			bool isInitialized() const;

			/**
			Indicates that the application is closing down and the backends shouldn't
			rely on it for some things (such as processing events).
			*/
			bool isApplicationClosingDown() const;

			/**
			Create the videoWidget() and perform any required initialization
			*/
			virtual VideoWidget* _initialize( QWidget* videoWidgetParent ) = 0;

			/**
			Cleanup anything that has been initialized by _initialize(), excluding the
			videoWidget() which is destroyed after calling _finalize() (all references
			to it must be cleaned up, however)
			*/
			virtual void _finalize() = 0;

			/**
			Return false if there is an error and the opening of the file must be aborted,
			otherwise, return true (all internal cleanup must be done before returning).
			If the player is not leaved in a state where is about to start playing after
			the call, it must set the content of playingAfterCall to false; otherwise it's
			content must be set to true.
			The function doesn't need to block until playback is actually started
			*/
			virtual bool _openFile( const QString& filePath, bool& playingAfterCall ) = 0;

			/**
			Cleanup any internal structures associated with the opened file. This function
			is called with the player already stopped.
			videoWidget() might be NULL when this function is called.
			*/
			virtual void _closeFile() = 0;

			/**
			Return false if there is an error and playback must be aborted;	otherwise return
			true (all internal cleanup must be done before returning).
			*/
			virtual bool _play() = 0;

			/**
			Return false if there is an error and playback must be aborted;	otherwise return
			true (all internal cleanup must be done before returning).
			*/
			virtual bool _pause() = 0;

			/**
			Return false if there is an error and playback must be aborted;	otherwise return
			true (all internal cleanup must be done before returning).
			*/
			virtual bool _seek( double seconds, bool accurate ) = 0;

			/**
			Return false if there is an error and playback must be aborted;	otherwise return
			true (all internal cleanup must be done before returning).
			*/
			virtual bool _stop() = 0;

			/**
			Return false if there is an error and playback must be aborted;	otherwise return
			true (all internal cleanup must be done before returning).
			Supported must be set.
			*/
			virtual bool _setActiveAudioStream( int audioStream, bool& supported ) = 0;

			/**
			Return false if there is an error and playback must be aborted;	otherwise return
			true (all internal cleanup must be done before returning).
			*/
			virtual bool _setVolume( double volume ) = 0;

			void updatePosition( double position ); // seconds
			void updateLength( double length ); // seconds
			void updateFramesPerSecond( double framesPerSecond );
			void updateAudioStreams( const QStringList& audioStreams );
			void updateState( Player::State state );

			void setLogarithmicVolume( bool logarithmic );
			double logarithmicVolume( double percentage );

		private:

			void resetState();

			bool logarithmicVolume();

			/** return true if the player is initialized after the call */
			bool initialize( QWidget* videoWidgetParent );

			void finalize();

			/**
			return true if the player is initialized after the call.
			if the player was not initilized, the call return false imediatly.
			*/
			bool reinitialize();

			/**
			Used to indicate the backend that the application is closing down
			*/
			void setApplicationClosingDown();

		private slots:

			/** is the videowidget() gets destroyed before the player, we finalize the player */
			void onVideoWidgetDestroyed();

			/** called if the player fails to set the state to Playing after opening the file */
			void onOpenFileTimeout();

		signals:

		private:

			QString m_filePath;

			Player::State m_state;
			double m_position;
			double m_length;
			double m_framesPerSecond;
			double m_minPositionDelta;
			int m_activeAudioStream;
			QStringList m_audioStreams;

			bool m_muted;
			bool m_logarithmicVolume;
			double m_userVolume;
			double m_volume;

			VideoWidget* m_videoWidget;

			QTimer* m_openFileTimer;

			bool m_applicationClosingDown;

		protected:

			AppConfigGroup m_configGroup;
	};
}

#endif
