/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *   based on Kaffeine by Jürgen Kofler                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qapplication.h>
#include <qeventloop.h>
#include <qdir.h>
#include <qfile.h>

#include <kdebug.h>
#include <klocale.h>
#include <kurl.h>

#include "xineplayerbackend.h"
#include "xineplayerconfigdefs.h"
#include "xineplayerconfigwidget.h"
#include "xineplayervideolayerwidget.h"

#include <xine/xineutils.h>

using namespace SubtitleComposer;

#define EVENT_PLAYBACK_FINISHED		 QEvent::User + 1
#define EVENT_CHANNELS_CHANGED		 QEvent::User + 2
#define EVENT_FRAME_FORMAT_CHANGED	 QEvent::User + 3

XinePlayerBackend::XinePlayerBackend():
	PlayerBackend( defaultConfigGroup() ),
	m_connection( 0 ),
	m_xineEngine( 0 ),
	m_audioDriver( 0 ),
	m_videoDriver( 0 ),
	m_xineStream( 0 ),
	m_eventQueue( 0 ),
	m_updatePosition( true ),
	m_softwareMixer( false ),
	m_streamIsSeekable( false )
{
	connect( &m_timesTimer, SIGNAL(timeout()), this, SLOT(updatePosition()) );
}

XinePlayerBackend::~XinePlayerBackend()
{
	if ( isInitialized() )
		__finalize();
}

VideoWidget* XinePlayerBackend::_initialize( QWidget* videoWidgetParent )
{
	XinePlayerVideoLayerWidget* videoLayer = new XinePlayerVideoLayerWidget( 0 );
	VideoWidget* videoWidget = new VideoWidget( videoLayer, videoWidgetParent );
	if ( ! initializeXine( videoWidget->videoLayer()->winId() ) )
	{
		delete videoWidget;
		finalizeXine();
		kdError() << "xine initialization failed!" << endl;
		return 0;
	}

	videoLayer->setVideoDriver( m_videoDriver );
	connect( videoLayer, SIGNAL(geometryChanged()), this, SLOT(onVideoLayerGeometryChanged()) );

	return videoWidget;
}

void XinePlayerBackend::_finalize()
{
	return __finalize();
}

void XinePlayerBackend::__finalize()
{
	m_timesTimer.stop();

	finalizeXine();
}

AppConfigGroup XinePlayerBackend::defaultConfigGroup()
{
	QMap<QString,QString> defaults;

// 	defaults[OPT_AUDIO_DRIVER] = "auto";
	defaults[OPT_AUDIO_DRIVER] = QString::null;
// 	defaults[OPT_VIDEO_DRIVER] = "auto";
	defaults[OPT_VIDEO_DRIVER] = QString::null;

	return AppConfigGroup( "Xine", defaults );
}

SubtitleComposer::AppConfigGroupWidget* XinePlayerBackend::newAppConfigGroupWidget( QWidget* parent )
{
	return new XinePlayerConfigWidget( m_configGroup, parent );
}

bool XinePlayerBackend::_openFile( const QString& filePath, bool& playingAfterCall )
{
	playingAfterCall = true;

	// the volume is adjusted when file playback starts and it's best if it's initially at 0
	xine_set_param( m_xineStream, m_softwareMixer ? XINE_PARAM_AUDIO_AMP_LEVEL : XINE_PARAM_AUDIO_VOLUME, 0 );

	m_streamIsSeekable = false;

	KURL fileURL;
	fileURL.setProtocol( "file" );
	fileURL.setPath( filePath );

	if ( ! xine_open( m_xineStream, fileURL.url().local8Bit() ) )
		return false;

	// no subtitles
	xine_set_param( m_xineStream, XINE_PARAM_SPU_CHANNEL, -1 );

	if ( ! xine_play( m_xineStream, 0, 0 ) )
		return false;

	updateState( Player::Playing );

	// this methods do nothing if the information is not available
	updateVideoData();
	updateAudioData();
	updatePosition();

	m_timesTimer.start( 20 );

	return true;
}

void XinePlayerBackend::_closeFile()
{
}

bool XinePlayerBackend::_play()
{
	if ( xine_get_status( m_xineStream ) != XINE_STATUS_PLAY )
		xine_play( m_xineStream, 0, 0 ); // was stopped
	else
		xine_set_param( m_xineStream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL ); // was paused

	updateState( Player::Playing );

	m_timesTimer.start( 20 );

	return true;
}

bool XinePlayerBackend::_pause()
{
	m_timesTimer.stop();

	xine_set_param( m_xineStream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE );
	updateState( Player::Paused );

	return true;
}

bool XinePlayerBackend::_seek( double seconds, bool accurate )
{
// 	QTime time;
// 	time.start();

	if ( m_streamIsSeekable )
	{
		int targetTime = (int)(seconds*1000+0.5);

		if ( isPaused() )
			xine_set_param( m_xineStream, m_softwareMixer ? XINE_PARAM_AUDIO_AMP_MUTE : XINE_PARAM_AUDIO_MUTE, 1 );

		xine_play( m_xineStream, 0, targetTime );

		if ( accurate )
		{
			int time;

			if ( ! xine_get_pos_length( m_xineStream, 0, &time, 0 ) )
				return true;

			m_updatePosition = false;

			xine_set_param( m_xineStream, XINE_PARAM_SPEED, /*XINE_SPEED_FAST_4*/64 );

			while ( targetTime - time > 200 /*|| time - targetTime > 100 ||*/ )
			{
				QApplication::eventLoop()->processEvents( QEventLoop::ExcludeUserInput );

				if ( ! xine_get_pos_length( m_xineStream, 0, &time, 0 ) )
					break;
			}

			if ( ! isPaused() )
				xine_set_param( m_xineStream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL );

			m_updatePosition = true;
		}

		if ( isPaused() )
		{
			xine_set_param( m_xineStream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE );
			xine_set_param( m_xineStream, m_softwareMixer ? XINE_PARAM_AUDIO_AMP_MUTE : XINE_PARAM_AUDIO_MUTE, 0 );
		}
	}

// 	kdDebug() << time.elapsed() << endl;

	return true;
}

bool XinePlayerBackend::_stop()
{
	m_timesTimer.stop();

	xine_stop( m_xineStream );
	updateState( Player::Ready );

	return true;
}

bool XinePlayerBackend::_setActiveAudioStream( int audioStream, bool& supported )
{
	supported = true;
	xine_set_param( m_xineStream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, audioStream );

	return true;
}

bool XinePlayerBackend::_setVolume( double volume )
{
	xine_set_param( m_xineStream, m_softwareMixer ? XINE_PARAM_AUDIO_AMP_LEVEL : XINE_PARAM_AUDIO_VOLUME, (int)(volume+0.5) );

	return true;
}


void XinePlayerBackend::updateVideoData()
{
	if ( ! m_xineStream )
		return;

	if ( xine_get_stream_info( m_xineStream, XINE_STREAM_INFO_HAS_VIDEO ) )
	{
		int fps = xine_get_stream_info( m_xineStream, XINE_STREAM_INFO_FRAME_DURATION );
		if ( fps > 0 )
		{
			updateFramesPerSecond( 90000.0 / fps );

			// tweak prebuffer so we can be sure to show only a single frame
// 			kdDebug() << "PREBUFFER " << xine_get_param( m_xineStream, XINE_PARAM_METRONOM_PREBUFFER ) << endl;
			xine_set_param( m_xineStream, XINE_PARAM_METRONOM_PREBUFFER, fps );
// 			kdDebug() << "PREBUFFER " << xine_get_param( m_xineStream, XINE_PARAM_METRONOM_PREBUFFER ) << endl;
		}

		int width = xine_get_stream_info( m_xineStream, XINE_STREAM_INFO_VIDEO_WIDTH );
		int height = xine_get_stream_info( m_xineStream, XINE_STREAM_INFO_VIDEO_HEIGHT );
		double dar = xine_get_stream_info( m_xineStream, XINE_STREAM_INFO_VIDEO_RATIO ) / 10000.0;
		videoWidget()->setVideoResolution( width, height, dar );
	}
	else
		videoWidget()->setVideoResolution( 0, 0 );

	m_streamIsSeekable = xine_get_stream_info( m_xineStream, XINE_STREAM_INFO_SEEKABLE );

	QSize size = videoWidget()->videoLayer()->size();
	QPoint globalPos = videoWidget()->videoLayer()->mapToGlobal( QPoint( 0, 0 ) );
	m_videoLayerGeometry = QRect( globalPos.x(), globalPos.y(), size.width(), size.height() );
}

void XinePlayerBackend::updateAudioData()
{
	if ( ! m_xineStream )
		return;

	QStringList audioStreams;

	int channels = xine_get_stream_info( m_xineStream, XINE_STREAM_INFO_MAX_AUDIO_CHANNEL );
	for( int index = 0; index < channels; ++index )
	{
		QString audioStream = QString( "Audio Stream #%1" ).arg( index + 1 );
		char lang[128];
		if ( xine_get_audio_lang( m_xineStream, index, lang ) )
			audioStream += QString( " - " ) + lang;
		audioStreams << audioStream;
	}

	updateAudioStreams( audioStreams );
}

void XinePlayerBackend::updatePosition()
{
	if ( ! m_xineStream || ! m_updatePosition )
		return;

	int time, length;
	if ( xine_get_pos_length( m_xineStream, 0, &time, &length ) )
	{
		PlayerBackend::updatePosition( time / 1000.0 );
		PlayerBackend::updateLength( length / 1000.0 );

		updateState( Player::Playing );
	}
}

void XinePlayerBackend::onVideoLayerGeometryChanged()
{
	QSize size = videoWidget()->videoLayer()->size();
	QPoint globalPos = videoWidget()->videoLayer()->mapToGlobal( QPoint( 0, 0 ) );
	m_videoLayerGeometry = QRect( globalPos.x(), globalPos.y(), size.width(), size.height() );
}

void XinePlayerBackend::xineEventListener( void* p, const xine_event_t* event )
{
	if ( p == NULL )
		return;

	XinePlayerBackend* xinePlayer = (XinePlayerBackend*)p;

	switch ( event->type )
	{
		case XINE_EVENT_UI_PLAYBACK_FINISHED:
			QApplication::postEvent( xinePlayer, new QCustomEvent( EVENT_PLAYBACK_FINISHED ) );
			break;

		case XINE_EVENT_UI_CHANNELS_CHANGED:
			QApplication::postEvent( xinePlayer, new QCustomEvent( EVENT_CHANNELS_CHANGED ) );
			break;

		case XINE_EVENT_FRAME_FORMAT_CHANGE:
			QApplication::postEvent( xinePlayer, new QCustomEvent( EVENT_FRAME_FORMAT_CHANGED ) );
			break;

		default:
			break;
	}
}

void XinePlayerBackend::customEvent( QCustomEvent* event )
{
	switch ( event->type() )
	{
		case EVENT_PLAYBACK_FINISHED:
			_stop();
			break;

		case EVENT_CHANNELS_CHANGED:
			updateAudioData();
			break;

		case EVENT_FRAME_FORMAT_CHANGED:
			updateVideoData();
			break;

		default:
			break;
	}
}

bool XinePlayerBackend::initializeXine( WId winId )
{

#ifdef HAVE_XCB
	int screen_nbr = 0;
	m_connection = xcb_connect( NULL, &screen_nbr );
#else
	XInitThreads();
	m_connection = XOpenDisplay( NULL );
#endif

	if ( ! m_connection )
	{
		kdDebug() << "Failed to connect to X-Server!" << endl;
		return false;
	}

	if ( ! (m_xineEngine = xine_new()) )
	{
		kdDebug() << "Can't init xine Engine!" << endl;
		return false;
	}

	QString configFilePath( QDir::homeDirPath() + "/.xine/config" );
	if ( QFile::exists( configFilePath ) )
		xine_config_load( m_xineEngine, QFile::encodeName( configFilePath ) );

	xine_init( m_xineEngine );

	m_softwareMixer = (bool)xine_config_register_bool(
		m_xineEngine,
		"audio.mixer_software",
		1, NULL, NULL, 10,
		&XinePlayerBackend::audioMixerMethodChangedCallback,
		this
	);

#ifdef HAVE_XCB
	xcb_screen_iterator_t screen_it = xcb_setup_roots_iterator( xcb_get_setup( m_connection ) );
	while ( screen_it.rem > 1 && screen_nbr > 0 )
	{
		xcb_screen_next(&screen_it);
		--screen_nbr;
	}
	m_x11Visual.connection = m_connection;
	m_x11Visual.screen = screen_it.data;
	m_x11Visual.window = winId;
#else
	m_x11Visual.display = m_connection;
	m_x11Visual.screen = DefaultScreen( m_connection );
	m_x11Visual.d = winId;
#endif
// 	m_x11Visual.dest_size_cb = &XinePlayerBackend::destSizeCallback;
	m_x11Visual.frame_output_cb = &XinePlayerBackend::frameOutputCallback;
	m_x11Visual.user_data = (void*)this;


	QStringList videoDriverNames = QStringList::split( ' ', "xv xvmc opengl xxmc sdl xshm fb XDirectFB DirectFB aa caca auto" );
	videoDriverNames.prepend( m_configGroup.option( OPT_VIDEO_DRIVER ) );
	for ( QStringList::ConstIterator it = videoDriverNames.begin(), end = videoDriverNames.end(); it != end; ++it )
	{
		if ( (*it).isEmpty() )
			continue;

		m_videoDriver = xine_open_video_driver(
			m_xineEngine,
			(*it).ascii(),
#ifdef HAVE_XCB
			XINE_VISUAL_TYPE_XCB,
#else
			XINE_VISUAL_TYPE_X11,
#endif
			(void*)&(m_x11Visual)
		);

		if ( m_videoDriver )
			break;
	}

	if ( ! m_videoDriver )
	{
		kdDebug() << "All video drivers failed to initialize!" << endl;
		return false;
	}


	QStringList audioDriverNames = QStringList::split( ' ', "alsa oss jack pulseaudio esd auto" );
	audioDriverNames.prepend( m_configGroup.option( OPT_AUDIO_DRIVER ) );
	for ( QStringList::ConstIterator it = audioDriverNames.begin(), end = audioDriverNames.end(); it != end; ++it )
		if ( ! (*it).isEmpty() && (m_audioDriver = xine_open_audio_driver( m_xineEngine, (*it).ascii(), NULL )) != NULL )
			break;

	if ( ! m_audioDriver )
	{
		kdDebug() << "All audio drivers failed to initialize!" << endl;
		return false;
	}

	m_xineStream = xine_stream_new(
		m_xineEngine,
		m_audioDriver,
		m_videoDriver
	);

	if ( ! m_xineStream )
	{
		kdDebug() << "Can't create a new xine Stream!" << endl;
		return false;
	}

	m_eventQueue = xine_event_new_queue( m_xineStream );
	xine_event_create_listener_thread( m_eventQueue, &XinePlayerBackend::xineEventListener, (void*)this );

	return true;
}

void XinePlayerBackend::finalizeXine()
{
	if ( m_xineStream )
		xine_close( m_xineStream );

	if ( m_eventQueue )
	{
		xine_event_dispose_queue( m_eventQueue );
		m_eventQueue = 0;
	}

	if ( m_xineStream )
	{
		xine_dispose( m_xineStream );
		m_xineStream = 0;
	}

	if ( m_audioDriver )
	{
		xine_close_audio_driver( m_xineEngine, m_audioDriver );
		m_audioDriver = 0;
	}

	if ( m_videoDriver )
	{
		xine_close_video_driver( m_xineEngine, m_videoDriver );
		m_videoDriver = 0;
	}

	if ( m_xineEngine )
	{
		xine_exit( m_xineEngine );
		m_xineEngine = 0;
	}

	if ( m_connection )
	{
#ifdef HAVE_XCB
		xcb_disconnect( m_connection );
#else
		XCloseDisplay( m_connection );
#endif
		m_connection = NULL;
	}
}

void XinePlayerBackend::destSizeCallback(	void* p, int /*video_width*/, int /*video_height*/, double /*video_aspect*/,
									int* dest_width, int* dest_height, double* dest_aspect )
{
	if ( p == NULL )
		return;

	XinePlayerBackend* xinePlayer = (XinePlayerBackend*)p;
	*dest_width = xinePlayer->m_videoLayerGeometry.width();
	*dest_height = xinePlayer->m_videoLayerGeometry.height();
	*dest_aspect = 1.0;
}

void XinePlayerBackend::frameOutputCallback(	void* p, int /*video_width*/, int /*video_height*/, double /*video_aspect*/,
										int* dest_x, int* dest_y, int* dest_width, int* dest_height,
										double* dest_aspect, int* win_x, int* win_y )
{
	if ( p == NULL )
		 return;

	XinePlayerBackend* xinePlayer = (XinePlayerBackend*)p;

	*dest_x = 0;
	*dest_y = 0 ;
	*dest_width = xinePlayer->m_videoLayerGeometry.width();
	*dest_height = xinePlayer->m_videoLayerGeometry.height();
	*dest_aspect = 1.0;
	*win_x = xinePlayer->m_videoLayerGeometry.x();
	*win_y = xinePlayer->m_videoLayerGeometry.y();
}

void XinePlayerBackend::audioMixerMethodChangedCallback( void* p, xine_cfg_entry_t* entry )
{
	if ( p == NULL )
		return;

	XinePlayerBackend* xinePlayer = (XinePlayerBackend*)p;
	xinePlayer->m_softwareMixer = (bool)entry->num_value;
}
