/**************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "attachablewidget.h"

#include <qlayout.h>

#include <kdebug.h>

#define MOUSE_IGNORE_MSECS 500

AttachableWidget::AttachableWidget( bool animated, unsigned stepDuration ):
	QWidget( 0 ),
	m_widget( 0 ),
	m_animated( animated ),
	m_hiding( true ),
	m_animStepDuration( stepDuration )
{
	hide();
}

AttachableWidget::~AttachableWidget()
{
}

bool AttachableWidget::isAnimated() const
{
	return m_animated;
}

void AttachableWidget::setAnimated( bool animated )
{
	m_animated = animated;
}

unsigned AttachableWidget::animStepDuration() const
{
	return m_animStepDuration;
}

void AttachableWidget::setAnimStepDuration( unsigned msecs )
{
	m_animStepDuration = msecs;
}


void AttachableWidget::attach( QWidget* widget, Place place )
{
	if ( m_widget )
	{
		kdWarning() << "attempted to attach to another widget while already attached" << endl;
		return;
	}

	m_widget = widget;
	m_place = place;

	m_ignoreMouse = true;
	m_ignoreMouseTID = startTimer( MOUSE_IGNORE_MSECS );

	reparent( m_widget, 0, QPoint( 0, 0 ), false );
	hide();

	m_widget->installEventFilter( this );
	m_widget->setMouseTracking( true );
}

void AttachableWidget::dettach()
{
	if ( ! m_widget )
	{
		kdWarning() << "attempted to dettach while not attached to any widget" << endl;
		return;
	}

	m_widget->removeEventFilter( this );
	m_widget->setMouseTracking( false );

	m_widget = 0;

	killTimer( m_ignoreMouseTID );

	reparent( 0, 0, QPoint( 0, 0 ), false );
	hide();
}

// cuando se reinvierte la animacion, no se debe recomenzar desde 0
// sino desde donde esta
void AttachableWidget::toggleVisible( bool visible )
{
	if ( visible && m_hiding )
	{
		m_hiding = false;
		killTimer( m_animTID );

		bool wasShown = isShown();

		show();

		int width = m_widget->width();
		int height = this->height();
		int y = m_place == Top ? 0 : m_widget->height() - height;

		resize( width, height );

		if ( m_animated )
		{
			if ( m_place == Top )
			{
				m_animDirection = Downward;
				m_animCurrentY = - height;
			}
			else
			{
				m_animDirection = Upward;
				m_animCurrentY = m_widget->height();
			}

			m_animFinalY = y;

			if ( wasShown )
				m_animCurrentY = this->y();
			else
				move( 0, m_animCurrentY );

			m_animTID = startTimer( m_animStepDuration );
		}
		else
			move( 0, y );
	}
	else if ( ! visible && ! m_hiding )
	{
		m_hiding = true;
		killTimer( m_animTID );

		if ( m_animated )
		{
			if ( m_place == Top )
			{
				m_animDirection = Upward;
				m_animFinalY = - height();
			}
			else
			{
				m_animDirection = Downward;
				m_animFinalY = m_widget->height();
			}

			m_animCurrentY = y();

			m_animTID = startTimer( m_animStepDuration );
		}
		else
			hide();
	}
}

void AttachableWidget::timerEvent( QTimerEvent* event )
{
	if ( event->timerId() == m_animTID ) // Advance animation
	{
		if ( m_animCurrentY == m_animFinalY )
		{
			killTimer( m_animTID );

			if ( (m_animDirection == Upward && m_place == Top) ||
				 (m_animDirection == Downward && m_place == Bottom) )
				hide();
		}
		else
		{
			if ( m_animDirection == Upward )
				m_animCurrentY--;
			else
				m_animCurrentY++;

			move( x(), m_animCurrentY );
		}
	}
	else if ( event->timerId() == m_ignoreMouseTID )
	{
		killTimer( m_ignoreMouseTID );
		m_ignoreMouse = false;
	}
}

bool AttachableWidget::eventFilter( QObject* object, QEvent* event )
{
	if ( ! m_ignoreMouse )
	{
		if ( event->type() == QEvent::MouseMove )
		{
			QMouseEvent* e = (QMouseEvent*)event;

			const int margin = height() + 10;

			QPoint pos = e->pos();

			if ( m_place == Bottom )
			{
				if ( pos.y() > m_widget->height() - margin )
					toggleVisible( true );
				else
					toggleVisible( false );
			}
			else
			{
				if ( pos.y() < margin )
					toggleVisible( true );
				else
					toggleVisible( false );
			}
		}
	}

	return QWidget::eventFilter( object, event );
}

#include "attachablewidget.moc"
