/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "textedit.h"

#include <kdebug.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kspell.h>
#include <kaccel.h>

#include <qpopupmenu.h>

TextEdit::TextEdit( QWidget* parent ):
	KTextEdit( parent, 0 ),
	m_spell( 0 )
{
	setTextFormat( Qt::PlainText );
	setPointSize( 10 );
}

TextEdit::~TextEdit()
{
}

void TextEdit::setSelection( int startIndex, int endIndex )
{
	selectAll( false );

	if ( startIndex > endIndex )
		return;

	int startPar = -1, endPar = -1;
	int parStartIndex = 0, parEndIndex;

	const int parCount = this->paragraphs();
	for ( int par = 0; par < parCount; ++par, parStartIndex = parEndIndex + 1 )
	{
		parEndIndex = parStartIndex + paragraphLength( par );

		if ( startPar < 0 && startIndex <= parEndIndex )
		{
			startIndex -= parStartIndex;
			startPar = par;
		}

		if ( endPar < 0 && endIndex <= parEndIndex )
		{
			endIndex -= parStartIndex;
			endPar = par;
		}

		if ( startPar >= 0 && endPar >= 0 )
			break;
	}

	QTextEdit::setSelection( startPar, startIndex, endPar, endIndex + 1 );
}

void TextEdit::setText( const QString& text )
{
	int idx1 = 0, idx2;
	while ( (idx2 = text.find( '\n', idx1 )) != -1 )
	{
		setAlignment( Qt::AlignCenter );
		append( text.mid( idx1, idx2-idx1 ) );
		idx1 = idx2 + 1;
	}
	setAlignment( Qt::AlignCenter  );
	append( text.mid( idx1 ) );
}

/// next methods are (mostly) taken shamelessly from KTextEdit
/// we disable the auto checking of spelling since it doesn't play nice with our formatting
/// and reimplement the spelling related stuff (same reason)

QPopupMenu* TextEdit::createPopupMenu( const QPoint& pos )
{
	enum { IdUndo, IdRedo, IdSep1, IdCut, IdCopy, IdPaste, IdClear, IdSep2, IdSelectAll };

	QPopupMenu* menu = QTextEdit::createPopupMenu( pos );

	if ( isReadOnly() )
		menu->changeItem( menu->idAt(0), SmallIconSet( "editcopy" ), menu->text( menu->idAt(0) ) );
	else
	{
		int id = menu->idAt( 0 );

		if ( isUndoRedoEnabled() )
		{
			menu->changeItem( id - IdUndo, SmallIconSet( "undo" ), menu->text( id - IdUndo) );
			menu->changeItem( id - IdRedo, SmallIconSet( "redo" ), menu->text( id - IdRedo) );
		}
		else
		{
			menu->removeItem( id - IdUndo );
			menu->removeItem( id - IdRedo );
			menu->removeItem( id - IdSep1 );
		}

		menu->changeItem( id - IdCut, SmallIconSet( "editcut" ), menu->text( id - IdCut) );
		menu->changeItem( id - IdCopy, SmallIconSet( "editcopy" ), menu->text( id - IdCopy) );
		menu->changeItem( id - IdPaste, SmallIconSet( "editpaste" ), menu->text( id - IdPaste) );
		menu->changeItem( id - IdClear, SmallIconSet( "editclear" ), menu->text( id - IdClear) );

		if ( checkSpellingEnabled() )
		{
			menu->insertSeparator();
			id = menu->insertItem( SmallIconSet( "spellcheck" ), i18n( "Check Spelling..." ), this, SLOT( checkSpelling() ) );
			if( text().isEmpty() )
				menu->setItemEnabled( id, false );
		}

		menu->insertSeparator();
		id = menu->insertItem( i18n( "Allow Tabulations" ), this, SLOT(toggleTabChangesFocus()) );
		menu->setItemChecked( id, ! tabChangesFocus() );
	}

	return menu;
}

void TextEdit::toggleTabChangesFocus()
{
	setTabChangesFocus( ! tabChangesFocus() );
}

void TextEdit::checkSpelling()
{
	delete m_spell;
	m_spell = new KSpell( this, i18n( "Spell Checking" ), this, SLOT(onSpellCheckReady()), 0, true, true );

	connect( m_spell, SIGNAL(death()), this, SLOT(onSpellCheckCleanup()) );

	connect( m_spell, SIGNAL(done(const QString&)), this, SLOT(onSpellCheckDone()) );

	connect( m_spell, SIGNAL(misspelling(const QString&,const QStringList&,unsigned int)),
			 this, SLOT(onWordMisspelled(const QString&,const QStringList&,unsigned int)) );

	connect( m_spell, SIGNAL(corrected(const QString&,const QString&,unsigned int)),
			 this, SLOT(onWordCorrected(const QString&,const QString&,unsigned int)) );
}

void TextEdit::onSpellCheckReady()
{
	m_spell->check( text() );
}

void TextEdit::onWordMisspelled( const QString& text, const QStringList&, unsigned int pos )
{
	setSelection( pos, pos + text.length() - 1 );
}

void TextEdit::onWordCorrected( const QString& oldWord, const QString& newWord, unsigned int pos )
{
	if ( oldWord != newWord )
	{
		setSelection( pos, pos + oldWord.length() - 1 );
		removeSelectedText();
		insert( newWord );
	}
}

void TextEdit::onSpellCheckDone()
{
	m_spell->cleanUp();
}

void TextEdit::onSpellCheckCleanup()
{
	delete m_spell;
	m_spell = 0L;
}

#include "textedit.moc"
